//
//  AppDelegate.m
//  TMFDemo
//
//  Created by klaudz on 5/4/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "AppDelegate.h"
#import "TMFConfigurations.h"
#import "TMFDemoConfigurations.h"

#import "TMFBase.h"

// TMFShark
#import "TMFSharkCenterConfiguration.h"
#import "TMFSharkCenter+Profile.h"
#import "TMFSharkCenter.h"
#import "TMFSharkDemoUserConfiguration.h"

// CloudInstruction
#import "TMFInstructionCenter.h"
#import "TMFInstructionCenter+Iteration.h"
#import "TMFPushGo.h"

// WCDB
#import "TMFWCDBSampleUsage.h"

// QMUIKit
#import <QMUIKit/QMUIKit.h>
#import "QDCommonUI.h"
#import "QDUIHelper.h"
#import "QDThemeManager.h"

// TMFJSBridge
#import "TMFJSBridgeConfiguration.h"
#import "TMFJSAPIs.h"
#import "TMFJSAPIs_Custom.h"
#import "TMFJSBridgeWKWebViewController.h"

// TMFWebOffline
#import "TMFWebOfflineService.h"

// TMFPush
#import "TMFPush.h"
#import "TMFPush+Logger.h"
#import "TMFDataPush.h"

// QAPM
#import <QAPM/QAPM.h>
#import <QAPM/QAPMModelStableConfig.h>
#import <QAPM/QAPMFoomProfile.h>

#import "QAPMCoreDataManager.h"
// TMFShare
#import "TMFShareManager.h"

// SDWebImage
#import <SDWebImage/SDWebImage.h>

// Map
//  TencentMap
#import <QMapKit/QMapKit.h>
//  Amap
#import <AMapFoundationKit/AMapFoundationKit.h>
#import <MAMapKit/MAMapKit.h>
#import <AMapSearchKit/AMapSearchKit.h>
// Map Demo
#import "AMapViewController.h"

// YTDetector
#import "TMFLiveDetectionConfiguration.h"

// Patch
#import "TMFChrist.h"
#import "TMFChristViewController.h"

// Distribution
#import "TMFDistribution.h"

// Profile
#import "TMFProfile.h"

// XLog
#import "TMFXLogger.h"


// TMFSafeKeyboard
#import "TMFSafeKeyboardConfigApi.h"
#import "TMFSKGetKeyPositionNetHelper.h"
#import "SKGetKeyDataImpl.h"
//  ICDP
#import "TMFICDPSettings.h"

#import "TMFRouter.h"
#import "TMFAnalyse.h"

@interface AppDelegate () <TMFPushDelegate, TMFAppDelegate>
@property (nonatomic, strong)NSMutableDictionary *configValue;
@property (nonatomic, copy)NSString *configName;
@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    [self prepareEverything];
    return YES;
}


- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
}


- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}


- (void)applicationWillEnterForeground:(UIApplication *)application {
    // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
}


- (void)applicationDidBecomeActive:(UIApplication *)application {
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    
    // 自动检查和更新离线包
    [self checkUpdateWebOfflineAutomatically];
}


- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    
    [[TMFXLogger defaultLogger] close];
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-implementations"
- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(nullable NSString *)sourceApplication annotation:(id)annotation {
#pragma clang diagnostic pop
    
    // iOS9.0- 监听分享App的拉起处理
    return [self handleShareWithOpenURL:url];
}

- (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options {
    
    // iOS9.0+ 监听分享App的拉起处理
    return [self handleShareWithOpenURL:url];
}

- (BOOL)application:(UIApplication *)application continueUserActivity:(NSUserActivity *)userActivity restorationHandler:(void(^)(NSArray<id<UIUserActivityRestoring>> * __nullable restorableObjects))restorationHandler
{
    return [[TMFShareManager shareManager] handleOpenUniversalLink:userActivity];
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmismatched-parameter-types"
- (void)application:(UIApplication *)application handleActionWithIdentifier:(NSString *)identifier forRemoteNotification:(NSDictionary *)userInfo completionHandler:(nonnull void (^)(void))completionHandler
#pragma clang diagnostic pop
{
    NSLog(@"[TMFPush] did handle action for remote notification, userInfo: %@", userInfo);
    for (NSString *key in userInfo) {
        id value = userInfo[key];
        NSLog(@"[TMFPush] key: %@, value: %@", key, value);
    }
    
    completionHandler();
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmismatched-parameter-types"
- (void)application:(UIApplication *)application handleActionWithIdentifier:(NSString *)identifier forRemoteNotification:(NSDictionary *)userInfo withResponseInfo:(NSDictionary *)responseInfo completionHandler:(nonnull void (^)(void))completionHandler
#pragma clang diagnostic pop
{
    NSLog(@"[TMFPush] did handle action for remote notification, userInfo: %@, responseInfo: %@", userInfo, responseInfo);
    for (NSString *key in userInfo) {
        id value = userInfo[key];
        NSLog(@"[TMFPush] key: %@, value: %@", key, value);
    }
    
    completionHandler();
}

#pragma mark - TMFRouter 组件化相关代理
- (void)tmf_application:(UIApplication *)application beforeDidFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    NSLog(@"tmf_application:beforeDidFinishLaunchingWithOptions:");
}

- (TMFAppDelegateCallbackResult)tmf_application:(UIApplication *)application handleDidFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    NSLog(@"tmf_application:handleDidFinishLaunchingWithOptions:");
    [self prepareEverything];
    return TMFAppDelegateCallbackResultContinue;
}

- (void)tmf_application:(UIApplication *)application afterDidFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    NSLog(@"tmf_application:afterDidFinishLaunchingWithOptions:");
}

- (BOOL)tmf_application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options {
    return YES;
}

- (void)tmf_applicationWillResignActive:(UIApplication *)application {
    
}

- (void)tmf_applicationDidEnterBackground:(UIApplication *)application {
    
}

- (void)tmf_applicationWillEnterForeground:(UIApplication *)application {
    
}

- (void)tmf_applicationDidBecomeActive:(UIApplication *)application {
    
}

- (void)tmf_applicationWillTerminate:(UIApplication *)application {
    
}

- (void)tmf_applicationDidReceiveMemoryWarning:(UIApplication *)application {
    
}

- (void)prepareEverything {
    
    [self prepareBase];
    
    [self prepareProfile];
    [self preparePushGo];
    [self prepareDistribution];
    [self prepareWebOffline];
    [self preparePatch];
    [self prepareJSBridge];

    [self prepareQMUI];

    [self prepareWCDB];
    
    [self prepareQAPM];
    [self prepareShare];
    [self preparePush];
    [self prepareDataPush];

    [self prepareSDWebImage];
    [self prepareMap];
    [self prepareLiveDetection];
    [self prepareSafeKeyboard];
    [self prepareICDP];
    [self prepareRouter];
    [self prepareAnalyse];
}


#pragma mark - Base
#define USE_JSON_CONFIG 1
- (void)prepareBase{
    [TMFBaseCore setAllowBaseCoreLogFilter:NO];
    
    [TMFSharkCenterConfiguration setLogLevels:1];
    [TMFXLogger setDebugLogLevels:TMFXLogDebugLevelAll];
    [TMFInstructionCenter setLogLevels:TMFBaseCoreLogLevelAll];
    [TMFInstructionCenter setApiVersion:TMFInstructionCenterApiVersionPushGo];
    [TMFServerManager setLogLevels:TMFBaseCoreLogLevelAll];
    
#if USE_JSON_CONFIG
    NSString *sanboxConfigPath = [TMFSharkEnvironmentConfiguration configFilePath];
    if ([[NSFileManager defaultManager] fileExistsAtPath:sanboxConfigPath]) {
        NSLog(@"‼️使用沙盒 Document\Environment 目录下的配置文件");
        [[TMFServerManager shareServerManagerWithConfigPath:sanboxConfigPath] activate];
    } else {
        NSLog(@"‼️使用 Project 的配置文件");
        [TMFServerManager start];
    }
#else
    //如果没有json格式的配置文件，可以使用TMFSharkCenterConfiguration来进行初始化
    TMFSharkCenterConfiguration *sharkConfig = [TMFSharkCenterConfiguration masterConfiguration];
    sharkConfig.customerID = TMF_CUSTOM_ID;
    sharkConfig.productID = TMF_PID;
    TMFServerConfiguration *serverConfig = [TMFServerConfiguration serverConfigurationWithSharkConfiguration:sharkConfig];
    TMFServerManager *serverManager = [TMFServerManager shareServerManagerWithConfiguration:serverConfig];
    sharkConfig.HTTPURL = TMF_GW_HTTP_URL;
    sharkConfig.productKey = TMF_APP_KEY;
    sharkConfig.RSAPublicKey = TMF_GW_RKEY;
    sharkConfig.SM2PublicKey = TMF_GW_SKEY;
    sharkConfig.enforceCheckMasterEvent = NO;       // 迁移旧的shark数据，不重新生成GUID
     // 如果离线包、染色日志的证书文件从ServerManager中获取，需要在离线包启动之前进行设置
    [serverManager activate];
#endif
    [self prepareXLog];
    [[TMFPushGoCenter shareCenter] performSelectorOnMainThread:@selector(activate) withObject:nil waitUntilDone:NO];
    
    NSLog(@"GUID: %@", [TMFSharkCenter masterCenter].GUID);
    NSLog(@"%@", [TMFServerManager shareServerManager].workSpace);
}

#pragma mark - Patch

- (void)preparePatch
{
    [TMFChristViewController copyFileToLibrary];
    [TMFChristManager setLogLevels:TMFBaseCoreLogLevelAll];
    TMFChristManager *patchManager = [TMFChristManager sharedManager];
    
    patchManager.initJesusBlock = ^BOOL{
        return YES;
    };
    patchManager.jesusEvaluateChristianBlock = ^BOOL(NSString * _Nonnull christianPath) {
        return YES;
    };
    NSString *publicKeyPath = [[NSBundle mainBundle] pathForResource:@"patch_public_key" ofType:@"der"];
    patchManager.publicKey = [NSData dataWithContentsOfFile:publicKeyPath];
    
    [patchManager evaluateLocalChristians];
//    [patchManager evaluateChristiansWithScript:[[NSString alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"main.js" ofType:nil] encoding:NSUTF8StringEncoding error:nil]];
    [patchManager checkForUpdates];
    NSLog(@"TMFChristManager.path = %@",[patchManager resourceDirectory]);
}

#pragma mark - QMUIKit

- (void)prepareQMUI {
    // QMUIConsole 默认只在 DEBUG 下会显示，作为 Demo，改为不管什么环境都允许显示
    [QMUIConsole sharedInstance].canShow = YES;
    
    // QD自定义的全局样式渲染
    [QDCommonUI renderGlobalAppearances];
    
    // 预加载 QQ 表情，避免第一次使用时卡顿
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [QDUIHelper qmuiEmotions];
    });
}

#define swap_int(_a, _b) \
int _t = _a;         \
_a = _b;             \
_b = _t;

// 对要上报的数据进行处理，比如自定义的压缩和加密实现。如果开启默认压缩GZip，则会先GZip，再调用此接口
- (NSData *)handleToReportData:(NSData *)inData {
    [self ChiperRC4:inData];
    return inData;
}

// https:处理返回的二进制数据。返回值true代表正确收到。
-(BOOL)handleHttpReturnData:(NSData *)returnData {
    NSString *returnString = [[NSString alloc] initWithData:returnData
                                                   encoding:NSUTF8StringEncoding];
    NSLog(@"[vip]return string is %@",returnString);
    return [self checkReturnData:returnData];
}

- (void)ChiperRC4:(NSData *)input {
    uint32_t sbox_size = 256;
    unsigned char sbox[sbox_size];
    const char *_key = "3c050fb7af76511e27f268083a9cbe3a";
    unsigned char *key = (unsigned char *)_key;
    uint32_t keysize = (uint32_t)strlen(_key);
    
    // Sbox Initilisieren
    unsigned char j = 0;
    uint32_t i = 0;
    for (i = 0; i < sbox_size; i++)
        sbox[i] = i;
    // Sbox randomisieren
    for (i = 0; i < sbox_size; i++) {
        j += sbox[i] + key[i % keysize];
        swap_int(sbox[i], sbox[j]);
    }
    
    j = 0;
    i = 0;
    int n = 0;
    
    unsigned char *content = (unsigned char *)[input bytes];
    for (n = 0; n < [input length]; n++) {
        i++;
        uint32_t index = i % sbox_size;
        j += sbox[index];
        swap_int(sbox[index], sbox[j]);
        content[n] = content[n] ^ (sbox[(sbox[index] + sbox[j]) & 0xFF]);
    }
}

- (BOOL) checkReturnData:(NSData *)returnData {
    NSLog(@"[vip]++++++++check data");
    return TRUE;
}

#pragma mark - TMFJSBridge

- (void)prepareJSBridge
{
    TMFJSBridgeConfiguration.logLevels = TMFBaseCoreLogLevelAll;
    TMFJSBridgeConfiguration *configuration = [TMFJSBridgeConfiguration sharedConfiguration];
    
    // AppInfo
    [configuration registerFunction:@"getVersion" withInvocationClass:[TMFJSBridgeInvocation_getVersion class]];
    [configuration registerFunction:@"getBuild" withInvocationClass:[TMFJSBridgeInvocation_getBuild class]];
    [configuration registerFunction:@"getGUID" withInvocationClass:[TMFJSBridgeInvocation_getGUID class]];
    [configuration registerFunction:@"setPageInfo" withInvocationClass:[TMFJSBridgeInvocation_setPageInfo class]];
    [configuration registerFunction:@"setCustomUserInfo" withInvocationClass:[TMFJSBridgeInvocation_setCustomUserInfo class]];
    
    // Container
    [configuration registerFunction:@"getContainerStack" withInvocationClass:[TMFJSBridgeInvocation_getContainerStack class]];
    [configuration registerFunction:@"openContainer" withInvocationClass:[TMFJSBridgeInvocation_openContainer class]];
    [TMFJSBridgeInvocation_openContainer handleWebViewControllerGeneration:^UIViewController<TMFJSBridgeWebViewControllerProtocol> * _Nullable(__kindof TMFJSBridgeInvocation * _Nonnull invocation, NSDictionary * _Nullable parameters) {
        NSString *URLString = parameters[@"url"];
        if ([URLString length] == 0) return nil;
        NSURL *URL = [NSURL URLWithString:URLString];
        if (URL == nil) return nil;
        UIViewController<TMFJSBridgeWebViewControllerProtocol> *webViewController = nil;
        
        // Demo for WKWebView
        webViewController = [[TMFJSBridgeWKWebViewController alloc] initWithURL:URL];
        return webViewController;
    }];
    [configuration registerFunction:@"closeContainer" withInvocationClass:[TMFJSBridgeInvocation_closeContainer class]];
    [configuration registerFunction:@"closeContainers" withInvocationClass:[TMFJSBridgeInvocation_closeContainers class]];
    [configuration registerFunction:@"setTitle" withInvocationClass:[TMFJSBridgeInvocation_setTitle class]];
    [configuration registerFunction:@"actionSheet" withInvocationClass:[TMFJSBridgeInvocation_actionSheet class]];
    [configuration registerFunction:@"alert" withInvocationClass:[TMFJSBridgeInvocation_alert class]];
    [configuration registerFunction:@"confirm" withInvocationClass:[TMFJSBridgeInvocation_confirm class]];
    [configuration registerFunction:@"setOptionMenu" withInvocationClass:[TMFJSBridgeInvocation_setOptionMenu class]];
    [configuration registerFunction:@"showOptionMenu" withInvocationClass:[TMFJSBridgeInvocation_showOptionMenu class]];
    [configuration registerFunction:@"hideOptionMenu" withInvocationClass:[TMFJSBridgeInvocation_hideOptionMenu class]];
    [configuration registerFunction:@"showLoading" withInvocationClass:[TMFJSBridgeInvocation_showLoading class]];
    [configuration registerFunction:@"hideLoading" withInvocationClass:[TMFJSBridgeInvocation_hideLoading class]];
    [configuration registerFunction:@"showToast" withInvocationClass:[TMFJSBridgeInvocation_showToast class]];
    
    // 3rdApp
    [configuration registerFunction:@"isAppInstalled" withInvocationClass:[TMFJSBridgeInvocation_isAppInstalled class]];
    [configuration registerFunction:@"openApp" withInvocationClass:[TMFJSBridgeInvocation_openApp class]];
    
    // Network
    [configuration registerFunction:@"getNetworkType" withInvocationClass:[TMFJSBridgeInvocation_getNetworkType class]];
    
    // Location
    [configuration registerFunction:@"getLocation" withInvocationClass:[TMFJSBridgeInvocation_getLocation class]];
    [configuration registerFunction:@"openLocation" withInvocationClass:[TMFJSBridgeInvocation_openLocation class]];
    [TMFJSBridgeInvocation_openLocation handleMapViewControllerGeneration:^UIViewController * _Nullable(__kindof TMFJSBridgeInvocation * _Nonnull invocation, NSDictionary * _Nullable parameters) {
        AMapViewController *mapController = [[AMapViewController alloc] init];
        mapController.parameters = parameters;
        return mapController;
    }];
    
    // Data
    [configuration registerFunction:@"getSharedStorage" withInvocationClass:[TMFJSBridgeInvocation_getSharedStorage class]];
    [configuration registerFunction:@"setSharedStorage" withInvocationClass:[TMFJSBridgeInvocation_setSharedStorage class]];
    [configuration registerFunction:@"removeSharedStorage" withInvocationClass:[TMFJSBridgeInvocation_removeSharedStorage class]];
    [configuration registerFunction:@"reportEvent" withInvocationClass:[TMFJSBridgeInvocation_reportEvent class]];
    
    // Share
    [configuration registerFunction:@"updateShareData" withInvocationClass:[TMFJSBridgeInvocation_updateShareData class]];
    [configuration registerFunction:@"shareApp" withInvocationClass:[TMFJSBridgeInvocation_shareApp class]];
    
    // Shark
    [configuration registerFunction:@"sendShark" withInvocationClass:[TMFJSBridgeInvocation_sendShark class]];
    
    // Demo Customized
    [configuration registerFunction:@"echo" withInvocationClass:[TMFJSBridgeInvocation_echo class]];
    [configuration registerFunction:@"customOptionMenu" withInvocationClass:[TMFJSBridgeInvocation_customOptionMenu class]];
    [configuration registerFunction:@"scanBankCard" withInvocationClass:[TMFJSBridgeInvocation_scanBankCard class]];
    [configuration registerFunction:@"LiveCheck" withInvocationClass:[TMFJSBridgeInvocation_LiveCheck class]];
    [configuration registerFunction:@"reportCurrentLocation" withInvocationClass:[TMFJSBridgeInvocation_reportCurrentLocation class]];
    
    // safe keyboard
    [configuration registerFunction:@"showSafeKeyboard" withInvocationClass:[TMFJSBridgeInvocation_safeKeyboardOpen class]];
    [configuration registerFunction:@"closeSafeKeyboard" withInvocationClass:[TMFJSBridgeInvocation_safeKeyboardClose class]];
    [configuration registerFunction:@"recoverSafeKeyboard" withInvocationClass:[TMFJSBridgeInvocation_safeKeyboardRecover class]];
    [configuration registerFunction:@"safeKeyboardWeakCheck" withInvocationClass:[TMFJSBridgeInvocation_safeKeyboardWeakChecking class]];
    [configuration registerFunction:@"safeKeyboardDoubleCheck" withInvocationClass:[TMFJSBridgeInvocation_safeKeyboardDoubleChecking class]];
}

#pragma mark - TMFWebOffline
// 离线包下载进度监听回调
void downloadProgressHandler(NSString *Bid, NSProgress *progress) {
    NSLog(@"[TMFWebOffline Prog] download Bid:%@, progerss:%.2lf", Bid, progress.completedUnitCount * 1.0 / progress.totalUnitCount);
}

- (void)webOfflineNoSpaceHandler{
    NSLog(@"[⚠️][TMFWebOffline no space]");
}

- (void)prepareWebOffline {
    [TMFWebOfflineService setLogLevels:TMFBaseCoreLogLevelAll];
    [TMFWebOfflineService registerDownloadProgressHandler:downloadProgressHandler];

    [TMFWebOfflineService setFallbackEnabled:YES];
#define WEBOFFLINE_USE_LOCAL_PEM 0
#if WEBOFFLINE_USE_LOCAL_PEM
    // 使用本地的离线包公钥
    NSString *publicKeyPath = [[NSBundle mainBundle] pathForResource:@"weboffline_public_key" ofType:@"pem"];
    NSString *publicKey = [NSString stringWithContentsOfFile:publicKeyPath encoding:NSUTF8StringEncoding error:nil];
#else
    // 使用tmf-ios-configurations.json中的离线包公钥信息
    NSString *publicKey = [TMFServerManager shareServerManager].serverConfiguration.webOfflineConfigurationPublicKey;
#endif
    [TMFWebOfflineService setPublicKey:publicKey];
    
#define ENABLE_PUSH_HANDLER 1
#ifdef ENABLE_PUSH_HANDLER
    // Note: 通过个性化配置，定义了默认的离线包PUSH处理器，后续收到离线包的PUSH将由SDK自行处理
    //  同时 `TMFWebOfflineServiceShouldUpdateNotification` 将不再广播!!!
    //  PUSH处理策略：`不更新`，`仅WiFi下更新`，`WiFi和蜂窝都更新`
    TMFWebOfflineConfiguration *configuration = [TMFWebOfflineConfiguration configuration];
    configuration.pushHandlePolicy = TMFWebOfflinePushHandlePolicyWiFiAndCellular;
    [TMFWebOfflineService activateWithConfiguration:configuration];
#else
    [TMFWebOfflineService activate];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(webOfflineServiceShouldUpdate:) name:TMFWebOfflineServiceShouldUpdateNotification object:nil];
#endif
    [[NSNotificationCenter defaultCenter]  addObserver:self selector:@selector(webOfflineNoSpaceHandler) name:(NSString *)TMFWebOfflineDownloadNoSpaceNotification object:nil];
}

- (void)checkUpdateWebOfflineAutomatically {
#if WEB_OFFLINE_TEST_AUTO_UPDATE
    static NSDate *lastUpdateTime = nil;
    // App 进前台时，每 30 分钟检查一次更新
    if (lastUpdateTime == nil || ABS([lastUpdateTime timeIntervalSinceNow]) > 30 * 60) {
        lastUpdateTime = [NSDate date];
        [TMFWebOfflineService checkAndUpdateWithBID:WEB_OFFLINE_TEST_BID completionHandler:^(BOOL isUpdated, NSError * _Nullable error) {
            // 自动更新离线包成功
        }];
    }
#endif
}

- (void)webOfflineServiceShouldUpdate:(NSNotification *)notification {
    // 离线包更新通知
    NSArray<NSString *> *BIDs = [notification.userInfo objectForKey:TMFWebOfflineServiceUpdatedBIDsKey];
    [self checkUpdateWebOfflineForcelyWithBIDs:BIDs];
}

- (void)checkUpdateWebOfflineForcelyWithBIDs:(NSArray<NSString *> *)BIDs {
    TMFWebOfflineServiceOptions *options = [TMFWebOfflineServiceOptions options];
    options.source = TMFWebOfflineServiceOptionsSourcePush;
    options.ignoresDelay = YES;
    options.ignoresFrequency = YES;
    [TMFWebOfflineService checkAndUpdateWithBIDs:BIDs options:options completionHandler:^(NSArray<NSString *> * _Nonnull updatedBIDs) {
        NSLog(@"updatedBIDs: %@", updatedBIDs);
    }];
}

#pragma mark - WCDB

- (void)prepareWCDB {
    // 这里演示如何调用WCDB
    [TMFWCDBSampleUsage setupUsage];
}

#pragma mark - QAPM

#if defined(DEBUG) || defined(RDM)

#define USE_VM_LOGGER

#ifdef USE_VM_LOGGER

/// 私有API请不要在发布APPSotre时使用。
typedef void (malloc_logger_t)(uint32_t type, uintptr_t arg1, uintptr_t arg2, uintptr_t arg3, uintptr_t result, uint32_t num_hot_frames_to_skip);
extern malloc_logger_t* __syscall_logger;

#endif

#endif

void loggerFunc(QAPMLoggerLevel level, const char* log) {
#ifdef RELEASE
    if (level <= QAPMLogLevel_Event) { ///外发版本log
        NSLog(@"%@", [NSString stringWithUTF8String:log]);
    }
#endif
    
#ifdef GRAY
    if (level <= QAPMLogLevel_Info) { ///灰度和外发版本log
        NSLog(@"%@", [NSString stringWithUTF8String:log]);
    }
#endif
    
#ifdef DEBUG
    if (level <= QAPMLogLevel_Debug) { ///内部版本、灰度和外发版本log
        NSLog(@"%@", [NSString stringWithUTF8String:log]);
    }
#endif
}


- (void)setUpFlex {
    
    if (![[NSUserDefaults standardUserDefaults] objectForKey:@"A_QAPM_HTTPMonitorURLList"]) {
        [[NSUserDefaults standardUserDefaults] setObject:@[@"https://gank.io",@"http://w-63209-28716-59529.479064108.sites.hk36.qifeiye.com/"] forKey:@"A_QAPM_HTTPMonitorURLList"];
    }
    
    if (![[NSUserDefaults standardUserDefaults] objectForKey:@"A_QAPM_WKWebview_URL"]) {
        [[NSUserDefaults standardUserDefaults] setObject:@"http://app.html5.qq.com/navi/index" forKey:@"A_QAPM_WKWebview_URL"];
    }
    
    if (![[NSUserDefaults standardUserDefaults] objectForKey:@"A_QAPM_Upload_Host"]) {
        [[NSUserDefaults standardUserDefaults] setObject:@"http://162.14.17.144:30080" forKey:@"A_QAPM_Upload_Host"];
    }
    
//    [[FLEXManager sharedManager] showExplorer];

}

- (void)prepareQAPM {
    
    
    [QAPMCoreDataManager initCoreDataBaseWithFileName:@"contact"];
    
    [self setUpFlex];
    
    NSLog(@"qapm sdk version : %@", [QAPM sdkVersion]);
    
    /********************************  QAPM SDK ****************/
    /// 设置QAPM 日志输出
    [QAPM registerLogCallback:loggerFunc];

 
    /// 内网可以根据需要打开
    [[QAPMModelStableConfig getInstance] setupModelAll:1];
    
    [QAPMConfig getInstance].uuidFromDsym = NO;
    NSString *uuid = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"com.tencent.qapm.uuid"];
    if(!uuid){
        uuid = @"0";
    
    }
    NSLog(@"uuid::::%@",uuid);
    [QAPMConfig getInstance].dysmUuid = uuid;
 
    #ifdef USE_VM_LOGGER
    // ！！！Sigkill功能私有API请不要在发布APPSotre时使用。开启这个功能可以监控到VM内存的分配的堆栈。
    [[QAPMFoomProfile getInstance] setVMLogger:(void**)&__syscall_logger];
    #endif
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    self.configValue = [NSMutableDictionary dictionaryWithDictionary:[defaults objectForKey:@"configDiction"]];

    if ([self.configValue isEqualToDictionary:[NSMutableDictionary new]]) {

        NSString *path = [[NSBundle mainBundle]pathForResource:@"QAPMConfig" ofType:@"plist"];
        NSMutableArray *tableArray= [[NSMutableArray alloc]initWithContentsOfFile:path];

        self.configValue = tableArray[0];

        self.configName = self.configValue[@"configurename"];

        [QAPMConfig getInstance].host = self.configValue[@"host"];

        [QAPMConfig getInstance].deviceID = @"1234567890";
//        [QAPMConfig getInstance].athenaHost = self.configValue[@"athenaHost"];
        
        [QAPMConfig getInstance].appconfigHost = self.configValue[@"configHost"];

        // 设置用户标记
        [QAPMConfig getInstance].userId = self.configValue[@"userId"];
        // 设置App版本号
        [QAPMConfig getInstance].customerAppVersion = self.configValue[@"customerAppVersion"];

        [QAPM startWithAppKey:self.configValue[@"startWithAppKey"]];


    } else if (self.configValue) {


        self.configName = self.configValue[@"configurename"];

        [QAPMConfig getInstance].host = self.configValue[@"host"];

//        [QAPMConfig getInstance].athenaHost = self.configValue[@"athenaHost"];
        [QAPMConfig getInstance].deviceID = @"1234567890";
        
        [QAPMConfig getInstance].appconfigHost = self.configValue[@"configHost"];

        // 设置用户标记
        [QAPMConfig getInstance].userId = self.configValue[@"userId"];
        // 设置App版本号
        [QAPMConfig getInstance].customerAppVersion = self.configValue[@"customerAppVersion"];

        [QAPM startWithAppKey:self.configValue[@"startWithAppKey"]];

    }
    
    [[NSUserDefaults standardUserDefaults] setObject:self.self.configName forKey:@"configname"];
}

#pragma mark - QAPM Yelllow Delgate
- (void)handleVCLeak:(UIViewController *)vc oprSeq:(NSString *)seq stackInfo:(NSString *)stack {
    
}


#pragma mark - Share

- (void)prepareShare {
    /*
     - QQAppId 需要到手Q平台申请
     - WeChatAppId 需要到微信移动开发平台申请
     */
    NSString *universalLink = @"https://switchy.wang/";
    [[TMFShareManager shareManager] registerApp:TMFShareAppWeChat withAppId:SHARE_WECHAT_APP_ID universalLink:universalLink];
    [[TMFShareManager shareManager] registerApp:TMFShareAppQQ withAppId:SHARE_QQ_APP_ID universalLink:universalLink];
    [[TMFShareManager shareManager] registerApp:TMFShareAppAlipay withAppId:SHARE_ALIPAY_APP_ID universalLink:universalLink];   // 支付宝暂时不支持 UniversalLink
    [[TMFShareManager shareManager] registerApp:TMFShareAppWeibo withAppId:SHARE_WEIBO_APP_ID universalLink:universalLink];
    [[TMFShareManager shareManager] registerApp:TMFShareAppDDing withAppId:SHARE_DDING_APP_ID universalLink:universalLink];     // 钉钉暂时不支持 UniversalLink
}

- (BOOL)handleShareWithOpenURL:(NSURL *)url {
    // 监听分享App的拉起处理
    /*
     - WeChat需要到 Info.plit 中
     - 在LSApplicationQueriesSchemes中增加一条URL Scheme，命名为：weixin
     - 在URL Types中添加一条URL Type，Identifier为：weixin，URL Schemes为：<appid，如：wx8071141a542f9dad>
     */
    /*
     - QQ需要到 Info.plit 中
     - 在LSApplicationQueriesSchemes中增加一条URL Scheme，命名规则为：tencent+appidm，mqqopensdkapiV2
     - 在URL Types中添加一条URL Type，Identifier为：mqq，URL Schemes为：<tencent+appid，如：tencent1108776361>
     */
    /*
     - AliPay需要到 Info.plit 中
     - 在LSApplicationQueriesSchemes中增加一条URL Scheme，命名规则为：alipay，alipayshare
     - 在URL Types中添加一条URL Type，Identifier为：alipayshare，URL Schemes为：<ap+appid，如：ap2019050864378948>
     */
    /*
     - Weibo需要到 Info.plit 中
     - 在LSApplicationQueriesSchemes中增加一条URL Scheme，命名规则为：sinaweibo，sinaweibohd，weibosdk，weibosdk2.5
     - 在URL Types中添加一条URL Type，Identifier为：com.weibo，URL Schemes为：<wb+appid，如：wb970420627>
     */
    return [[TMFShareManager shareManager] handleOpenURL:url];
}

#pragma mark - TMFPush

- (void)preparePush {
    [TMFPush setLogLevels:TMFBaseCoreLogLevelAll];
    [[TMFPush defaultManager] startPushWithDelegate:self];
    NSLog(@"[TMFPush] deviceToken: %@", [TMFPushTokenManager defaultTokenManager].deviceTokenString);
}

- (void)tmfPushDidRegisterDeviceToken:(NSString *)deviceToken error:(NSError *)error {
    NSLog(@"[TMFPush] did register deviceToken: %@, error: %@", deviceToken, error);
}

- (void)tmfPushDidReceiveRemoteNotification:(nonnull id)notification withCompletionHandler:(nullable void (^)(UIBackgroundFetchResult))completionHandler {
    NSLog(@"[TMFPush] did receive remote notification: %@", notification);
    if ([notification isKindOfClass:[NSDictionary class]]) {
        // NSDictionary
        NSDictionary *data = notification;
        for (NSString *key in data) {
            id value = data[key];
            NSLog(@"[TMFPush] key: %@, value: %@", key, value);
        }
        
        completionHandler(UIBackgroundFetchResultNewData);
    }
}

- (void)tmfPushUserNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(nullable UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions options))completionHandler API_AVAILABLE(ios(10.0)) {
    NSLog(@"[TMFPush] did receive remote notification(iOS10+): %@", notification);
    UNNotification *unNotification = notification;
    UNNotificationContent *content = unNotification.request.content;
    NSLog(@"[TMFPush] title: %@, body: %@", content.title, content.body);
    NSLog(@"[TMFPush] userInfo: %@", content.userInfo);
    
    completionHandler(UNNotificationPresentationOptionBadge|
                      UNNotificationPresentationOptionAlert|
                      UNNotificationPresentationOptionSound);
}

- (void)tmfPushUserNotificationCenter:(UNUserNotificationCenter *)center didReceiveNotificationResponse:(nullable UNNotificationResponse *)response withCompletionHandler:(void (^)(void))completionHandler API_AVAILABLE(ios(10.0)) {
    
    NSDictionary *userInfo = response.notification.request.content.userInfo;
    NSLog(@"[TMFPush] did handle action for remote notification, userInfo: %@", userInfo);
    
    completionHandler();
}

- (void)prepareDataPush
{
    // Data Push
    [[TMFDataPush defaultManager] observerDataPushWithHandler:^(TMFDataPushInfo * _Nonnull info) {
        NSLog(@"[TMFDataPush] did observer data push, value: %@", info.stringValue);
    }];
}

#pragma mark - Distribution

- (void)prepareDistribution
{
    [TMFDistributionManager setLogLevels:TMFBaseCoreLogLevelAll];
    
    // 初始化
    [[TMFDistributionManager sharedManager] initialize];
    
    // 注册自定义 Handler
    [[TMFDistributionManager sharedManager] setDidReceiveDistributionHandler:^(TMFDistributionInfo * _Nullable info,
                                                                               TMFDistributionCompletionBlock  _Nonnull completionHandler) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:info.distributionTitle
                                                                                 message:info.featureDescription
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel
                                                             handler:^(UIAlertAction * _Nonnull action) {
                                                                 completionHandler(NO);
                                                             }];
        UIAlertAction *updateAction = [UIAlertAction actionWithTitle:@"更新" style:UIAlertActionStyleDefault
                                                             handler:^(UIAlertAction * _Nonnull action) {
                                                                 if ([[UIApplication sharedApplication] canOpenURL:info.appStoreURL]) {
                                                                     [[UIApplication sharedApplication] openURL:info.appStoreURL];
                                                                 }
                                                                 completionHandler(YES);
                                                             }];
        [alertController addAction:updateAction];
        if (!info.updatesForcibly) {
            [alertController addAction:cancelAction];
        }
        
        // 弹出弹窗
        UIViewController *rootController = [UIApplication sharedApplication].keyWindow.rootViewController;
        [rootController presentViewController:alertController animated:YES completion:nil];
    }];
}

#pragma mark - TMFRouter
- (void)prepareRouter {
    [self prepareURLRouter];
    [self prepareService];
    [self prepareMicroApp];
}

- (void)prepareURLRouter {
    NSArray<NSString *> *array = @[@"TMFURLRouterTest1VC",@"TMFURLRouterTest2VC",@"TMFURLRouterWebVC"];
    NSBundle *bundle = [NSBundle bundleForClass:self.class];
    NSString *bundleName = [bundle.bundlePath.lastPathComponent componentsSeparatedByString:@"."].firstObject;
    for (NSString *aClassName in array) {
        Class aClass = NSClassFromString(aClassName);
        if (!aClass) {
            aClass = NSClassFromString([NSString stringWithFormat:@"%@.%@", bundleName, aClassName]);
        }
        if (aClass) {
            id<TMFURLRouterProtocol> p = (id<TMFURLRouterProtocol>)aClass;
            [p registerRouter];
        }
    }
}

- (void)prepareService {
    NSArray<NSString *> *array = @[@"TMFServiceTestA"];
    NSBundle *bundle = [NSBundle bundleForClass:self.class];
    NSString *bundleName = [bundle.bundlePath.lastPathComponent componentsSeparatedByString:@"."].firstObject;
    for (NSString *aClassName in array) {
        Class aClass = NSClassFromString(aClassName);
        if (!aClass) {
            aClass = NSClassFromString([NSString stringWithFormat:@"%@.%@", bundleName, aClassName]);
        }
        if (aClass) {
            id<TMFServiceContextProtocol> p = (id<TMFServiceContextProtocol>)aClass;
            [p registerService];
        }
    }
}

- (void)prepareMicroApp {
    NSArray<NSString *> *array = @[@"TMFMicroAppTestA"];
    NSBundle *bundle = [NSBundle bundleForClass:self.class];
    NSString *bundleName = [bundle.bundlePath.lastPathComponent componentsSeparatedByString:@"."].firstObject;
    for (NSString *aClassName in array) {
        Class aClass = NSClassFromString(aClassName);
        if (!aClass) {
            aClass = NSClassFromString([NSString stringWithFormat:@"%@.%@", bundleName, aClassName]);
        }
        if (aClass) {
            id<TMFMicroAppContextProtocol> p = (id<TMFMicroAppContextProtocol>)aClass;
            [p registerMicroApp];
        }
    }
}

#pragma mark - Profile

- (void)prepareProfile
{
    [TMFProfileReporter setLogLevels:TMFBaseCoreLogLevelAll];
    [[TMFProfileReporter defaultReporter] initialize];
}

#pragma mark - PushGo
- (void)preparePushGo{
    [TMFPushGoCenter setLogLevels:TMFBaseCoreLogLevelAll];
    [[TMFPushGoCenter shareCenter] performSelectorOnMainThread:@selector(activate) withObject:nil waitUntilDone:NO];
}

#pragma mark - XLog

- (void)prepareXLog {
    [TMFXLogger setDebugLogLevels:TMFXLogDebugLevelAll];

#if SUPPRT_ENCRYPT
    /**
     @note 如果需要对日志文件进行加密，需要设置公钥。此处的公钥和私钥仅用作测试，实际请参考 https://github.com/Tencent/mars/wiki/Xlog-加密使用指引
     配置
     @note 公钥：a6c20f0c2988446beb3e2fa30afa352884ff7256ae4518820bc1e3f3d921e4878a130fd64df55fee6b411af5ed979646e0d720301dd3c7799fdb2f158868cdc9
     @note 私钥：8e3a33ac7a27dea2e2339aaf35d88680d6a25daa68a0b21f0896eb78ebe4b65b
     */
    [[TMFXLogger defaultLogger]
        setPublicKey:
            @"a6c20f0c2988446beb3e2fa30afa352884ff7256ae4518820bc1e3f3d921e4878a130fd64df55fee6b411af5ed979646e0d720301dd3c7799fdb2f158868cdc9"];
#endif
    [[TMFXLogger defaultLogger] open];
    XLOG_INFO("TestDemo", @"%s", "open");
}

#pragma mark - SDWebImage

- (void)prepareSDWebImage {
    // Add a custom read-only cache path
    // 一般无需配置!!! 在需要资源包图片预加载的时候，可以做该配置!!!
    NSString *bundledPath = [[NSBundle mainBundle].resourcePath stringByAppendingPathComponent:@"TMFCustomPathImages"];
    [SDImageCache sharedImageCache].additionalCachePathBlock = ^NSString * _Nullable(NSString * _Nonnull key) {
        NSString *fileName = [[SDImageCache sharedImageCache] cachePathForKey:key].lastPathComponent;
        return [bundledPath stringByAppendingPathComponent:fileName.stringByDeletingPathExtension];
    };
}

#pragma mark - Map

#define USE_QMAP
- (void)prepareMap {
    // 根据实际选择使用的地图SDK进行APIKey的配置!!!
#ifdef USE_QMAP
    /*
     - 腾讯地图SDK初始化需要配置APIKey。该Key需要在腾讯地图官网控制台注册
     - 注册地址：https://lbs.qq.com/ios_v1/index.html
     */
    [QMapServices sharedServices].APIKey = @"您的key";
    
#else
    /*
     - 高德地图SDK初始化需要配置APIKey。该Key需要在高德官网控制台绑定对应的bundleid
     - 注册地址：http://lbs.amap.com/dev/#/
     */
    [AMapServices sharedServices].apiKey = @"";
#endif
}

#pragma mark - LiveDetection

- (void)prepareLiveDetection {
    
    /*
     - 活体检测封装在YTDetector.framework中，须在Objective-C++中使用!!!
     - 初始化活体检测需要申请AppId和licence!!!
     */
//    NSError *error = nil;
//    BOOL authorized = [[TMFLiveDetectionConfiguration sharedConfiguration] registerWithAppID:TMF_LIVE_KEY
//                                                                                     license:TMF_LIVE_LICENSE
//                                                                                       error:&error];
//    if (!authorized) {
//        NSLog(@"TMFLiveDetection failed to register, error: %@", error);
//    }
}

#pragma mark - SafeKeyboard

- (void)prepareSafeKeyboard{
    SKGetKeyDataImpl *helper = [SKGetKeyDataImpl new];
    [TMFSafeKeyboardConfigApi setFetchRemoteConfigDataNetHelper:helper];
    [TMFSafeKeyboardConfigApi setKeyboardDataConfigMode:TMFCONFIG_MODE_ONLINE];
    [TMFSafeKeyboardConfigApi setKeyboardLayoutConfigMode:TMFCONFIG_MODE_DEFAULT];
}

#pragma mark - ICDP

- (void)prepareICDP {
    [[TMFICDPSettings defaultSettings] install];
}

#pragma mark - Analyse
- (void)prepareAnalyse{
    [TMFAnalyseManager setLogLevels:TMFBaseCoreLogLevelAll];
    TMFAnalyseManager *alyManager = [TMFAnalyseManager shareManager];
    alyManager.config.autoTrack = YES;
    TMFAnalyseArea *area = [[TMFAnalyseArea alloc] init];
    area.country = @"CN";
    area.province = @"BJ";
    area.city = @"BJ";
    alyManager.area = area;
    [alyManager activate];
}
@end
