//
//  TMFAnalyseViewController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2022/7/1.
//  Copyright © 2022 Tencent. All rights reserved.
//

#import "TMFAnalyseViewController.h"
#import "TMFAnalyse.h"
#import "TMFAnalyseManager+DebugTest.h"
#import "TMFProfile.h"
#import "TMFAnalyseCustomEvent.h"
#import "TMFLocation.h"

@implementation TMFAnalyseViewController

#define ROW_REP_APPSTART    @"上报打开APP事件"
#define ROW_REP_LOGIN       @"上报登陆事件"
#define ROW_REP_LOGOUT      @"上报登出事件"
#define ROW_REP_PAGE        @"上报页面事件"
#define ROW_REP_APPEND      @"上报使用时长事件"
#define ROW_REP_CUSTOM      @"批量上报预制自定义事件"
#define ROW_REP_CUSTOM2     @"上报预制KV自定义事件"
#define ROW_REP_CUSTOM3     @"上报KV自定义事件"
#define ROW_REP_LOCATION    @"设置地区信息"
#define ROW_REP_UNLOCATION  @"置空地区信息"


- (void)initDataSource {
    [super initDataSource];
    self.dataSource = @[ROW_REP_APPSTART,
                        ROW_REP_LOGIN,
                        ROW_REP_LOGOUT,
                        ROW_REP_PAGE,
                        ROW_REP_APPEND,
                        ROW_REP_CUSTOM,
                        ROW_REP_CUSTOM2,
                        ROW_REP_CUSTOM3,
                        ROW_REP_LOCATION,
                        ROW_REP_UNLOCATION
                        ];
}

- (void)didSelectCellWithTitle:(NSString *)title {
    if ([title isEqualToString:ROW_REP_APPSTART]) {
        [self reportAppStart];
    } else if ([title isEqualToString:ROW_REP_LOGIN]) {
        [self reportLogin];
    } else if ([title isEqualToString:ROW_REP_LOGOUT]) {
        [self reportLogout];
    } else if ([title isEqualToString:ROW_REP_PAGE]) {
        [self reportPage];
    } else if ([title isEqualToString:ROW_REP_APPEND]) {
        [self reportAppEnd];
    } else if ([title isEqualToString:ROW_REP_CUSTOM]) {
        [self reportCustomEvent];
    } else if ([title isEqualToString:ROW_REP_CUSTOM2]) {
        [self reportCustomEventKV];
    } else if ([title isEqualToString:ROW_REP_CUSTOM3]) {
        [self reportCustomInputEventKV];
    } else if ([title isEqualToString:ROW_REP_LOCATION]) {
        [self reportLocation];
    } else if ([title isEqualToString:ROW_REP_UNLOCATION]) {
        [self clearLocation];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"数据采集-TMFAnalyse";
}

#pragma mark - Actions

- (void)reportAppStart {
    [[TMFAnalyseManager shareManager] testReportAppStart];
}

- (void)reportLogin {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"登陆" message:nil preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = @"userID";
        textField.text = @"Icarus";
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"登陆" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        NSString *userID = alertController.textFields[0].text;
        [self reportLogin:userID];
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)reportLogin:(NSString *)userID{
    [[TMFProfileReporter defaultReporter] updateCustomizedUserID:userID];
    [[TMFAnalyseManager shareManager] reportLoginEvent:userID completion:^(NSError * _Nullable error, NSString * _Nullable eventUUID) {
        NSLog(@"%@, %@", error, eventUUID);
    }];
}

- (void)reportLogout {
    [self reportLogin:nil];
}

- (void)reportPage {
    [[TMFAnalyseManager shareManager] testReportPage];
}

- (void)reportAppEnd {
    [[TMFAnalyseManager shareManager] testReportAppEnd];
}

- (void)reportCustomEvent {
    //
    long tag = (long)[[NSDate date] timeIntervalSince1970] * 1000;
    static int idx = 0;
    TMFAnalyseCustomEvent_53 *custoEvent_53 = [[TMFAnalyseCustomEvent_53 alloc] init];
    custoEvent_53.XXName = [NSString stringWithFormat:@"name_%ld_%d", tag, idx];
    custoEvent_53.XXSex = [NSString stringWithFormat:@"sex_%d", idx % 2];
    
    TMFAnalyseEventItem *item = [[TMFAnalyseEventItem alloc] init];
    item.event = custoEvent_53;
    
    [[TMFAnalyseManager shareManager] reportEvent:item completion:^(NSError * _Nullable error, NSString * _Nullable eventUUID) {
        NSLog(@"%@, %@", error, eventUUID);
    }];
    
    TMFAnalyseCustomEvent_60 *custoEvent_60 = [[TMFAnalyseCustomEvent_60 alloc] init];
    custoEvent_60.attr12 = tag * 100000 + idx * 1000 + 121;
    custoEvent_60.attr11 = tag * 100000 + idx * 1000 + 111;
    custoEvent_60._31233 = tag * 100000 + 312331;
    custoEvent_60._31232 = tag * 100000 + 312321;
    custoEvent_60._3123 = tag * 100000 + 31231;

    TMFAnalyseEventItem *item2 = [[TMFAnalyseEventItem alloc] init];
    item2.event = custoEvent_60;
    
    TMFAnalyseCustomEvent_60 *custoEvent_60_2 = [[TMFAnalyseCustomEvent_60 alloc] init];
    custoEvent_60_2.attr12 = tag * 100000 + idx * 1000 + 122;
    custoEvent_60_2.attr11 = tag * 100000 + idx * 1000 + 112;
    custoEvent_60_2._31233 = tag * 100000 + 312332;
    custoEvent_60_2._31232 = tag * 100000 + 312322;
    custoEvent_60_2._3123 = tag * 100000 + 31232;

    TMFAnalyseEventItem *item3 = [[TMFAnalyseEventItem alloc] init];
    item3.event = custoEvent_60_2;
    
    
    [[TMFAnalyseManager shareManager] reportEvents:@[item2, item3] completion:^(NSError * _Nullable error, NSString * _Nullable eventUUID) {
        NSLog(@"%@, %@", error, eventUUID);
    }];
    
    idx++;
}

- (void)reportCustomEventKV {
    [[TMFAnalyseManager shareManager] reportEvent:100060
                                         property:@{@"attr12":@"attr12"}
                                       completion:^(NSError * _Nullable error, NSString * _Nullable eventUUID) {
        NSLog(@"%@, %@", error, eventUUID);
    }];
}

- (void)reportCustomInputEventKV {
    static int stdEventId = 0;
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"自定义事件上报" message:nil preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = @"自定义事件ID";
        textField.text = [NSString stringWithFormat:@"%d", stdEventId];
    }];
    for (int i = 1; i <= 4; i++) {
        [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
            textField.placeholder = [NSString stringWithFormat:@"事件属性%d Key ", i];
        }];
        if (i != 4) {
            [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
                textField.placeholder = [NSString stringWithFormat:@"事件属性%d Value (String) ", i];
            }];
        } else {
            [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
                textField.placeholder = [NSString stringWithFormat:@"事件属性%d Value (Number) ", i];
            }];
        }
    }

    
    [alertController addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"上报" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        NSString *eventId = alertController.textFields[0].text;
        stdEventId = eventId.intValue;

        NSMutableDictionary *dic = [NSMutableDictionary dictionary];
        for (int i = 0; i < 4; i++) {
            NSString *paramKey = alertController.textFields[i * 2 + 1].text;
            NSString *paramValue = alertController.textFields[i * 2 + 2].text;
            if (i != 3) {
                if (paramKey && paramKey.length && paramValue && paramValue.length) {
                    [dic setObject:paramValue forKey:paramKey];
                }
            } else {
                if (paramKey && paramKey.length && paramValue && paramValue.length) {
                    [dic setObject:@(paramValue.integerValue) forKey:paramKey];
                }
            }
        }
    
        [[TMFAnalyseManager shareManager] reportEvent:eventId.intValue
                                             property:dic
                                           completion:^(NSError * _Nullable error, NSString * _Nullable eventUUID) {
            if (error) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Error" message:error.description preferredStyle:UIAlertControllerStyleAlert];
                    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
                    [self presentViewController:alertController animated:YES completion:nil];
                });
            }
        }];
        
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)reportLocation{
    [[TMFLocationCenter sharedCenter] requestLocationWithSuccessHandler:^(CLLocation *_Nonnull location) {
        location = [TMFLocationHelper applyChinaLocationShift:location];
        
        // 根据经纬度反向地理编译出地址信息
        CLGeocoder *geocoder = [[CLGeocoder alloc] init];
        NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"zh-Hans"];
        [geocoder tmfReverseGeocodeLocation:location preferredLocale:locale
                          completionHandler:^(NSArray<CLPlacemark *> *_Nullable placemarks, NSError *_Nullable error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (error == nil && placemarks.count > 0) {
                    CLPlacemark *placemark = placemarks.firstObject;
                    TMFAnalyseArea *area = [[TMFAnalyseArea alloc] init];
                    area.country = placemark.country;              // 国家
                    area.province = placemark.administrativeArea;  // 省份
                    area.city = placemark.locality;                // 城市
                    [[TMFAnalyseManager shareManager] setArea:area];
                    [self shouAlertMsg:@"地区信息设置成功"];
                } else {
                    [self shouAlertMsg:@"地区信息获取失败"];
                }
            });
        }];
    } failureHandler:^(NSError *_Nonnull error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self shouAlertMsg:@"定位失败"];
        });
    }];
}

- (void)clearLocation{
    [[TMFAnalyseManager shareManager] setArea:nil];
    [self shouAlertMsg:@"地区信息清理成功"];
}

- (void)shouAlertMsg:(NSString *)msg{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:msg preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end
