//
//  MQQCodeScannerOverlayView.m
//  MQQSecure
//
//  Created by Kloudz Liang on 12-12-27.
//  Copyright (c) 2012年 Tencent. All rights reserved.
//

#import "MQQCodeScannerOverlayView.h"
#import <QuartzCore/QuartzCore.h>

#define kScanLineAnimationDuring 2.5f
#define kScanLineImageHeight 120.f
#define kMQQScannerWidth 242.f


@implementation MQQCodeScannerOverlayView

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        CGFloat padding = (self.frame.size.width - kMQQScannerWidth) / 2;
        CGFloat y = padding + 30;
        cropRect = CGRectMake(padding, y, kMQQScannerWidth, kMQQScannerWidth);
        [self loadScaningView];
        [self addScanningAnimation];
    }
    return self;
}

- (void)loadScaningView
{
    CGFloat adjust = 8;
    
    UIImageView *leftTopImage = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MQQRCodeScan_Corner.png"]];
    leftTopImage.frame = CGRectMake(cropRect.origin.x-adjust, cropRect.origin.y-adjust, 39, 39);
    [self addSubview:leftTopImage];
    
    UIImageView *rightTopImage = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MQQRCodeScan_Corner.png"]];
    rightTopImage.transform = CGAffineTransformMakeRotation(M_PI/2);
    rightTopImage.frame = CGRectMake(cropRect.origin.x+cropRect.size.width-39+adjust, cropRect.origin.y-adjust, 39, 39);
    [self addSubview:rightTopImage];
    
    UIImageView *rightBottomImage = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MQQRCodeScan_Corner.png"]];
    rightBottomImage.transform = CGAffineTransformMakeRotation(M_PI);
    rightBottomImage.frame = CGRectMake(cropRect.origin.x+cropRect.size.width-39+adjust, cropRect.origin.y+cropRect.size.height-39+adjust, 39, 39);
    [self addSubview:rightBottomImage];
    
    UIImageView *leftBottomImage = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MQQRCodeScan_Corner.png"]];
    leftBottomImage.transform = CGAffineTransformMakeRotation(-M_PI/2);
    leftBottomImage.frame = CGRectMake(cropRect.origin.x-adjust, cropRect.origin.y+cropRect.size.height-39+adjust, 39, 39);
    [self addSubview:leftBottomImage];
    
    UIView *cropView = [[UIView alloc] initWithFrame:cropRect];
    cropView.clipsToBounds = YES;
    [self addSubview:cropView];
    
    _scanLineImageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MQQRCodeScan_Line.png"]];
    _scanLineImageView.frame = CGRectMake(0, -kScanLineImageHeight, CGRectGetWidth(cropRect), kScanLineImageHeight);
    [cropView addSubview:_scanLineImageView];
    _scanLineImageView.hidden = YES;
}

- (void)addScanningAnimation
{
    _scanLineImageView.hidden = NO;
    CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"transform.translation.y"];
    animation.toValue = [NSNumber numberWithFloat:cropRect.size.height+kScanLineImageHeight];
    animation.duration = kScanLineAnimationDuring;
    animation.removedOnCompletion = NO;
    animation.repeatCount = HUGE_VALF;
    [_scanLineImageView.layer addAnimation:animation forKey:@"animation"];
}

- (void)removeScanningAnimation
{
    _scanLineImageView.hidden = YES;
    [_scanLineImageView.layer removeAllAnimations];
}

- (void)dealloc
{
    [self removeScanningAnimation];
    _scanLineImageView = nil;
}

- (void)drawRect:(CGRect)rect
{
	[super drawRect:rect];
    
    CGContextRef c = UIGraphicsGetCurrentContext();
    [self drawMaskInContext:c];
	CGContextSaveGState(c);
    
	if( nil != _points ) {
		CGFloat blue[4] = {0.0f, 1.0f, 0.0f, 1.0f};
		CGContextSetStrokeColor(c, blue);
		CGContextSetFillColor(c, blue);
		
        CGRect smallSquare = CGRectMake(0, 0, 10, 10);
        for( NSValue* value in _points ) {
            CGPoint point = [self map:[value CGPointValue]];
            smallSquare.origin = CGPointMake(
                                             cropRect.origin.x + point.x - smallSquare.size.width / 2,
                                             cropRect.origin.y + point.y - smallSquare.size.height / 2);
            [self drawRect:smallSquare inContext:c];
        }
	}
}

- (void)drawMaskInContext:(CGContextRef)context
{
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, 0, 0);
    CGContextAddLineToPoint(context, self.bounds.size.width, 0);
    CGContextAddLineToPoint(context, self.bounds.size.width, self.bounds.size.height);
    CGContextAddLineToPoint(context, 0, self.bounds.size.height);
    CGContextAddLineToPoint(context, 0, 0);
    CGContextClosePath(context);
    CGContextSetRGBFillColor(context, 0, 0, 0, 0.5);
    CGContextDrawPath(context, kCGPathFill);
    CGContextClearRect(context, cropRect);
    
//    CGContextBeginPath(context);
//    CGContextSetStrokeColorWithColor(context, [UIColor colorWithWhite:0xff alpha:0.5f].CGColor);
//    CGContextSetLineWidth(context, 1.0f);
//	CGContextMoveToPoint(context, cropRect.origin.x, cropRect.origin.y);
//	CGContextAddLineToPoint(context, cropRect.origin.x + cropRect.size.width, cropRect.origin.y);
//	CGContextAddLineToPoint(context, cropRect.origin.x + cropRect.size.width, cropRect.origin.y + cropRect.size.height);
//	CGContextAddLineToPoint(context, cropRect.origin.x, cropRect.origin.y + cropRect.size.height);
//	CGContextAddLineToPoint(context, cropRect.origin.x, cropRect.origin.y);
//	CGContextStrokePath(context);
}

- (void)drawRect:(CGRect)rect inContext:(CGContextRef)context
{
	CGContextBeginPath(context);
	CGContextMoveToPoint(context, rect.origin.x, rect.origin.y);
	CGContextAddLineToPoint(context, rect.origin.x + rect.size.width, rect.origin.y);
	CGContextAddLineToPoint(context, rect.origin.x + rect.size.width, rect.origin.y + rect.size.height);
	CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y + rect.size.height);
	CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y);
	CGContextStrokePath(context);
}

- (CGPoint)map:(CGPoint)point
{
    int rotation = 90;
    return [self map:point rotation:rotation];
}

- (CGPoint)map:(CGPoint)point rotation:(int)rotation
{
    CGPoint center;
    center.x = cropRect.size.width/2;
    center.y = cropRect.size.height/2;
    float x = point.x - center.x;
    float y = point.y - center.y;
    switch(rotation) {
        case 0:
            point.x = x;
            point.y = y;
            break;
        case 90:
            point.x = -y;
            point.y = x;
            break;
        case 180:
            point.x = -x;
            point.y = -y;
            break;
        case 270:
            point.x = y;
            point.y = -x;
            break;
    }
    point.x = point.x + center.x;
    point.y = point.y + center.y;
    return point;
}

@end
