//
//  TMFCodeScannerListViewController.m
//  TMFDemo
//
//  Created by 石磊 on 2022/3/22.
//  Copyright © 2022 Tencent. All rights reserved.
//

#import "TMFCodeScannerListViewController.h"
#import "MQQRCodeScannerController.h"
#import "TMFCodeScannerSystemController.h"
#import "NSString+Localize.h"


@interface TMFCodeScannerListViewController ()

@end

@implementation TMFCodeScannerListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

#define ROW_TMFCREATECODESCAN                [@"custom interface scan" localizedString]
#define ROW_TMFOPENCODESCAN             [@"built-in interface scan" localizedString]
#define ROW_TMFSystemCODESCAN             [@"system framework scan" localizedString]

- (void)initDataSource {
    [super initDataSource];
    
    self.dataSource = @[ROW_TMFCREATECODESCAN,
                        ROW_TMFOPENCODESCAN,
                        ROW_TMFSystemCODESCAN
    ];
}

- (void)didSelectCellWithTitle:(NSString *)title {
    
    if ([title isEqualToString:ROW_TMFCREATECODESCAN]) {
        [self createQRCodeController];
    } else if ([title isEqualToString:ROW_TMFOPENCODESCAN]) {
        [self openQRCodeController];
    }else if ([title isEqualToString:ROW_TMFSystemCODESCAN]) {
        [self openSystemCodeController];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"QRCode";
}

- (void)createQRCodeController {
    MQQRCodeScannerController *viewController = [[MQQRCodeScannerController alloc] init];
    viewController.view.frame = self.view.frame;
    [self.navigationController pushViewController:viewController animated:YES];
}

- (void)openQRCodeController {
    TMFCodeScannerController *viewController = [[TMFCodeScannerController alloc] init];
    viewController.view.frame = self.view.frame;
    viewController.delegate = self;
    viewController.multSelect = YES;
//    viewController.containContext = @"shxibank";
    viewController.autoTorch = YES;
    [self.navigationController pushViewController:viewController animated:YES];
}
- (void)openSystemCodeController {
    TMFCodeScannerSystemController *viewController = [[TMFCodeScannerSystemController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}


- (void)codeScannerControllerDidCancel{
    NSLog(@"TMFCodeScannerController scan cancel");
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)codeScannerController:(TMFCodeScannerResource)type didScanResult:(NSArray<TMFCodeDetectorResult *> *)result {
    [self.navigationController popViewControllerAnimated:YES];
    [self showAlertMessage:[self getResultStr:result]];
}

- (void)showAlertMessage:(NSString *)message{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
    [alert addAction:action];
    [self presentViewController:alert animated:YES completion:nil];
}

- (NSString *)getResultStr:(NSArray *)results {
    NSMutableString* data = [NSMutableString new];

    for (TMFCodeDetectorResult *result in results) {
        if(result.typeName){
            [data appendString:result.typeName];
            [data appendString:@"\n"];
        }
        if(result.data){
            [data appendString:result.data];
            [data appendString:@"\n"];
        }

        [data appendString:@"====================="];

        [data appendString:@"\n\n"];

        NSMutableString *pointStr = [NSMutableString new];

        for (NSValue *value in result.points) {
            CGPoint point = [value CGPointValue];
            [pointStr appendFormat:@"(%f,%f)",point.x,point.y];
        }
    }
    
    return  data;
}

@end
