//
//  TMFCodeScannerSystemController.m
//  TMFDemo
//
//  Created by 苟苟亚明 on 2022/12/21.
//  Copyright © 2022 Tencent. All rights reserved.
//

#import "TMFCodeScannerSystemController.h"
#import <AVFoundation/AVFoundation.h>
#import "Masonry.h"
#import "NSString+Localize.h"
@interface TMFCodeScannerSystemController ()
@property (strong, nonatomic) AVCaptureSession *session;
@property (strong, nonatomic) NSMutableArray *qrCodesArray;
@property (strong, nonatomic) NSMutableArray *qrCodesButtonArray;
@property (strong, nonatomic) UIButton *reScanButton;
@property (strong, nonatomic) UIView *cameraView;
@property (copy, nonatomic) NSString *urlString;

@end

@implementation TMFCodeScannerSystemController
- (void)viewDidLoad{
    [super viewDidLoad];
    self.cameraView = [[UIView alloc]initWithFrame:self.view.bounds];
    self.view.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:self.cameraView];
    
    self.reScanButton = [UIButton buttonWithType:UIButtonTypeCustom];
    self.reScanButton.backgroundColor = [UIColor blueColor];
    self.reScanButton.hidden = YES;
    [self.reScanButton setTitle:[@"scan code again" localizedString] forState:UIControlStateNormal];
    self.reScanButton.backgroundColor = [UIColor blueColor];
    [self.cameraView addSubview:self.reScanButton];
    [self.reScanButton addTarget:self action:@selector(reScanBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.reScanButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.cameraView);
        make.bottom.equalTo(self.cameraView.mas_bottom).offset(-60);
        make.size.mas_equalTo(CGSizeMake(200, 60));
    }];
   
    self.qrCodesButtonArray = [NSMutableArray new];
    
}
- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    [AVCaptureDevice requestAccessForMediaType:AVMediaTypeVideo completionHandler:^(BOOL granted) {
        if (granted) {
            [self startRunning];// 扫码
        }else{
            dispatch_async(dispatch_get_main_queue(), ^{
               //@"无法访问照相机，请在设置中打开相机权限"
            });
        }
    }];
}

- (void)startRunning {
    if(![self.session isRunning]){
        [self.session performSelectorInBackground:@selector(startRunning) withObject:nil];
    }
    if(self.qrCodesButtonArray.count){
        [self.qrCodesButtonArray enumerateObjectsUsingBlock:^(UIButton *button, NSUInteger idx, BOOL *stop) {
            [button removeFromSuperview];
        }];
        self.qrCodesButtonArray = [NSMutableArray new];
        self.reScanButton.hidden = YES;
    }

}

- (void)stopRunning {
    __weak typeof(self)weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakSelf.session stopRunning];
    });
}

- (AVCaptureSession *)session {
    if (!_session) {
        AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
        AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice:device error:NULL];
        if (input == nil) {
            return nil;
        }
        AVCaptureMetadataOutput *output = [[AVCaptureMetadataOutput alloc]init];
        [output setMetadataObjectsDelegate:self queue:dispatch_get_main_queue()];
        AVCaptureSession *session = [[AVCaptureSession alloc]init];
        [session setSessionPreset:AVCaptureSessionPresetHigh];
        if ([session canAddInput:input]) {
            [session addInput:input];
        }
        if ([session canAddOutput:output]) {
            [session addOutput:output];
        }
        output.metadataObjectTypes = @[AVMetadataObjectTypeQRCode,AVMetadataObjectTypeCode128Code,AVMetadataObjectTypeCode93Code,AVMetadataObjectTypeCode39Code,AVMetadataObjectTypeCode39Mod43Code,AVMetadataObjectTypeEAN8Code,AVMetadataObjectTypeEAN13Code,AVMetadataObjectTypeUPCECode,AVMetadataObjectTypePDF417Code,AVMetadataObjectTypeAztecCode];
        AVCaptureVideoPreviewLayer *previewLayer = [AVCaptureVideoPreviewLayer layerWithSession:session];
        [previewLayer setVideoGravity:AVLayerVideoGravityResizeAspectFill];
        dispatch_async(dispatch_get_main_queue(), ^{
            previewLayer.frame = self.cameraView.bounds;
            [self.cameraView.layer insertSublayer:previewLayer atIndex:0];
        });
        _session = session;
    }
    return _session;
}

#pragma mark AVCaptureMetadataOutputObjectsDelegate
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputMetadataObjects:(NSArray *)metadataObjects fromConnection:(AVCaptureConnection *)connection{
    if ([metadataObjects count] >0){
        NSMutableArray *muchArray = [NSMutableArray new];
        [metadataObjects enumerateObjectsUsingBlock:^(AVMetadataMachineReadableCodeObject *obj, NSUInteger idx, BOOL *stop) {
            if ([obj.type isEqualToString:AVMetadataObjectTypeQRCode]) {// 这里只加了二维码
                [muchArray addObject:obj];
            }
        }];
        
        [self stopRunning];
        if([muchArray count] == 1){
            AVMetadataMachineReadableCodeObject * metadataObject = [muchArray objectAtIndex:0];
            NSString *stringValue = metadataObject.stringValue;
            self.urlString = stringValue.length?stringValue:@"";
            
            NSLog(@" 1111扫描后的url是:%@",self.urlString);
            
            if(self.urlString.length){
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self analyseResultAry:self.urlString];
                    self.reScanButton.hidden = NO;
                });
            }
        }else if([muchArray count] >1){
            self.qrCodesArray = [NSMutableArray new];
            [muchArray enumerateObjectsUsingBlock:^(AVMetadataMachineReadableCodeObject *result, NSUInteger idx, BOOL *stop) {
                NSMutableDictionary *dic = [NSMutableDictionary new];
                NSString *code = result.stringValue;
                [dic setObject:code forKey:@"code"];
                
                NSLog(@"2222 扫描后的url是:%@",code);
                
                CGRect frame = [self makeFrameWithCodeObject:result Index:self.qrCodesArray.count];
                NSString *frameStr = NSStringFromCGRect(frame);
                [dic setObject:frameStr forKey:@"frame"];
                [self.qrCodesArray addObject:dic];
            }];
            
            dispatch_async(dispatch_get_main_queue(), ^{
                self.reScanButton.hidden = NO;
            });
        }
    }
}

- (void)handleBtnAction:(UIButton *)sender {
    NSInteger index = sender.tag - 1000;
    if (index < self.qrCodesArray.count) {
        NSDictionary *dic = self.qrCodesArray[index];
        if([dic.allKeys containsObject:@"code"]){
            self.urlString = [dic objectForKey:@"code"]?[dic objectForKey:@"code"]:@"";
            NSLog(@"2222 扫描后的url是: 选中 %@",self.urlString);
            if(self.urlString.length){
                [self analyseResultAry:self.urlString];
            }
        }
    }
}

-(void)reScanBtnAction:(UIButton *)sender {
    [self startRunning];
}

-(CGRect)makeFrameWithCodeObject:(AVMetadataMachineReadableCodeObject *)objc Index:(NSInteger)index
{
    
    CGSize isize = self.cameraView.frame.size;
    float Wout = 0.00;
    float Hout = 0.00;
    BOOL wMore = YES;
    
    if (isize.width/isize.height > self.cameraView.bounds.size.width/self.cameraView.bounds.size.height) {
        wMore = YES;
        Wout = (isize.width/isize.height)* self.cameraView.bounds.size.height;
        Wout = Wout - self.cameraView.bounds.size.width;
        Wout = Wout/2;
    }else{
        wMore = NO;
        Hout = (isize.height/isize.width)* self.cameraView.bounds.size.width;
        Hout = Hout  - self.cameraView.bounds.size.height;
        Hout = Hout/2;
    }
    
    CGPoint point1 = CGPointZero;
    CGPoint point2 = CGPointZero;
    CGPoint point3 = CGPointZero;
    CGPoint point4 = CGPointZero;
    
    CGRect frame = objc.bounds;
    NSArray *array = objc.corners;
    CGPoint P = frame.origin;
    CGSize S = frame.size;
    
    for (int n = 0; n< array.count; n++) {
        
        CGPoint point = CGPointZero;
        CFDictionaryRef dict = (__bridge CFDictionaryRef)(array[n]);
        CGPointMakeWithDictionaryRepresentation(dict, &point);

        point.x = point.y +  point.x;
        point.y = point.x - point.y;
        point.x = point.x - point.y;

        point.x = (1-point.x);

        if (wMore) {
            point.x = (point.x * (isize.width/isize.height)* self.cameraView.bounds.size.height) - Wout;
            point.y = self.cameraView.bounds.size.height *(point.y);
        }else{
            point.x = self.cameraView.bounds.size.width *(point.x);
            point.y = (point.y) * (isize.height/isize.width)* self.cameraView.bounds.size.width - Hout;
        }
        if (n == 0) {
            point1 = point;
        }
        if (n == 1) {
            point2 = point;
        }
        if (n == 2) {
            point3 = point;
        }
        if (n == 3) {
            point4 = point;
        }
    }

    float minX = point1.x;
    minX = minX>point2.x?point2.x:minX;
    minX = minX>point3.x?point3.x:minX;
    minX = minX>point4.x?point4.x:minX;
    
    float minY = point1.y;
    minY = minY>point2.y?point2.y:minY;
    minY = minY>point3.y?point3.y:minY;
    minY = minY>point4.y?point4.y:minY;
    P.x = minX;
    P.y = minY;
    
    float maxX = point1.x;
    maxX = maxX<point2.x?point2.x:maxX;
    maxX = maxX<point3.x?point3.x:maxX;
    maxX = maxX<point4.x?point4.x:maxX;
    
    float maxY = point1.y;
    maxY = maxY<point2.y?point2.y:maxY;
    maxY = maxY<point3.y?point3.y:maxY;
    maxY = maxY<point4.y?point4.y:maxY;
    
    S.width = maxX - minX;
    S.height = maxY - minY;
    

    CGRect QRFrame = CGRectMake(P.x , P.y  , S.width, S.height);
    
    UIButton *tempButton = [UIButton buttonWithType:UIButtonTypeCustom];
    tempButton.backgroundColor = [UIColor blueColor];
    tempButton.frame = QRFrame;
    [self.cameraView addSubview:tempButton];
    tempButton.tag = 1000 + index;
    [tempButton addTarget:self action:@selector(handleBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.qrCodesButtonArray addObject:tempButton];
    
    return QRFrame;
}

-(void)analyseResultAry:(NSString *)resultAsString{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:resultAsString preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self popoverPresentationController];
    }];
    [alert addAction:action];
    [self presentViewController:alert animated:YES completion:nil];
}

@end
