//
//  TMFDistributionViewController.m
//  TMFDemo
//
//  Created by bentonxiu on 2019/8/8.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFDistributionViewController.h"
#import "TMFDistribution.h"
#import <QMUIKit/QMUIKit.h>

@implementation TMFDistributionViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"TMFDistribution";
    self.view.backgroundColor = UIColor.whiteColor;
    
    UIButton *customButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [customButton setTitle:@"custom processing" forState:UIControlStateNormal];
    [customButton addTarget:self action:@selector(_onCustomButtonClicked:) forControlEvents:UIControlEventTouchUpInside];
    customButton.bounds = CGRectMake(0, 0, 200, 50);
    customButton.center = self.view.center;
    [self.view addSubview:customButton];
}

- (void)_jumpAppStoreWithURL:(NSURL *)appStoreURL
{
    if ([[UIApplication sharedApplication] canOpenURL:appStoreURL]) {
        [[UIApplication sharedApplication] openURL:appStoreURL];
    }
}

- (void)_onCustomButtonClicked:(id)sender
{
    // 注册自定义弹窗处理
    [[TMFDistributionManager sharedManager] setDidReceiveDistributionHandler:^(TMFDistributionInfo * _Nullable info,
                                                                               TMFDistributionCompletionBlock  _Nonnull completionHandler) {
        UIViewController *rootController = [UIApplication sharedApplication].keyWindow.rootViewController;
        if ([rootController.presentedViewController isKindOfClass:[UIAlertController class]]) {
            // 当前 alertController 正在展示中
            return;
        }
        
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:info.distributionTitle
                                                                                 message:info.featureDescription
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel
                                                             handler:^(UIAlertAction * _Nonnull action) {
                                                                 completionHandler(NO);
                                                             }];
        UIAlertAction *updateAction = [UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault
                                                             handler:^(UIAlertAction * _Nonnull action) {
                                                                 [self _jumpAppStoreWithURL:info.appStoreURL];
                                                                 completionHandler(YES);
                                                             }];
        [alertController addAction:updateAction];
        if (!info.updatesForcibly) {
            [alertController addAction:cancelAction];
        }
        
        // 弹出弹窗
        [rootController presentViewController:alertController animated:YES completion:nil];
    }];
    
    [QMUITips showInfo:@"Successfully replaced with custom publishing update\nTrigger the update again and take a look" inView:self.view hideAfterDelay:2.f];
}

@end
