//
//  GrayscaleInfoViewController.m
//  TMF-ICDP-Demo
//
//  Created by zhaozhang on 2020/3/30.
//  Copyright © 2020 Tencent. All rights reserved.
//

#import "TMFICDPGrayInfoController.h"
#import "TMFICDPSettingInfoCell.h"
#import "ICDPInterface.h"
#import <Masonry.h>
#import "NSString+Localize.h"
#define UIColorFromRGB(r,g,b) [UIColor colorWithRed:(r)/255.0 green:(g)/255.0 blue:(b)/255.0 alpha:1.0]
@interface TMFICDPGrayInfoController ()<UITableViewDelegate,UITableViewDataSource>
@property (nonatomic,strong)UITableView *tableView;
@property (nonatomic,copy) NSArray *titleArr;
@property (nonatomic,strong)NSMutableArray *contentArr;
@property (nonatomic,copy) NSArray *keyArr;
@property (nonatomic,strong)UIBarButtonItem *rightItem;
@property (nonatomic,assign) BOOL isEdit;
@property (nonatomic,weak)UITextField *tmpTextField;
@end

@implementation TMFICDPGrayInfoController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.view addSubview:self.tableView];
    _rightItem = [[UIBarButtonItem alloc] initWithTitle:[@"edit" localizedString] style:UIBarButtonItemStylePlain target:self action:@selector(editOrSaveAction)];
    self.navigationItem.rightBarButtonItem = _rightItem;
    ICDPInterface *iCDPInterface = [ICDPInterface sharedInstance];
    _titleArr = @[[@"device ID" localizedString],[@"user ID" localizedString],[@"current system" localizedString],[@"city" localizedString]];
    _keyArr = @[@"deviceID",@"userId",@"deviceOS",@"currentCity"];
    _contentArr = @[iCDPInterface.deviceID, iCDPInterface.userId,iCDPInterface.deviceOS,iCDPInterface.currentCity?:@"",].mutableCopy;
    _tableView.userInteractionEnabled = NO;
    [_tableView mas_makeConstraints:^(MASConstraintMaker *make) {
       make.right.offset(0);
       make.bottom.offset(0);
       if (@available(iOS 11.0,*)) {
           make.top.equalTo(self.view.mas_safeAreaLayoutGuideTop).offset(20);
       } else {
           make.top.equalTo(self.view.mas_top).offset(20);
       }
       make.left.equalTo(self.view);
    }];
    [_tableView registerNib:[UINib nibWithNibName:@"TMFICDPSettingInfoCell" bundle:[NSBundle mainBundle]] forCellReuseIdentifier:@"TMFICDPSettingInfoCell"];
    self.view.backgroundColor = UIColorFromRGB(246, 247, 249);
    // Do any additional setup after loading the view.
}

- (void)editOrSaveAction {
    _isEdit = !_isEdit;
    _rightItem.title = _isEdit?[@"save" localizedString]:[@"edit" localizedString];
    _tableView.userInteractionEnabled = _isEdit;
    if (_isEdit) {
        [_tmpTextField becomeFirstResponder];
    } else {
        [_keyArr enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [[ICDPInterface sharedInstance] setValue:_contentArr[idx] forKey:obj];
        }];
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:[@"successfully saved" localizedString] message:nil preferredStyle:UIAlertControllerStyleAlert];
         [self presentViewController:alert animated:YES completion:nil];
         [self performSelector:@selector(dismiss:) withObject:alert afterDelay:2.0];
    }
   
}

- (void)dismiss:(UIAlertController *)alert{
    [alert dismissViewControllerAnimated:YES completion:nil];
}

- (UITableView *)tableView {
    if (!_tableView) {
        _tableView = [UITableView new];
        _tableView.delegate = self;
        _tableView.dataSource = self;
        _tableView.backgroundColor = UIColorFromRGB(246, 247, 249);
        _tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
        _tableView.rowHeight = 49;
        _tableView.showsVerticalScrollIndicator = NO;
        _tableView.scrollEnabled = NO;
        UIButton *footerBtn = [[UIButton alloc]init];
        [footerBtn setTitle:[UIDevice currentDevice].identifierForVendor.UUIDString forState:UIControlStateNormal];
        [footerBtn setTitleColor:[UIColor redColor] forState:UIControlStateNormal];
        footerBtn.titleLabel.font = [UIFont systemFontOfSize:12];
        [footerBtn addTarget:self action:@selector(copyUUIDStringClick) forControlEvents:UIControlEventTouchUpInside];
        [footerBtn sizeToFit];
        _tableView.tableFooterView = footerBtn;
    }
     return _tableView;
}

- (void)copyUUIDStringClick{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"⚠️" message:[@"copy uuid successfully" localizedString] preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alert animated:YES completion:nil];
    [UIPasteboard generalPasteboard].string = [UIDevice currentDevice].identifierForVendor.UUIDString;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _titleArr.count;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    TMFICDPSettingInfoCell *cell = [tableView dequeueReusableCellWithIdentifier:@"TMFICDPSettingInfoCell" forIndexPath:indexPath];
    [cell reloadCellWithData:_titleArr[indexPath.row] contentArr:_contentArr[indexPath.row]];
    __weak typeof(self) weakSelf = self;
    cell.changeContentBlock = ^(NSString * _Nonnull content) {
        weakSelf.contentArr[indexPath.row] = content;
    };
    if(indexPath.row == 0) {
        _tmpTextField = cell.infoTextField;
    }
    return cell;
}



/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
