//
//  SettingInfoViewController.m
//  ICDPDemo
//
//  Created by zhaozhang on 2019/12/3.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFICDPSettingInfoController.h"
#import "TMFICDPSettingInfoCell.h"
#import "TMFICDPSettingInfoData.h"
#import <Masonry.h>
#import "NSString+Localize.h"
#define UIColorFromRGB(r,g,b) [UIColor colorWithRed:(r)/255.0 green:(g)/255.0 blue:(b)/255.0 alpha:1.0]

@interface TMFICDPSettingInfoController ()<UITableViewDelegate,UITableViewDataSource>
@property (nonatomic,strong)UITableView *tableView;
@property (nonatomic,copy) NSArray *titleArr;
@property (nonatomic,strong)NSMutableArray *contentArr;
@property (nonatomic,copy) NSArray *keyArr;
@property (nonatomic,strong)UIBarButtonItem *rightItem;
@property (nonatomic,assign) BOOL isEdit;
@property (nonatomic,weak)UITextField *tmpTextField;
@end

@implementation TMFICDPSettingInfoController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.view addSubview:self.tableView];
    _rightItem = [[UIBarButtonItem alloc] initWithTitle:[@"edit" localizedString] style:UIBarButtonItemStylePlain target:self action:@selector(editOrSaveAction)];
    self.navigationItem.rightBarButtonItem = _rightItem;
    TMFICDPSettingInfoData *settingInfoData = [TMFICDPSettingInfoData sharedInstance];
    _titleArr = @[[@"splash component" localizedString],[@"announcement component" localizedString],[@"carousel component" localizedString],[@"single image component" localizedString],[@"popup component" localizedString],[@"list component" localizedString],[@"cut screen component" localizedString]];
    _contentArr = @[settingInfoData.splashId,
                    settingInfoData.noticeId,
                    settingInfoData.carouselId,
                    settingInfoData.singleId,
                    settingInfoData.popupId,
                    settingInfoData.listId,
                    settingInfoData.cutScreenID].mutableCopy;
    _keyArr = @[@"splashId",@"noticeId",@"carouselId",@"singleId",@"popupId",@"listId",@"cutScreenID"];
    _tableView.userInteractionEnabled = NO;
    [_tableView mas_makeConstraints:^(MASConstraintMaker *make) {
       make.right.offset(0);
       make.bottom.offset(0);
       if (@available(iOS 11.0,*)) {
           make.top.equalTo(self.view.mas_safeAreaLayoutGuideTop).offset(20);
       } else {
           make.top.equalTo(self.view.mas_top).offset(20);
       }
       make.left.equalTo(self.view);
    }];
    [_tableView registerNib:[UINib nibWithNibName:@"TMFICDPSettingInfoCell"
                                           bundle:[NSBundle mainBundle]] forCellReuseIdentifier:@"TMFICDPSettingInfoCell"];
    self.view.backgroundColor = UIColorFromRGB(246, 247, 249);
}

- (void)editOrSaveAction {
    _isEdit = !_isEdit;
    _rightItem.title = _isEdit?[@"save" localizedString]:[@"edit" localizedString];
    _tableView.userInteractionEnabled = _isEdit;
    if (_isEdit) {
        [_tmpTextField becomeFirstResponder];
    } else {
        [_keyArr enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [[TMFICDPSettingInfoData sharedInstance] setValue:_contentArr[idx] forKey:obj];
        }];
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:[@"successfully saved" localizedString] message:nil preferredStyle:UIAlertControllerStyleAlert];
         [self presentViewController:alert animated:YES completion:nil];
         [self performSelector:@selector(dismiss:) withObject:alert afterDelay:2.0];
    }
   
}

- (void)dismiss:(UIAlertController *)alert{
    [alert dismissViewControllerAnimated:YES completion:nil];
}

- (UITableView *)tableView {
    if (!_tableView) {
        _tableView = [UITableView new];
        _tableView.delegate = self;
        _tableView.dataSource = self;
        _tableView.backgroundColor = UIColorFromRGB(246, 247, 249);
        _tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
        _tableView.rowHeight = 49;
        _tableView.showsVerticalScrollIndicator = NO;
        _tableView.scrollEnabled = NO;
    }
     return _tableView;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _titleArr.count;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    TMFICDPSettingInfoCell *cell = [tableView dequeueReusableCellWithIdentifier:@"TMFICDPSettingInfoCell" forIndexPath:indexPath];
    [cell reloadCellWithData:_titleArr[indexPath.row] contentArr:_contentArr[indexPath.row]];
    __weak typeof(self) weakSelf = self;
    cell.changeContentBlock = ^(NSString * _Nonnull content) {
        weakSelf.contentArr[indexPath.row] = content;
    };
    if(indexPath.row == 0) {
        _tmpTextField = cell.infoTextField;
    }
    return cell;
}


@end
