//
//  ICDPSettings.m
//  TMF-ICDP-Demo
//
//  Created by spencerluo(骆寰宇) on 2020/8/17.
//  Copyright © 2020 v_vzzzhang. All rights reserved.
//

#import "TMFICDPSettings.h"
#import "ICDPManager.h"
#import <UIKit/UIKit.h>
#import "TMFSharkCenter.h"

#define TEST_AD_HOST  @"http://154.8.202.237:30028/"
#define POC_AD_HOST  @"http://tmf.qq.com:30028/"

#define TEST_ATHENA_HOST  @"http://154.8.202.237:30028/athena/"
#define POC_ATHENA_HOST  @"http://140.143.13.7:30281/athena/"


@implementation TMFICDPSettings

+ (instancetype)defaultSettings {
    static dispatch_once_t onceToken;
    static TMFICDPSettings *settings;
    dispatch_once(&onceToken, ^{
        settings = [TMFICDPSettings new];
    });
    return settings;
}

- (instancetype)init
{
    self = [super init];
    if (self) {
        _hostType = [[[NSUserDefaults standardUserDefaults] valueForKey:@"hostType"] integerValue];
        _usingCustomHost = [[NSUserDefaults standardUserDefaults] boolForKey:@"usingCustomHost"];
        
        _userID = [[NSUserDefaults standardUserDefaults] valueForKey:@"userID"] ?: @"test";
        _appID = [[NSUserDefaults standardUserDefaults] valueForKey:@"appID"] ?: [TMFSharkCenterConfiguration masterConfiguration].productKey;
        _qapmKey = [[NSUserDefaults standardUserDefaults] valueForKey:@"qapmKey"] ?: @"2ea3f401-8082";
        
        _customADHost = [[NSUserDefaults standardUserDefaults] valueForKey:@"customADHost"];
        _customAthenaHost = [[NSUserDefaults standardUserDefaults] valueForKey:@"customAthenaHost"];

    }
    return self;
}

- (void)install {
    [ICDPManager sharedManager].appId = self.appID;
    [ICDPManager sharedManager].userId = self.userID;

    [ICDPManager sharedManager].deviceID = [[[UIDevice currentDevice] identifierForVendor] UUIDString];
    [ICDPManager sharedManager].customerAppVersion = @"1.0.0";

    [[ICDPManager sharedManager] install];
    
    if (self.usingCustomHost) {
        [ICDPManager sharedManager].adHost = self.customADHost;
    } else {
        if (self.hostType == ICDPHostTypeTest) {
            [ICDPManager sharedManager].adHost = TEST_AD_HOST;
        } else {
            [ICDPManager sharedManager].adHost = POC_AD_HOST;
        }
    }
}

- (void)setAppID:(NSString *)appID {
    _appID = appID;
    [[NSUserDefaults standardUserDefaults] setValue:appID forKey:@"appID"];
}

- (void)setQapmKey:(NSString *)qapmKey {
    _qapmKey = qapmKey;
    [[NSUserDefaults standardUserDefaults] setValue:qapmKey forKey:@"qapmKey"];
}


- (void)setUserID:(NSString *)userID {
    _userID = userID;
    [[NSUserDefaults standardUserDefaults] setValue:userID forKey:@"userID"];
    [ICDPManager sharedManager].userId = userID;
}

- (void)setHostType:(ICDPHostType)hostType {
    _hostType = hostType;
    if (!self.usingCustomHost) {
        if (hostType == ICDPHostTypeTest) {
            [ICDPManager sharedManager].adHost = TEST_AD_HOST;
        } else {
            [ICDPManager sharedManager].adHost = POC_AD_HOST;
        }
    }
    [[NSUserDefaults standardUserDefaults] setInteger:hostType forKey:@"hostType"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)setUsingCustomHost:(BOOL)usingCustomHost {
    _usingCustomHost = usingCustomHost;
    [[NSUserDefaults standardUserDefaults] setValue:@(usingCustomHost) forKey:@"usingCustomHost"];
    if (!usingCustomHost) {
        if (self.hostType == ICDPHostTypeTest) {
            [ICDPManager sharedManager].adHost = TEST_AD_HOST;
        } else {
            [ICDPManager sharedManager].adHost = POC_AD_HOST;
        }
    }
}

- (void)setCustomADHost:(NSString *)customADHost {
    _customADHost = customADHost;
    [[NSUserDefaults standardUserDefaults] setValue:customADHost forKey:@"customADHost"];
    if (self.usingCustomHost) {
        [ICDPManager sharedManager].adHost = customADHost;
    }
}

- (void)setCustomAthenaHost:(NSString *)customAthenaHost {
    _customAthenaHost = customAthenaHost;
    [[NSUserDefaults standardUserDefaults] setValue:customAthenaHost forKey:@"customAthenaHost"];
}

@end
