//
//  ICDPSettingsTableViewController.m
//  TMF-ICDP-Demo
//
//  Created by spencerluo(骆寰宇) on 2020/8/17.
//  Copyright © 2020 v_vzzzhang. All rights reserved.
//

#import "TMFICDPSettingsController.h"
#import "TMFICDPSettings.h"
#import "ICDPManager.h"
#import "UIView+Toast.h"
#import "ICDPManager.h"

@interface TMFICDPSettingsController ()<UITextFieldDelegate>

@property (weak, nonatomic) IBOutlet UISwitch *customHostSwitch;
@property (weak, nonatomic) IBOutlet UITextField *appIDTextField;
@property (weak, nonatomic) IBOutlet UITextField *userIDTextField;
@property (weak, nonatomic) IBOutlet UITableViewCell *pocCell;
@property (weak, nonatomic) IBOutlet UITableViewCell *testCell;
@property (weak, nonatomic) IBOutlet UITextField *adHostTextField;

@end

@implementation TMFICDPSettingsController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.appIDTextField.text = [ICDPManager sharedManager].appId;
    self.userIDTextField.text = [ICDPManager sharedManager].userId;
    
    self.appIDTextField.delegate = self;
    self.appIDTextField.placeholder = @"输入智慧投放 App ID";
    self.userIDTextField.delegate = self;
    self.appIDTextField.placeholder = @"输入自定义 用户ID";
    self.adHostTextField.delegate = self;
    self.adHostTextField.placeholder = @"输入智慧投放服务器地址";

    
    
    switch ([TMFICDPSettings defaultSettings].hostType) {
        case ICDPHostTypePOC:
            self.pocCell.accessoryType = UITableViewCellAccessoryCheckmark;
            self.testCell.accessoryType = UITableViewCellAccessoryNone;
            break;
        case ICDPHostTypeTest:
            self.pocCell.accessoryType = UITableViewCellAccessoryNone;
            self.testCell.accessoryType = UITableViewCellAccessoryCheckmark;
            break;
        default:
            break;
    }
    
    self.customHostSwitch.on = [TMFICDPSettings defaultSettings].usingCustomHost;
    self.adHostTextField.enabled = [TMFICDPSettings defaultSettings].usingCustomHost;
    if ([TMFICDPSettings defaultSettings].usingCustomHost) {
        self.adHostTextField.text = [TMFICDPSettings defaultSettings].customADHost;
    } else {
        self.adHostTextField.text = [ICDPManager sharedManager].adHost;
    }
    [[ICDPManager sharedManager] addMonitorCutscreenWithBoothId:@"123"];
}

- (IBAction)customHostSwitchDidChange:(UISwitch *)sender {
    [TMFICDPSettings defaultSettings].usingCustomHost = sender.isOn;
    self.adHostTextField.enabled = sender.isOn;
        
    if ([TMFICDPSettings defaultSettings].usingCustomHost) {
        self.adHostTextField.text = [TMFICDPSettings defaultSettings].customADHost;
    } else {
        self.adHostTextField.text = [ICDPManager sharedManager].adHost;
    }

    [self.tableView reloadData];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section != 1) {
        return;
    }
    if (indexPath.row == 0) {
        self.pocCell.accessoryType = UITableViewCellAccessoryCheckmark;
        self.testCell.accessoryType = UITableViewCellAccessoryNone;
        [TMFICDPSettings defaultSettings].hostType = ICDPHostTypePOC;
        if (![TMFICDPSettings defaultSettings].usingCustomHost) {
            self.adHostTextField.text = [ICDPManager sharedManager].adHost;
        }
    } else if (indexPath.row == 1) {
        self.testCell.accessoryType = UITableViewCellAccessoryCheckmark;
        self.pocCell.accessoryType = UITableViewCellAccessoryNone;
        [TMFICDPSettings defaultSettings].hostType = ICDPHostTypeTest;
        if (![TMFICDPSettings defaultSettings].usingCustomHost) {
            self.adHostTextField.text = [ICDPManager sharedManager].adHost;
        }
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 44;
}


- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    NSString *text = textField.text;
    if (textField == self.appIDTextField) {
        if (![text isEqualToString:[TMFICDPSettings defaultSettings].appID]) {
            [self.view makeToast:@"请重启App以生效配置" duration:1.0 position:CSToastPositionTop];
        }
        [TMFICDPSettings defaultSettings].appID = text;
    }  else if (textField == self.userIDTextField) {
        [TMFICDPSettings defaultSettings].userID = text;
        [self.view makeToast:@"设置成功" duration:1.0 position:CSToastPositionTop];
    } else if (textField == self.adHostTextField) {
        [TMFICDPSettings defaultSettings].customADHost = text;
        [self.view makeToast:@"设置成功" duration:1.0 position:CSToastPositionTop];
    }
}


@end
