//
//  TMFJSBridgeInvocation_customOptionMenu.m
//  TMFDemo
//
//  Created by klaudz on 2019/5/12.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFJSBridgeInvocation_customOptionMenu.h"
#import "TMFJSBridgeInvocation+Protected.h"
#import "NSString+Localize.h"

@interface TMFJSBridgeInvocation_customOptionMenu ()

/**
 @brief 菜单选项条目
 */
@property (nonatomic, copy) NSArray *items;

@end

@implementation TMFJSBridgeInvocation_customOptionMenu

- (void)invokeWithParameters:(NSDictionary *)parameters
{
    NSArray *items = MQQArrayValue(parameters[@"items"], nil);
    
    if ([items count] == 0) {
        [self failWithValues:nil completed:YES];
        return;
    }
    
    self.items = items;
    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:[@"more" localizedString] style:UIBarButtonItemStylePlain target:self
                                                                          action:@selector(rightBarButtonItemClicked:)];
    self.webViewController.navigationItem.rightBarButtonItem = rightBarButtonItem;
    
    [self finishWithValues:nil completed:NO];
}

- (void)rightBarButtonItemClicked:(id)item
{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];
    for (NSDictionary *item in self.items) {
        if (![item isKindOfClass:[NSDictionary class]]) {
            continue;
        }
        NSString *title = MQQStringValue(item[@"title"], nil);
        if ([title length] == 0) {
            continue;
        }
        NSInteger index = MQQIntegerValue(item[@"index"], -1);
        [alertController addAction:[UIAlertAction actionWithTitle:title style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            NSMutableDictionary *values = [NSMutableDictionary dictionary];
            values[@"index"] = @(index);
            values[@"tips"] = [@"clicked" localizedString];
            
            if (index == 1) {
                NSString *URLString = self.JSBridge.webViewURL.absoluteString;
                if (URLString) {
                    [[UIPasteboard generalPasteboard] setString:URLString];
                }
            }
            
            [self callbackWithIdentifier:item[@"action"] values:values];
        }]];
    }
    [alertController addAction:[UIAlertAction actionWithTitle:[@"cancel" localizedString] style:UIAlertActionStyleCancel handler:nil]];
    [self.webViewController presentViewController:alertController animated:YES completion:nil];
}

@end
