//
//  TMFJSBridgeInvocation_scanBankCard.m
//  TMFDemo
//
//  Created by klaudz on 2019/5/12.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFJSBridgeInvocation_scanBankCard.h"
#import "TMFJSBridgeInvocation+Protected.h"

#import "TMFOCRSystemAuthorizer.h"
#import "UIView+EMBCFrameAdjust.h"

#import "TMFOCRProcessViewController.h"
#import "TMFOCRCameraViewController.h"

@interface TMFJSBridgeInvocation_scanBankCard () <UINavigationControllerDelegate,UIImagePickerControllerDelegate, TMFOCRViewControllerDelegate>
@end

@implementation TMFJSBridgeInvocation_scanBankCard

- (void)invokeWithParameters:(NSDictionary *)parameters {
    NSString *device = MQQStringValue(parameters[@"device"], nil);
    if ([device isEqualToString:@"photo"]) {
        [self photoDidClick];
    } else {
        [self cameraDidClick];
    }
}

- (void)cameraDidClick {
    __weak typeof(&*self)weakSelf = self;
    [TMFOCRSystemAuthorizer requestAuthorizedWithType:TMFOCRSystemAuthorizerTypeCamera completion:^(TMFOCRSystemAuthorizerStatus status) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (TMFOCRSystemAuthorizerStatusAuthorized == status) {
                TMFOCRCameraViewController *viewController = [[TMFOCRCameraViewController alloc] initWithScanType:TMFOCRScanTypeBankcard];
                viewController.enableInterestRect = YES;
                viewController.delegate = self;
                [weakSelf.webViewController.navigationController pushViewController:viewController animated:YES];
            } else {
                 [self failWithValues:nil completed:YES];
            }
        });
    }];
}

//本地相机调取
- (void)photoDidClick {
    __weak typeof(&*self)weakSelf = self;
    [TMFOCRSystemAuthorizer requestAuthorizedWithType:TMFOCRSystemAuthorizerTypePhoto completion:^(TMFOCRSystemAuthorizerStatus status) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (TMFOCRSystemAuthorizerStatusAuthorized == status) {
                //相册选取
                UIImagePickerController *pickerVC = [[UIImagePickerController alloc] init];
                pickerVC.delegate = weakSelf;
                [weakSelf.webViewController presentViewController:pickerVC animated:YES completion:NULL];
            } else {
                [self failWithValues:nil completed:YES];
            }
        });
    }];
}

#pragma mark - image picker delegate

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *,id> *)info {
    UIImage *pickerImage = [info objectForKey:UIImagePickerControllerOriginalImage];
    TMFOCRProcessViewController *viewController = [[TMFOCRProcessViewController alloc] initWithOriginalImage:pickerImage
                                                                                                    scanType:TMFOCRScanTypeBankcard];
    viewController.delegate = self;
    [self.webViewController.navigationController pushViewController:viewController animated:NO];
    [self.webViewController dismissViewControllerAnimated:YES completion:nil];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [self.webViewController dismissViewControllerAnimated:YES completion:nil];
    [self failWithValues:nil completed:YES];
}

#pragma mark - TMFOCRCameraViewControllerDelegate

- (void)OCRViewController:(UIViewController *)viewController didCompleteWithResult:(nullable TMFOCRScanModel *)model {
    
    NSMutableDictionary *returnValue = [NSMutableDictionary dictionary];
    if (model) {
        TMFOCRBankcardModel *curModel = (TMFOCRBankcardModel *)model;
        if (curModel.name) {
            returnValue[@"name"] = curModel.name;
        }
        if (curModel.number) {
            returnValue[@"number"] = curModel.number;
        }
        if (curModel.bank) {
            returnValue[@"bank"] = curModel.bank;
        }
        if (curModel.cardName) {
            returnValue[@"cardName"] = curModel.cardName;
        }
        if (curModel.cardType) {
            returnValue[@"cardType"] = curModel.cardType;
        }
        if (curModel.validDate) {
            returnValue[@"validDate"] = curModel.validDate;
        }
        [self finishWithValues:returnValue completed:YES];
    } else {
        [self failWithValues:nil completed:YES];
    }
}

@end
