//
//  TMFJSBridgeInvocation_reportCurrentLocation.m
//  TMFDemo
//
//  Created by hauzhong on 2019/7/5.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFJSBridgeInvocation_reportCurrentLocation.h"
#import "TMFJSBridgeInvocation+Protected.h"

// TMFLocation
#import "TMFLocationCenter.h"
#import "TMFLocationHelper.h"
#import "CLGeocoder+TMFExtension.h"

@interface TMFJSBridgeInvocation_reportCurrentLocation ()

/**
 @brief 位置信息请求ID，可以用来停止请求位置信息
 */
@property (nonatomic, assign) TMFLocationRequestIdentifier locationRequestIdentifier;

@end

@implementation TMFJSBridgeInvocation_reportCurrentLocation

- (void)invokeWithParameters:(NSDictionary *)parameters {
    
    self.locationRequestIdentifier = [[TMFLocationCenter sharedCenter] requestLocationWithSuccessHandler:^(CLLocation * _Nonnull location) {
        
        self.locationRequestIdentifier = TMFLocationRequestIdentifierInvalid;
        
        // 火星坐标翻译
        location = [TMFLocationHelper applyChinaLocationShift:location];
        
        // 根据经纬度反向地理编译出地址信息
        CLGeocoder *geocoder = [[CLGeocoder alloc] init];
        NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"zh-Hant"];
        [geocoder tmfReverseGeocodeLocation:location preferredLocale:locale completionHandler:^(NSArray<CLPlacemark *> * _Nullable placemarks,
                                                                                                NSError * _Nullable error) {
            if (error == nil && placemarks.count > 0) {
                CLPlacemark *placemark = placemarks.firstObject;
//                [[TMFLocationAnalyticsService defaultService] reportWithPlacemark:placemark];
                [self finishWithValues:nil completed:YES];
            } else {
                [self failWithValues:nil completed:YES];
            }
        }];
        
    } failureHandler:^(NSError * _Nonnull error) {
        
        self.locationRequestIdentifier = TMFLocationRequestIdentifierInvalid;
        
        [self failWithValues:nil completed:YES];
    }];
}

- (void)cancel
{
    [[TMFLocationCenter sharedCenter] cancelRequest:self.locationRequestIdentifier];
    self.locationRequestIdentifier = TMFLocationRequestIdentifierInvalid;
}

@end
