//
//  TMFJSBridgeWKWebViewController.m
//  TMFDemo
//
//  Created by klaudz on 18/4/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFJSBridgeWKWebViewController.h"
#import "TMFWkWebView.h"
#import "TMFSafeKeyboardWrapperApi+WebApi.h"

NS_ASSUME_NONNULL_BEGIN

@interface TMFJSBridgeWKWebViewController ()<TMFSKCustomWebInputViewDelegate, TMFSafeKeyboardDelegate>

/**
 @brief 当前加载的URL
 */
@property (nonatomic, strong) NSURL *URL;

/**
 @brief webview
 */
@property (nonatomic, strong) TMFWkWebView *webView;

@end

@implementation TMFJSBridgeWKWebViewController

@synthesize tmf_JSBridge = _tmf_JSBridge;

- (instancetype)initWithURL:(NSURL *)URL
{
    self = [super init];
    if (self) {
        self.URL = URL;
        self.tmf_JSBridge = [[TMFJSBridge alloc] initWithWebViewController:self];
        
        self.title = @"TMFJSBridge Demo";
    }
    return self;
}

- (void)dealloc
{
    NSLog(@"%s", __func__);
    [self.tmf_JSBridge clearWebViewControllerAndCancel];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
//    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"更多" style:UIBarButtonItemStylePlain target:nil action:NULL];
//    self.navigationItem.rightBarButtonItem = rightBarButtonItem;
//    
    self.navigationController.toolbarHidden = NO;
    [self setToolbarItems:@[
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRewind target:self action:@selector(backButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFastForward target:self action:@selector(forwardButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(refreshButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCompose target:self action:@selector(goButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
    ]];
    // keyboard
    self.codeViewKeyboardWrapper = [[TMFSafeKeyboardWrapper alloc] initWithKeyboard:TMFSafeKeyboard.new];
    [self.codeViewKeyboardWrapper setKeyboardDelegate:self];
           
    TMFWkWebView *webView = [[TMFWkWebView alloc] initWithFrame:self.view.frame];
    webView.UIDelegate = self;
    webView.navigationDelegate = self;
    self.webView = webView;
    [self.view addSubview:self.webView];
    
    NSURLRequest *request = [NSURLRequest requestWithURL:self.URL];
    [self.webView loadRequest:request];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    [self.tmf_JSBridge notifyWithEvent:@"containerAppear" parameters:nil];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    
    [self.tmf_JSBridge notifyWithEvent:@"containerDisappear" parameters:nil];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

#pragma mark - TMFWkWebViewDelegate

- (void)webView:(WKWebView *)webView
    decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction
                    decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler{
    NSURLRequest *request = navigationAction.request;
    
    // Handle the JSBridge operation if it can
    if ([self.tmf_JSBridge canHandleURL:request.URL]) {
        [self.tmf_JSBridge handleURL:request.URL];
        decisionHandler(WKNavigationActionPolicyCancel);
        return;
    }
    
    decisionHandler(WKNavigationActionPolicyAllow);
}

- (void)webView:(WKWebView *)webView didFinishNavigation:(null_unspecified WKNavigation *)navigation
{
    // Inject TMFJSBridge into webView
    [self.tmf_JSBridge injectJSBridgeIntoWebView:nil];
}

- (void)webView:(WKWebView *)webView
    runJavaScriptAlertPanelWithMessage:(NSString *)message
                      initiatedByFrame:(WKFrameInfo *)frame
                     completionHandler:(void (^)(void))completionHandler {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
    [alert addAction:action];
    [self presentViewController:alert animated:YES completion:nil];
    completionHandler();
}

- (void)webView:(WKWebView *)webView
    runJavaScriptConfirmPanelWithMessage:(NSString *)message
                        initiatedByFrame:(WKFrameInfo *)frame
                       completionHandler:(void (^)(BOOL))completionHandler {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"提示" message:message?:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:([UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        completionHandler(NO);
    }])];
    [alertController addAction:([UIAlertAction actionWithTitle:@"确认" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        completionHandler(YES);
    }])];
    [self presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - Button

- (void)backButtonClicked:(id)button
{
    if ([self.webView canGoBack]) {
        [self.webView goBack];
    }
}

- (void)forwardButtonClicked:(id)button
{
    if ([self.webView canGoForward]) {
        [self.webView goForward];
    }
}

- (void)refreshButtonClicked:(id)button
{
    [self.webView reload];
}

- (void)goButtonClicked:(id)button {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Go to URL" message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = self.webView.URL.absoluteString;
        textField.placeholder = @"URL";
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Go" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        NSURL *URL = [NSURL URLWithString:alertController.textFields.firstObject.text];
        if (URL) {
            NSURLRequest *request = [NSURLRequest requestWithURL:URL];
            [self.webView loadRequest:request];
        }
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - TMFJSBridgeWebViewControllerProtocol

- (UIView *)tmf_webView
{
    return self.webView;
}

#pragma mark - keyboard
- (BOOL)safeKeyboardShouldReturn:(TMFSafeKeyboard *)safeKeyboard{
    return YES;
}
- (void)safeKeyboardUpdateViewConstraints:(TMFSafeKeyboard *)safeKeyboard {}
- (void)safeKeyboardWillAppear:(TMFSafeKeyboard *)safeKeyboard {}
- (void)safeKeyboardDidAppear:(TMFSafeKeyboard *)safeKeyboard {}
- (void)safeKeyboardWillDisappear:(TMFSafeKeyboard *)safeKeyboard {}
- (void)safeKeyboardDidDisappear:(TMFSafeKeyboard *)safeKeyboard {}


@end

NS_ASSUME_NONNULL_END
