//
//  TMFJSBridge demo.js
//  TMF
//
//  Created by klaudz on 4/6/19.
//  Copyright (c) 2019 Tencent. All rights reserved.
//


/******************** TMFJSBridge 接入层 ********************/

// JS API 接口调用
/*
TMFJSBridge.invoke(apiName, {
    param0 : param0,            // any，参数 0
    param1 : param1,            // any，参数 1
    // ...
    paramN : paramN,            // any，参数 n
}, function (res) {
    console.log({
        ret     : res.ret,      // integer，接入层错误码，有效值：0 表示成功，1 表示接入层失败，2 表示业务层失败，-1 表示取消（部分接口有取消操作）
        errMsg  : res.errMsg,   // string，接入层错误详细信息
    });
});
*/

// JS API 事件监听
/*
document.addEventListener(eventName, function (e) {
    var res = e.tmf;
    console.log({
        param0 : res.param0,    // any，参数 0
        param1 : res.param1,    // any，参数 1
        // ...
        paramN : res.paramN,    // any，参数 n
    });
}, false);
*/

/******************** Log ********************/

function tmf_log(content, logType) {
    if (logType == 1 || logType == null) {
        console.log(content);
    } else if (logType == 2) {
        setTimeout(function() { alert(content); }, 1);
    } else {
        
    }
}

function tmf_logObject(object, logType) {
    tmf_log(JSON.stringify(object), logType);
}

/******************** 初始化 ********************/

/**
 * 当 window.onload 后，H5 容器会初始化，TMFJSBridge 会注入容器，然后触发 TMFJSBridge 初始化完毕（TMFJSBridgeReady）事件。
 * TMFJSBridge 注入是一个异步过程，因此尽可能监听该事件后，再使用 TMFJSBridge.invoke 来调用 JS API。
 */
function tmf_ready(callback) {
    if (window.TMFJSBridge) {
        callback && callback();
    } else {
        document.addEventListener("TMFJSBridgeReady", callback, false);
    }
}

/******************** 基础信息 ********************/

// 获取版本号
function tmf_getVersion() {
    TMFJSBridge.invoke('getVersion', {
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            version : res.version,  // string，当前 app 的版本号
        });
    });
}

// 获取 Build 号
function tmf_getBuild() {
    TMFJSBridge.invoke('getBuild', {
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            build : res.build,  // string，当前 app 的 build 号
        });
    });
}

// 获取 GUID
function tmf_getGUID() {
    TMFJSBridge.invoke('getGUID', {
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            guid : res.guid,    // string，当前用户的 GUID
        });
    });
}

// 获取截屏
function tmf_snapshot(range, saveToGallery, dataType, imageFormat, quality, maxWidth, maxHeight) {
    
    TMFJSBridge.invoke('snapshot', {
        range: range.length === 0 ? 'screen':range,
        saveToGallery: saveToGallery.length === 0 ? '1':saveToGallery,
        dataType: dataType.length === 0 ? 'none':dataType,
        imageFormat: imageFormat.length === 0 ? 'jpg':imageFormat,
        quality: quality.length === 0 ? '75':quality,
        maxWidth: maxWidth.length === 0 ? '0':maxWidth,
        maxHeight: maxHeight.length === 0 ? '0':maxHeight,
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            url : saveAsFile?res.fileURL:res.dataURL,
        });
    });
}

// 设置页面信息
function tmf_setPageInfo() {
    TMFJSBridge.invoke('setPageInfo', {
        reportKey   : 'home_page_1.0.0.1' // string，必选，上报标志，标识某个/些页面，reportEvent() 上报时会作为参数上报
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 设置账号
function tmf_setCustomUserInfo() {
    TMFJSBridge.invoke('setCustomUserInfo', {
        userId   :  'CustomUser_007'   // string，必选，帐号 ID
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

/******************** 页面上下文 ********************/

// 获取 H5 容器栈
function tmf_getContainerStack() {
    TMFJSBridge.invoke('getContainerStack', {
    }, function(res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            containerStack : res.containerStack,    // array，H5 容器栈，格式如下：
                                                    /*
                                                     *  [
                                                     *      { containerId: id_0, url: '...', }, // 容器 0
                                                     *      { containerId: id_1, url: '...', }, // 容器 1
                                                     *      { containerId: id_2, url: '...', }, // 容器 2
                                                     *      // ...
                                                     *      {                                   // 容器 N
                                                     *        containerId: id_n,                // integer，容器 ID
                                                     *        url: '...',                       // string，容器 URL
                                                     *      }, // 容器 N
                                                     *  ]
                                                     */
        });
    });
}

// 在新的 H5 容器打开页面
function tmf_openContainer(url) {
    TMFJSBridge.invoke('openContainer', {
        url     : url ? url : window.location.href, // string，必选，URL 地址
        params  : {                                 // dictionary，可选，参数
            defaultTitle    : 'Default Title',      // string，可选，默认标题
            showsContainerTitle  : true,            // bool，可选，是否展示页面标题
        },
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 关闭当前 H5 容器
function tmf_closeContainer() {
    TMFJSBridge.invoke('closeContainer', {
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 关闭多个 H5 容器
function tmf_closeContainers() {
    TMFJSBridge.invoke('closeContainers', {
        containerIds : [ 1, 2 ],    // array，必选，需要关闭的 H5 容器 ID 的集合
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 判断其他应用是否安装
function tmf_isAppInstalled() {
    TMFJSBridge.invoke('isAppInstalled', {
        url_ios             : 'mqqsecure://',           // string，必选（iOS），iOS app URL 地址
        packageName_android : 'com.tencent.mqqsecure'   // string，必选（Android），Android app 包名
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            url         : res.url,          // string，实际判断的 URL 地址
            installed   : res.installed,    // bool，是否安装了 app
        });
    });
}

// 启动其他应用
function tmf_openApp() {
    TMFJSBridge.invoke('openApp', {
        url_ios             : 'mqqsecure://',           // string，必选（iOS），iOS app URL 地址
        url_android         : 'mqqsecure://',           // string，必选（Android），Android app URL 地址
        packageName_android : 'com.tencent.mqqsecure'   // string，必选（Android），Android app 包名
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            url         : res.url,          // string，实际打开的 URL 地址
            packageName : res.packageName,  // string，实际打开的包名
            error       : res.error,        // integer，错误码
        });
    });
}

// 跳转原生页面
function tmf_routerURL() {
    TMFJSBridge.invoke('routerURL', {
        url     : 'tmf://router/test/1/push', // string，必选，原生页面 URL 地址
        params  : {                                 // dictionary，可选，参数
            titleStr    : 'H5通过JSAPI跳转',      // string，可选，默认标题
        },
    }, function (res) {
//        tmf_logObject(res, 2);
    });
}

/******************** 基础事件 ********************/

// 页面出现
function tmf_onContainerAppear() {
    document.addEventListener('containerAppear', function (e) {
        var res = { };
        res.eventName = 'containerAppear';
        res.tmf = e.tmf;
        tmf_logObject(res, 2);
        tmf_logObject(res);
    }, false);
}

// 页面消失
function tmf_onContainerDisappear() {
    document.addEventListener('containerDisappear', function (e) {
        var res = { };
        res.eventName = 'containerDisappear';
        res.tmf = e.tmf;
        tmf_logObject(res, 2);
        tmf_logObject(res);
    }, false);
}

/******************** 网络接口 ********************/

// 获取网络状态
function tmf_getNetworkType() {
    TMFJSBridge.invoke('getNetworkType', {
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            networkType : res.networkType,    // string，网络类型，有效值：'none'，'cellular'，'wifi'
        });
    });
}

/******************** 地理位置 ********************/

// 获取当前地理位置
function tmf_getLocation() {
    TMFJSBridge.invoke('getLocation', {
        type : 'wgs84', // string，必选，坐标类型，有效值：'wgs84' 表示真实坐标，'gcj02' 表示火星坐标
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            latitude    : res.latitude,     // double，纬度，范围为 90 ~ -90
            longitude   : res.longitude,    // double，经度，范围为 180 ~ -180
            speed       : res.speed,        // double，速度，单位为 m/s
            accuracy    : res.accuracy,     // double，位置精度
        });
    });
}

// 使用内置地图查看位置
function tmf_openLocation() {
    TMFJSBridge.invoke('openLocation', {
        latitude    : 22.5403145,   // double，必选，纬度，范围为 90 ~ -90
        longitude   : 113.874348,   // double，必选，经度，范围为 180 ~ -180
        name        : '',           // string，可选，位置名
        address     : '',           // string，可选，地址详情说明
        scale       : 1.0,          // double，可选，地图缩放级别，默认为 1.0
    }, function (res) {
        tmf_logObject(res, 2);
    });
}

/******************** 分享 ********************/

// 自定义 app 分享内容
function tmf_updateShareData() {
    TMFJSBridge.invoke('updateShareData', {
        appKeys : [ 'wx_message', 'wx_timeline', 'qq_message' ],    // array，必选，分享的 app 组合，'wx_message' 表示微信消息，'wx_timeline' 表示微信朋友圈，'qq_message' 表示 QQ 消息，'alipay_message' 表示支付宝消息，'weibo_timeline' 表示微博, 'sms' 表示短信消息, 'dding_message' 表示钉钉消息
        title   : 'title',                                          // string，可选，分享标题
        desc    : 'description',                                    // string，可选，分享描述
        link    : 'http://www.qq.com',                              // string，可选，分享链接
        imgUrl  : '',                                               // string，可选，分享图标
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            error : res.error,  // integer，错误码
        });
    });
}

// 分享到 app
function tmf_shareApp(appkey) {
    TMFJSBridge.invoke('shareApp', {
        appKeys         : appkey,                   // string，必选，分享的 app，'wx_message' 表示微信消息，'wx_timeline' 表示微信朋友圈，'qq_message' 表示 QQ 消息，'alipay_message' 表示支付宝消息，'weibo_timeline' 表示微博, 'sms' 表示短信消息, 'dding_message' 表示钉钉消息
        title           : 'title',                  // string，可选，分享标题
        desc            : 'description',            // string，可选，分享描述
        link            : 'http://www.qq.com',      // string，可选，分享链接
        imgUrl          : '',                       // string，可选，分享图标
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            error : res.error,  // integer，错误码
        });
    });
}


/******************** 界面 ********************/

// 设置标题及相关属性
function tmf_setTitle() {
    let obj = {name:"zhangsan", sex:1, age:18};
    TMFJSBridge.invoke('setTitle', {
        par         : obj,
        title       : 'Title',      // string，可选，标题
        titleColor  : '#666666',    // string，可选，标题颜色
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 选择列表
function tmf_actionSheet() {
    TMFJSBridge.invoke('actionSheet', {
        title               : 'Title',                          // string，可选，标题
        message             : 'Message',                        // string，可选，信息
        buttons             : [ 'Eat', 'Drink', 'Play' ],       // array，必选，按钮列表
        cancelButton        : 'Cancel',                         // string，可选，取消按钮
        destructiveButton   : 'Delete',                         // string，可选，删除按钮（iOS 特有，红色按钮）
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            clickedButtonIndex  : res.clickedButtonIndex,   // integer，点击按钮的索引，cancelButton 为 0，buttons 从 1 开始从上往下递增，destructiveButton 为 -1
        });
    });
}

// 警告框
function tmf_alert() {
    TMFJSBridge.invoke('alert', {
        title       : 'Title',      // string，可选，标题
        message     : 'Message',    // string，可选，信息
        okButton    : 'OK',         // string，必选，确定按钮
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            clickedButtonIndex  : res.clickedButtonIndex,   // integer，点击按钮的索引，okButton 为 1
        });
    });
}

// 确认框
function tmf_confirm() {
    TMFJSBridge.invoke('confirm', {
        title           : 'Title',      // string，可选，标题
        message         : 'Message',    // string，可选，信息
        okButton        : 'OK',         // string，必选，确定按钮
        cancelButton    : 'Cancel',     // string，必选，取消按钮
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            clickedButtonIndex  : res.clickedButtonIndex,   // integer，点击按钮的索引，cancelButton 为 0，okButton 为 1
        });
    });
}

// 设置右上角按钮
function tmf_setOptionMenu() {
    TMFJSBridge.invoke('setOptionMenu', {
        title       : 'More',       // string，可选，标题
        titleColor  : '#666666',    // string，可选，标题颜色
        items       :[
                      {
            index       : 0 ,
            title       : '关闭网页（set）',   // string，可选，标题
            titleColor  : '#123456',   // string，可选，标题颜色
            action      : function(res){
                //alert(JSON.stringify(res));
                tmf_closeContainer();
            }
        },
                      {
            index       : 1 ,
            title       : 'tip1',   // string，可选，标题
            titleColor  : '#345678',   // string，可选，标题颜色
            action      : function(res){
                alert(JSON.stringify(res));
            }
        },
                      {
            index       : 2 ,
            title       : 'tip2',
            titleColor  : '#56789a',
            action      : function(res){
                alert(JSON.stringify(res));
            }
        }
                      ]
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 显示右上角按钮
function tmf_showOptionMenu() {
    TMFJSBridge.invoke('showOptionMenu', {
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 隐藏右上角按钮
function tmf_hideOptionMenu() {
    TMFJSBridge.invoke('hideOptionMenu', {
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

/******************** 工具类 ********************/

// 获取共享的 KV 数据
// iOS 为 NSUserDefaults
function tmf_getSharedStorage() {
    TMFJSBridge.invoke('getSharedStorage', {
        //key   : 'key1',               // string，必选，KV 的 key（Deprecated）
        keys    : [ 'key1', 'key2' ],   // array，必选，支持传入多组 KV 的 keys
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            //value : res.value,        // any，KV 的 value
            values  : res.values,       // dictionary，多组 KV 的 keys 和 values，格式如下：
                                        /*
                                         *  { 
                                         *    key1: value1, // any，KV 1 的 value
                                         *    key1: value1, // any，KV 2 的 value
                                         *    // ...
                                         *    keyN: valueN, // any，KV N 的 value
                                         *  }
                                         */
            value1  : res.values.key1,
            value2  : res.values.key2,
        });
    });
}

// 设置共享的 KV 数据
// iOS 为 NSUserDefaults
function tmf_setSharedStorage() {
    TMFJSBridge.invoke('setSharedStorage', {
        //key   : 'key1',   // string，必选，KV 的 key（Deprecated）
        //value : 'value1', // string，必选，KV 的 value（Deprecated）
        values  : {         // dictionary，必选，支持传入多组 KVs
            key1 : "value1",
            key2 : "value2",
        }
    }, function (res) {
        tmf_logObject(res, 2);
    });
}

// 移除共享的 KV 数据
// iOS 为 NSUserDefaults
function tmf_removeSharedStorage() {
    TMFJSBridge.invoke('removeSharedStorage', {
        //key : 'key1',                 // string，必选，KV 的 key（Deprecated）
        keys  : [ 'key1', 'key2' ],     // array，必选，支持传入多组 KV 的 keys
    }, function (res) {
        tmf_logObject(res, 2);
    });
}

// 上报埋点
function tmf_reportEvent() {
    // 接口上报时会带上以下参数
    //  1. setPageInfo() 中设定的 reportKey
    TMFJSBridge.invoke('reportEvent', {
        event   : '',   // string，必选，事件名
        params  : {     // dictionary，必选，上报的 KV 值
            // ...
        },
    }, function (res) {
        //tmf_logObject(res, 2);
    });
}

// 网关接口调用（RPC）
function tmf_sendShark() {
    TMFJSBridge.invoke('sendShark', {
        apiName : "TMFEcho",                                // string，必选，Shark 命令字（API name）
        //cmdId : 998,                                      // integer，可选，Shark 命令字（Cmd ID）（Deprecated）
        headers : { 'headers_key1' : 'headers_value1' },    // dictionary，可选，业务自定义的请求头部
        cookies : { 'cookies_key1' : 'cookies_value1' },    // dictionary，可选，业务自定义的请求 Cookies
        queries : { 'queries_key1' : 'queries_value1' },    // dictionary，可选，业务自定义的请求 Queries
        data    : 'test',                                   // string，可选，业务自定义的请求数据
        timeout : 10000,                                    // integer，可选，超时时间，单位 ms
    }, function (res) {
        tmf_logObject(res, 2);
        tmf_logObject({
            error               : res.error,                // integer，业务的返回错误码，没有错误时为 undefined
            errorDescription    : res.errorDescription,     // string，业务的返回错误描述，没有错误时为 undefined
            httpResponseCode    : res.httpResponseCode,     // integer，业务返回的 HTTP 状态码
            headers             : res.headers,              // dictionary，业务的返回头部
            data                : res.data,                 // string，业务的返回数据
        });
    });
}


// QMUI
function tmf_showToast(){
    TMFJSBridge.invoke("showToast", {
        title: "hello",
        detail:"TMFDemo",
        duration:2.0
    }, function(res){
        
    });
}

function tmf_showLoading(){
    TMFJSBridge.invoke("showLoading", {
        title: "TMFDemo",
        detail:"loading...",
    }, function(res){
        setTimeout(function(){
            TMFJSBridge.invoke("hideLoading")
        }, 3000);
    });
}

function tmf_unInvoke(){
    TMFJSBridge.invoke("unInvoke", {
    }, function(res){
        tmf_logObject(res, 2);
    });
}
