#import "TMFPoseCameraViewController.h"

#import <TMFLiveDetection/TMFLiveDetection.h>
#import "NSString+Localize.h"

@interface TMFPoseCameraViewController () <TMFLiveCameraVideoDeviceDelegate, TMFPoseDetectorDelegate>
@property (weak, nonatomic) IBOutlet UIView *contentView;
@property (weak, nonatomic) IBOutlet UIImageView *previewView;
@property (weak, nonatomic) IBOutlet UIImageView *maskView;  //用于显示人脸五官位置和人脸框
@property (weak, nonatomic) IBOutlet UIButton *btnStart;
@property (weak, nonatomic) IBOutlet UIButton *btnStop;
@property (weak, nonatomic) IBOutlet UIButton *btnOpenGL;
@property (weak, nonatomic) IBOutlet UIButton *btnGLView;
@property (weak, nonatomic) IBOutlet UIButton *btnTips;
@property (nonatomic, strong) TMFLiveCameraVideoDevice *cameraDevice;
@property (nonatomic, strong) CIDetector *sysDetector;

@property (nonatomic, assign) BOOL increasing;
@property (nonatomic, assign) CGFloat curRed;

@property (nonatomic, strong) UILabel *label;

@property (strong, nonatomic) IBOutlet UIButton *option1button;
@property (strong, nonatomic) IBOutlet UIButton *option2button;


@property (strong) TMFPoseDetector *poseDetector;

@property (nonatomic, assign) BOOL called;

@end


@implementation TMFPoseCameraViewController

- (void)dealloc {
    [self.cameraDevice stopSession];
    //调用JSAPI回调函数
    if (!self.called && self.delegate) {
        [self callbackJsapiWithValues:NO message:@"cancel"];
    }
}

- (id)init
{
    NSString *xibName =  @"TMFPoseCameraViewController-568h";
    self = [super initWithNibName:xibName bundle:nil];
    if (self) {
        self.poseDetector = [[TMFPoseDetector alloc] init];
        self.poseDetector.delegate = self;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = [@"live detection-Pose" localizedString];
    
    BOOL authRet = [self.poseDetector preparePoseConfiguration];
    if (!authRet) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        UIAlertView *alert =[[UIAlertView alloc]initWithTitle:@"tips" message:[NSString stringWithFormat:@"auth check failed!!!"] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        [alert show];
#pragma clang diagnostic pop
        return;
    }
    else {
        NSLog(@"Configuration auth check pass.");
    }
    
    self.cameraDevice = [[TMFLiveCameraVideoDevice alloc] initWithPreviewView:self.previewView position:AVCaptureDevicePositionFront];
    self.cameraDevice.delegate = self;
    
    [self onClickBlink:nil];
    
    [self onClickStartButton:nil];
    
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (IBAction)onClickStartButton:(id)sender {
    [self.cameraDevice startSession];
}

- (IBAction)onClickStopButton:(id)sender {
    [self.cameraDevice stopSession];
}

- (IBAction)onClickSwitchButton:(id)sender {
    [self.cameraDevice switchCapturePosition];
}
- (IBAction)onClickBlink:(id)sender {
    [self.poseDetector setPoseType:LIVE_POSETYPE_BLINK_EYE];
    [self.btnTips setTitle:[@"please blink" localizedString] forState:UIControlStateNormal];
}
- (IBAction)onClickOpenMouth:(id)sender {
    [self.poseDetector setPoseType:LIVE_POSETYPE_OPEN_MOUSE];
    [self.btnTips setTitle:[@"please open mouth" localizedString] forState:UIControlStateNormal];
}
- (IBAction)onClickSafetyRecommand:(id)sender {
    [self.poseDetector setSafety:LIVE_SAFETY_LEVEL_RECOMMAND];
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    UIAlertView *alert=[[UIAlertView alloc]initWithTitle:@"SAFETY_RECOMMAND" message:@"" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
    [alert show];
#pragma clang diagnostic pop
}
- (IBAction)onClickSafetyHigh:(id)sender {
    [self.poseDetector setSafety:LIVE_SAFETY_LEVEL_HIGH];
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    UIAlertView *alert=[[UIAlertView alloc]initWithTitle:@"SAFETY_HIGH" message:@"" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
    [alert show];
#pragma clang diagnostic pop
}
- (IBAction)onClickSafetyLow:(id)sender {
    [self.poseDetector setSafety:LIVE_SAFETY_LEVEL_LOW];
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    UIAlertView *alert=[[UIAlertView alloc]initWithTitle:@"SAFETY_LOW" message:@"" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
    [alert show];
#pragma clang diagnostic pop
}

- (void)drawFaceImageWithImage:(UIImage*)image rect:(CGRect)rect feature:(NSMutableArray *)featPointArray
{
    CGSize imageSize = image.size;
    //UIGraphicsBeginImageContextWithOptions(imageSize,YES,1.0);
    UIGraphicsBeginImageContextWithOptions(imageSize,NO,1.0);
    [image drawInRect:CGRectMake(0, 0, imageSize.width, imageSize.height)];
    
    //step1: clear all content
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
    CGContextFillRect(context, CGRectMake(0, 0, imageSize.width, imageSize.height));
    
    CGContextSetStrokeColorWithColor(context, [UIColor redColor].CGColor);
    CGContextSetFillColorWithColor(context, [UIColor whiteColor].CGColor);
    if (imageSize.height > 0 && imageSize.width > 0) {
        
        CGContextMoveToPoint(context, rect.origin.x, rect.origin.y);
        CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y + rect.size.height);
        CGContextStrokePath(context);
        CGContextMoveToPoint(context, rect.origin.x, rect.origin.y);
        CGContextAddLineToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y);
        CGContextStrokePath(context);
        CGContextMoveToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y + rect.size.height);
        CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y + rect.size.height);
        CGContextStrokePath(context);
        CGContextMoveToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y + rect.size.height);
        CGContextAddLineToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y);
        CGContextStrokePath(context);
    }
    //draw points
    for (NSValue *ptValue in featPointArray) {
        CGPoint pt = [ptValue CGPointValue];
        CGContextFillRect(context, CGRectMake(pt.x - 1, pt.y - 1, 3, 3));
    }
    
    UIImage *o_image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    dispatch_async(dispatch_get_main_queue(), ^{
        UIImage *flip_image = [UIImage imageWithCGImage:o_image.CGImage scale:o_image.scale orientation:UIImageOrientationUpMirrored];
        [self.maskView setImage:flip_image];
        //NSLog(@"endTime");
    });
}


//进行人脸检测，并且把检测到的人脸框和五官画到画面上
- (void)doDetectFace:(CMSampleBufferRef)buffer
{
    [self.poseDetector detectFace:buffer];
}

- (void)updateLabel:(NSString *)score
{
    self.label.text = [NSString stringWithFormat:@"%@ %@",[@"Lighting conditions (1.0 is the best):" localizedString], score];
}

- (UIImage *) imageFromSampleBuffer:(CMSampleBufferRef) sampleBuffer {
    // 为媒体数据设置一个CMSampleBuffer的Core Video图像缓存对象
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    // 锁定pixel buffer的基地址
    CVPixelBufferLockBaseAddress(imageBuffer, 0);
    
    // 得到pixel buffer的基地址
    void *baseAddress = CVPixelBufferGetBaseAddress(imageBuffer);
    
    // 得到pixel buffer的行字节数
    size_t bytesPerRow = CVPixelBufferGetBytesPerRow(imageBuffer);
    // 得到pixel buffer的宽和高
    size_t width = CVPixelBufferGetWidth(imageBuffer);
    size_t height = CVPixelBufferGetHeight(imageBuffer);
    
    // 创建一个依赖于设备的RGB颜色空间
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    
    // 用抽样缓存的数据创建一个位图格式的图形上下文（graphics context）对象
    CGContextRef context = CGBitmapContextCreate(baseAddress, width, height, 8,
                                                 bytesPerRow, colorSpace, kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst);
    // 根据这个位图context中的像素数据创建一个Quartz image对象
    CGImageRef quartzImage = CGBitmapContextCreateImage(context);
    // 解锁pixel buffer
    CVPixelBufferUnlockBaseAddress(imageBuffer,0);
    
    // 释放context和颜色空间
    CGContextRelease(context);
    CGColorSpaceRelease(colorSpace);
    
    // 用Quartz image创建一个UIImage对象image
    UIImage *image = [UIImage imageWithCGImage:quartzImage];
    
    // 释放Quartz image对象
    CGImageRelease(quartzImage);
    
    return (image);
}

#pragma mark - TMFLiveCameraVideoDeviceDelegate
- (void)cameraDeviceEvent:(TMFLiveCameraDeviceEvent)event withAguments:(NSDictionary *)args
{
    switch (event) {
        case TMFLiveCameraDeviceEvent_FrameReceived:
        {
            //NSLog(@"startTime");
            CMSampleBufferRef buffer = (CMSampleBufferRef)[[args objectForKey:@"buffer"] integerValue];
            [self doDetectFace:buffer];
        }
            break;
            
        default:
            break;
    }
}

#pragma mark - TMFPoseCameraViewModelControllerDelegate
- (void)poseDetector:(TMFPoseDetector *)poseDetector faceImageWithImage:(nullable UIImage *)faceImage rect:(CGRect)rect feature:(NSMutableArray *)featPointArray {
    [self drawFaceImageWithImage:faceImage rect:CGRectZero feature:featPointArray];
}

- (void)poseDetector:(TMFPoseDetector *)poseDetector faceDidFinishWithError:(nullable NSError *)error{
    NSString *strTips = @"check pass";
    if (error) {
        strTips = [NSString stringWithFormat:@"network failed (%ld)", (long)error.code];
        [self callbackJsapiWithValues:NO message:@"fail"];
    } else {
        [self callbackJsapiWithValues:YES message:@"ok"];
    }
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    UIAlertView *alert = [[UIAlertView alloc]initWithTitle:strTips message:nil delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
    [alert show];
#pragma clang diagnostic pop
    [self.navigationController popViewControllerAnimated:YES];
}

//通过delegate回调JSApi接口，把检测结果会传给JSAPI
- (void)callbackJsapiWithValues:(BOOL)pass message:(NSString*)message {
    if (!self.called && self.delegate) {
        self.called = YES;
        [self.delegate poseViewController:self didCompleteWithResult:pass message:message];
    }
}

@end
