
//
//  Created by CosperYu on 2019/03/02.
//  Copyright (c) 2019年 Tencent Youtu. All rights reserved.
//

#import "TMFReflectCameraViewController.h"

#import <TMFLiveDetection/TMFLiveDetection.h>

//weak strong
#ifndef theWeakRef
#define theWeakRef(weakVar, strongVar) __weak __typeof(&*strongVar) weakVar = strongVar
#endif
#ifndef Weak_Self
#define Weak_Self() theWeakRef(weakSelf, self)
#endif
#ifndef theStrongRef
#define theStrongRef(strongVar, weakVar) __strong __typeof(&*weakVar) strongVar = weakVar
#endif
#ifndef Strong_Self
#define Strong_Self() theStrongRef(strongSelf,weakSelf)
#endif

#define isIphoneX ({\
int tmp = 0;\
if (@available(iOS 11.0, *)) {\
if (!UIEdgeInsetsEqualToEdgeInsets([UIApplication sharedApplication].delegate.window.safeAreaInsets, UIEdgeInsetsZero)) {\
tmp = 1;\
}else{\
tmp = 0;\
}\
}else{\
tmp = 0;\
}\
tmp;\
})


@interface TMFReflectCameraViewController () <TMFLiveCameraDeviceDelegate, UIAlertViewDelegate, TMFReflectDelegate>
//绘制人脸
@property (weak, nonatomic) IBOutlet UIView *contentView;
//用于控制屏幕颜色变化
@property (weak, nonatomic) IBOutlet UIView *shapeView;

//灰色背景板
@property (weak, nonatomic) IBOutlet UIImageView *backViewBg;
//前置人脸边框
@property (weak, nonatomic) IBOutlet UIImageView *backView;
//人脸红色外框
@property (weak, nonatomic) IBOutlet UIImageView *faceRangeView;

//返回按钮
@property (weak, nonatomic) IBOutlet UIButton *btnBack;
//开始按钮
@property (weak, nonatomic) IBOutlet UIButton *btnBegin;

@property (weak, nonatomic) IBOutlet UILabel * tipsLabel;

@property (nonatomic, weak) IBOutlet UILabel *lux_label;

//状态控制
@property (nonatomic, assign) LivenessDetectionState curState;
@property (nonatomic, assign) FacePreviewingAdvise currentAdvise;
//记录屏幕亮度
@property (nonatomic, assign) CGFloat recordBrightness;

@property (nonatomic, strong) UIImage * inRectImg;
@property (nonatomic, strong) UIImage * outRectImg;

//持有摄像头设备
@property (nonatomic, strong) TMFLiveCameraRecordDevice *device;

@property (nonatomic, strong) TMFReflectDetector *reflectDetector;

@property (nonatomic, assign) BOOL called;

@end

@implementation TMFReflectCameraViewController


#pragma mark - Life Circle

- (id)init
{
    NSString *xibName =  @"TMFReflectCameraViewController-568h";
    self = [super initWithNibName:xibName bundle:nil];
    if (self) {
        self.reflectDetector = [[TMFReflectDetector alloc] init];
        self.reflectDetector.delegate = self;
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"活体识别-反光";
    
    //检查授权文件
    BOOL authRet = [self.reflectDetector prepareReflectConfiguration];
    if (!authRet) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        UIAlertView *alert =[[UIAlertView alloc]initWithTitle:@"tips" message:[NSString stringWithFormat:@"auth check failed!!!"] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        [alert show];
#pragma clang diagnostic pop
        return;
    }
    else {
        NSLog(@"Configuration auth check pass.");
    }
    
    //启动设备
    self.device = [[TMFLiveCameraRecordDevice alloc] initWithPosition:AVCaptureDevicePositionFront];
    self.device.delegate = self;
    
    //init device with auto
    self.device.focusMode = AVCaptureFocusModeContinuousAutoFocus;
    self.device.exposureMode = AVCaptureExposureModeContinuousAutoExposure;
    self.device.whiteBalanceMode = AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance;
    
    //界面设置
    self.lux_label.layer.cornerRadius = 8;
    self.lux_label.layer.masksToBounds = true;
    
    [self.contentView setContentMode:UIViewContentModeScaleToFill];
    [self.contentView.layer addSublayer:self.device.previewLayer];
    
    self.tipsLabel.layer.cornerRadius = 20.0;//2.0是圆角的弧度，根据需求自己更改
    [self.tipsLabel.layer setMasksToBounds:YES];
    
    if (isIphoneX) {
        self.inRectImg = [UIImage imageNamed:@"faceDetect_x"];
        self.outRectImg = [UIImage imageNamed:@"faceNotDetect_x"];
        self.backViewBg.image = [UIImage imageNamed:@"wbcf_face_detect_x"];
    }
    else {
        self.inRectImg = [UIImage imageNamed:@"faceDetect"];
        self.outRectImg = [UIImage imageNamed:@"faceNotDetect"];
        self.backViewBg.image = [UIImage imageNamed:@"wbcf_face_detect"];
    }
    self.backView.image = self.outRectImg;
}

- (void)dealloc
{
    NSLog(@"TMFReflectCameraViewController dealloc");
    [self.device stopDevice];
    //调用JSAPI回调函数
    if (!self.called && self.delegate) {
        [self callbackJsapiWithValues:NO message:@"cancel"];
    }
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    //启动摄像头
    [self.device startRecord];
    self.device.delegate = self;
    
    //设置preview的frame
    self.device.previewLayer.frame = CGRectMake(0, 0, CGRectGetWidth(self.contentView.bounds), CGRectGetHeight(self.contentView.bounds));
    
    //开始监控光线强度
    [self.device startDetectBrightness_showLux:^(float value) {
        dispatch_async(dispatch_get_main_queue(), ^{
            self.lux_label.text = [NSString stringWithFormat:@"lux: %f",value];
        });
    }];
    
    //记录并设置屏幕亮度
    self.recordBrightness = [UIScreen mainScreen].brightness;
    self.shapeView.backgroundColor = [UIColor clearColor];
    [UIScreen mainScreen].brightness = 1;
    
    [self.reflectDetector viewWillAppear];
    
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    
    //停止检测光线强度，不执行这里会造成viewController不会释放，因为viewcontroller的block被持有
    [self.device stopDetectBrightness_showLux];
    
    //停止摄像头
    [self.device stopRecord];
    self.device.delegate = nil;
    
    //终止反光过程
    [self.reflectDetector viewDidDisappear];
    
    //恢复屏幕亮度
    [UIScreen mainScreen].brightness = self.recordBrightness;
}


#pragma mark - IBAction

- (void)stage1BtnPressed:(id)sender {
    
}

//返回
- (IBAction)handleBackEvent:(UIButton *)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
#pragma clang diagnostic ignored "-Wdeprecated-implementations"
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
#pragma clang diagnostic pop
{
    [self handleBackEvent:nil];
}


#pragma mark - UI Controller
- (void)setTips:(NSString*)strTips
{
    self.tipsLabel.text = strTips;
}

//显示界面提示语信息
- (void)setAdvisedShow:(FacePreviewingAdvise)advise
{
    if (self.currentAdvise == advise) {
        return;
    }
    self.currentAdvise = advise;
    dispatch_async(dispatch_get_main_queue(), ^{
        if (advise == ADVISE_PASS) {
            self.backView.image = self.inRectImg;
            self.tipsLabel.text = @"Keep your face on the screen.";
            UIColor *color = [UIColor colorWithRed:64/255.0 green:158/255.0 blue:255/255.0 alpha:1.0];
            self.tipsLabel.textColor = color;
            self.btnBegin.enabled = YES;
            self.btnBegin.backgroundColor = color;
        }
        else {
            self.backView.image = self.outRectImg;
            UIColor *color = [UIColor colorWithRed:233/255.0 green:75/255.0 blue:44/255.0 alpha:1.0];
            self.tipsLabel.textColor = color;
            self.btnBegin.enabled = NO;
            self.btnBegin.backgroundColor = [UIColor grayColor];
            switch (advise) {
                case ADVISE_TOO_FAR:
                    self.tipsLabel.text = @"Be closer";
                    break;
                case ADVISE_TOO_CLOSE:
                    self.tipsLabel.text = @"Be farer";
                    break;
                case ADVISE_NOT_IN_RECT:
                    self.tipsLabel.text = @"Keep your face in the rect.";
                    break;
                case ADVISE_NO_FACE:
                    self.tipsLabel.text = @"Keep your face on the screen.";
                    break;
                case ADVISE_INCORRECT_POSTURE:
                    self.tipsLabel.text = @"Keep your face on the right position.";
                    break;
                case ADVISE_CLOSE_EYE:
                    self.tipsLabel.text = @"Open your eye.";
                    break;
                default:
                    self.tipsLabel.text = [NSString stringWithFormat:@"input error: %d",advise];
                    break;
            }
        }
    });
}

- (void)drawFaceImageWithImage:(CGRect)rect feature:(CGPoint)pt{
    self.faceRangeView.hidden = NO;
    
    rect.origin.y -= CGRectGetHeight(self.contentView.bounds)/4;
    pt.y -= CGRectGetHeight(self.contentView.bounds)/4;
    CGSize imageSize = CGSizeMake(540, 960);
    //UIGraphicsBeginImageContextWithOptions(imageSize,YES,1.0);
    UIGraphicsBeginImageContextWithOptions(imageSize,NO,1.0);
    //    [image drawInRect:CGRectMake(0, 0, imageSize.width, imageSize.height)];
    
    //step1: clear all content
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
    CGContextFillRect(context, CGRectMake(0, 0, imageSize.width, imageSize.height));
    
    CGContextSetStrokeColorWithColor(context, [UIColor redColor].CGColor);
    CGContextSetFillColorWithColor(context, [UIColor whiteColor].CGColor);
    
    if (imageSize.height > 0 && imageSize.width > 0) {
        
        CGContextMoveToPoint(context, rect.origin.x, rect.origin.y);
        CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y + rect.size.height);
        CGContextStrokePath(context);
        CGContextMoveToPoint(context, rect.origin.x, rect.origin.y);
        CGContextAddLineToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y);
        CGContextStrokePath(context);
        CGContextMoveToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y + rect.size.height);
        CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y + rect.size.height);
        CGContextStrokePath(context);
        CGContextMoveToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y + rect.size.height);
        CGContextAddLineToPoint(context, (rect.origin.x + rect.size.width), rect.origin.y);
        CGContextStrokePath(context);
    }
    
    //draw points
    CGContextFillRect(context, CGRectMake(pt.x - 1, pt.y - 1, 3, 3));
    
    //    UIImage *o_image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    dispatch_async(dispatch_get_main_queue(), ^{
        //          UIImage *flip_image = [UIImage imageWithCGImage:o_image.CGImage scale:o_image.scale orientation:UIImageOrientationUpMirrored];
        //        [faceRangeView setImage:o_image];
    });
}

#pragma mark - CameraDeviceDelegate
- (void)cameraDeviceEvent:(TMFLiveCameraDeviceEvent)event withAguments:(NSDictionary *)args
{
    CMSampleBufferRef buffer = (CMSampleBufferRef)[[args objectForKey:@"buffer"] integerValue];
    //if you need uiimage
    //    UIImage *faceImage = [UIImage yt_imageFromSampleBuffer:buffer];
    [self.reflectDetector cameraDeviceEvent:event sampleBuffer:buffer];
    
}

#pragma mark - TMFReflectDelegate
/* 检测到相机的焦点位置 */
- (void)reflectDetector:(TMFReflectDetector *)reflect foucsPoint:(CGPoint)point {
    self.device.focusPoint = point;
    self.device.exposurePoint = point;
}

/* 检测到人脸屏幕坐标 */
- (void)reflectDetector:(TMFReflectDetector *)reflectDetector faceImageWithRect:(CGRect)rect feature:(CGPoint)point {
    //在屏幕绘制人脸红色框
    Weak_Self();
    dispatch_async(dispatch_get_main_queue(), ^{
        Strong_Self();
        if (strongSelf) {
            [strongSelf drawFaceImageWithImage:rect feature:(CGPoint)point];
        }
    });
}

/* 检测到人脸状态结果 */
- (void)reflectDetector:(TMFReflectDetector *)reflectDetector facePreviewAdvise:(FacePreviewingAdvise)advise {
    [self setAdvisedShow:advise];
    
    if (advise == ADVISE_INCORRECT_POSTURE) {
        dispatch_async(dispatch_get_main_queue(), ^{
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
            UIAlertView *alert =[[UIAlertView alloc]initWithTitle:@"tips" message:@"Keep your face on the right position." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
            [alert show];
#pragma clang diagnostic pop
        });
    }
    else if (advise == ADVISE_NO_FACE) {
        //隐藏人脸边框
        dispatch_async(dispatch_get_main_queue(), ^{
            self.faceRangeView.hidden = YES;
        });
    }
}

/* 反光活体识别流程开始 */
- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageWillStart:(LivenessDetectionState)state {
    
}

- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageDidStarted:(NSError *)error{
    if (error) {
        NSString *strTips = nil;
        switch (error.code) {
            case NETRESPONSE_CONFIG_ERROR:
                strTips = @"response config data parse error.";
                break;
            default:
                strTips = [NSString stringWithFormat:@"network failed (%ld)", (long)error.code];
                break;
        }
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:strTips message:@"" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        [alert show];
#pragma clang diagnostic pop
    }
    
}

- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageWithBrightnessDidStarted:(NSString *)Json {
    
}

- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageWillFinish:(LivenessDetectionState)state {
    //设置tips
    [self setTips:@"Hold your position"];
}

- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageDidFailed:(NSError *)error {
    self.view.userInteractionEnabled = YES;
    
    //解锁相机对焦和白平衡参数，到自动调整模式
    self.device.focusMode = AVCaptureFocusModeContinuousAutoFocus;
    self.device.exposureMode = AVCaptureExposureModeContinuousAutoExposure;
    self.device.whiteBalanceMode = AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance;
    
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
    UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"tips" message:[NSString stringWithFormat:@"reflect finish with error: %ld",(long)error.code] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
    [alert show];
#pragma clang diagnostic pop
    [self callbackJsapiWithValues:NO message:@"fail"];
}

- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageDidFinishWithError:(nullable NSError *)error{
    self.view.userInteractionEnabled = YES;
    
    //解锁相机对焦和白平衡参数，到自动调整模式
    self.device.focusMode = AVCaptureFocusModeContinuousAutoFocus;
    self.device.exposureMode = AVCaptureExposureModeContinuousAutoExposure;
    self.device.whiteBalanceMode = AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance;
    
    //重新初始化变量
    [self.reflectDetector reset];
    self.btnBegin.hidden = NO;
    NSString *strTips = nil;
    
    if (!error) {
        strTips = @"check pass";
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:strTips message:@"" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        [alert show];
#pragma clang diagnostic pop
        [self callbackJsapiWithValues:YES message:@"ok"];
    } else {
        switch (error.code) {
            case REFLECT_NOT_FOUND_FACE:
                strTips = @"check failed (压入的反光图片流中检测不到人脸)";
                break;
            default:
                strTips = [NSString stringWithFormat:@"check failed (%ld)", (long)error.code];
                break;
        }
        [self.delegate reflectViewController:self didCompleteWithResult:YES message:strTips];
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"net connect failed, please try again later." message:strTips delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        [alert show];
#pragma clang diagnostic pop
        [self callbackJsapiWithValues:YES message:@"fail"];
    }
}

- (void)reflectDetector:(TMFReflectDetector *)reflectDetector dealStageDidFinished:(LivenessDetectionState)state {
    //锁定相机白平衡等基本参数
    if (self.device.focusMode != AVCaptureFocusModeLocked) {
        self.device.focusMode = AVCaptureFocusModeLocked;
    }
    if (self.device.exposureMode != AVCaptureExposureModeLocked) {
        self.device.exposureMode = AVCaptureExposureModeLocked;
    }
    if (self.device.whiteBalanceMode != AVCaptureWhiteBalanceModeLocked) {
        self.device.whiteBalanceMode = AVCaptureWhiteBalanceModeLocked;
    }
    
    NSLog(@"---->start agreflect");
}

- (nonnull TMFLiveCameraRecordDevice *)getCurrentCameraDevice {
    return self.device;
}

- (nonnull id)getCurrentShapeView {
    return self.shapeView;
}
//通过delegate回调JSApi接口，把检测结果会传给JSAPI
- (void)callbackJsapiWithValues:(BOOL)pass message:(NSString*)message {
    if (!self.called && self.delegate) {
        self.called = YES;
        [self.delegate reflectViewController:self didCompleteWithResult:pass message:message];
    }
}

@end
