//
//  AMapViewController.m
//  TMFDemo
//
//  Created by hauzhong on 2019/4/30.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "AMapViewController.h"

#import "NSObject+MQQValueExtended.h"

#import <MAMapKit/MAMapKit.h>

#import <QMUIKit/QMUIKit.h>
#import "TMFLocationManager.h"

@interface AMapViewController () <MAMapViewDelegate,TMFLocationManagerDelegate>
@property (nonatomic, strong) MAMapView *mapView;
@property (nonatomic, strong) TMFLocationManager *locationManager;
@property (nonatomic, strong) QMUITips *tipsView;
@end

@implementation AMapViewController

- (void)dealloc {
    self.tipsView = nil;
    self.locationManager = nil;
    self.parameters = nil;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.title = @"AMap(高德地图)";
    
    self.locationManager = [[TMFLocationManager alloc] init];
    self.locationManager.locationDelegate = self;
    
    self.mapView = [[MAMapView alloc] initWithFrame:self.view.bounds];
    self.mapView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.mapView.delegate = self;
    
    [self.view addSubview:self.mapView];
    
    [self initToolBar];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    self.navigationController.toolbar.barStyle      = UIBarStyleBlack;
    self.navigationController.toolbar.translucent   = YES;
    [self.navigationController setToolbarHidden:NO animated:animated];
    
    if (!self.parameters) { /* Native调用进入 */
        [self requestLocationAlert];
    } else { /* JS API调用进入 */
        NSNumber *latitude = MQQNumberValue([self.parameters objectForKey:@"latitude"], nil);
        NSNumber *longitude = MQQNumberValue([self.parameters objectForKey:@"longitude"], nil);
        
        /*
         可选参数:
         NSString *name = MQQStringValue([self.parameters objectForKey:@"name"], nil);
         NSString *address = MQQStringValue([self.parameters objectForKey:@"address"], nil);
         NSNumber *scale = MQQNumberValue([self.parameters objectForKey:@"scale"], nil);
         */
         
        /* 标准地图转成火星地图定位 */
        CLLocation *location = [[CLLocation alloc] initWithLatitude:[latitude doubleValue] longitude:[longitude doubleValue]];
        location = [TMFLocationHelper applyChinaLocationShift:location];
        [self.mapView setCenterCoordinate:location.coordinate animated:YES];
    }
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.locationManager stopRequestLocation];
}

- (void)requestLocationAlert {
    QMUIAlertAction *action1 = [QMUIAlertAction actionWithTitle:@"好的" style:QMUIAlertActionStyleDefault
                                                        handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                            [self requestLocation];
                                                        }];
    QMUIAlertAction *action2 = [QMUIAlertAction actionWithTitle:@"不要啦" style:QMUIAlertActionStyleCancel handler:NULL];
    QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:nil message:@"将要进入定位请求，是否允许？"
                                                                          preferredStyle:QMUIAlertControllerStyleAlert];
    [alertController addAction:action1];
    [alertController addAction:action2];
    [alertController showWithAnimated:YES];
}

- (void)requestLocation {
    self.tipsView = [QMUITips showLoadingInView:self.view hideAfterDelay:30];
    [self.locationManager startRequestLocation];
}

#pragma mark - Action Handlers

- (void)mapTypeAction:(UISegmentedControl *)segmentedControl {
    self.mapView.mapType = segmentedControl.selectedSegmentIndex;
}

#pragma mark - Initialization

- (void)initToolBar {
    UIBarButtonItem *flexbleItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                                 target:self
                                                                                 action:nil];
    
    UISegmentedControl *mapTypeSegmentedControl = [[UISegmentedControl alloc] initWithItems:
                                                   [NSArray arrayWithObjects:
                                                    @"标准",
                                                    @"卫星",
                                                    @"标准黑夜",
                                                    @"导航",
                                                    @"公交",
                                                    nil]];
    mapTypeSegmentedControl.selectedSegmentIndex  = self.mapView.mapType;
    [mapTypeSegmentedControl addTarget:self action:@selector(mapTypeAction:) forControlEvents:UIControlEventValueChanged];
    
    UIBarButtonItem *mayTypeItem = [[UIBarButtonItem alloc] initWithCustomView:mapTypeSegmentedControl];
    
    self.toolbarItems = [NSArray arrayWithObjects:flexbleItem, mayTypeItem, flexbleItem, nil];
}

#pragma mark - Map Delegate
- (void)mapViewRegionChanged:(MAMapView *)mapView {
    NSLog(@"====1 %s", __FUNCTION__);
}

/*!
 @brief 地图区域即将改变时会调用此接口
 @param mapView 地图View
 @param animated 是否动画
 */
- (void)mapView:(MAMapView *)mapView regionWillChangeAnimated:(BOOL)animated {
    
}

/*!
 @brief 地图区域改变完成后会调用此接口
 @param mapView 地图View
 @param animated 是否动画
 */
- (void)mapView:(MAMapView *)mapView regionDidChangeAnimated:(BOOL)animated {
    NSLog(@"====2 %s", __FUNCTION__);
}

/**
 *  地图将要发生移动时调用此接口
 *
 *  @param mapView       地图view
 *  @param wasUserAction 标识是否是用户动作
 */
- (void)mapView:(MAMapView *)mapView mapWillMoveByUser:(BOOL)wasUserAction {
    
}

/**
 *  地图移动结束后调用此接口
 *
 *  @param mapView       地图view
 *  @param wasUserAction 标识是否是用户动作
 */
- (void)mapView:(MAMapView *)mapView mapDidMoveByUser:(BOOL)wasUserAction {
    
}

/**
 *  地图将要发生缩放时调用此接口
 *
 *  @param mapView       地图view
 *  @param wasUserAction 标识是否是用户动作
 */
- (void)mapView:(MAMapView *)mapView mapWillZoomByUser:(BOOL)wasUserAction {
    
}

/**
 *  地图缩放结束后调用此接口
 *
 *  @param mapView       地图view
 *  @param wasUserAction 标识是否是用户动作
 */
- (void)mapView:(MAMapView *)mapView mapDidZoomByUser:(BOOL)wasUserAction {
    
}

/**
 *  单击地图底图调用此接口
 *
 *  @param mapView    地图View
 *  @param coordinate 点击位置经纬度
 */
- (void)mapView:(MAMapView *)mapView didSingleTappedAtCoordinate:(CLLocationCoordinate2D)coordinate {
    
}

/**
 *  长按地图底图调用此接口
 *
 *  @param mapView    地图View
 *  @param coordinate 长按位置经纬度
 */
- (void)mapView:(MAMapView *)mapView didLongPressedAtCoordinate:(CLLocationCoordinate2D)coordinate {
    
}

/*!
 @brief 根据anntation生成对应的View
 @param mapView 地图View
 @param annotation 指定的标注
 @return 生成的标注View
 */
- (MAAnnotationView*)mapView:(MAMapView *)mapView viewForAnnotation:(id <MAAnnotation>)annotation {
    return nil;
}

/*!
 @brief 当mapView新添加annotation views时调用此接口
 @param mapView 地图View
 @param views 新添加的annotation views
 */
- (void)mapView:(MAMapView *)mapView didAddAnnotationViews:(NSArray *)views {
    
}

/*!
 @brief 当选中一个annotation views时调用此接口
 @param mapView 地图View
 @param view 选中的annotation views
 */
- (void)mapView:(MAMapView *)mapView didSelectAnnotationView:(MAAnnotationView *)view {
    
}

/*!
 @brief 当取消选中一个annotation views时调用此接口
 @param mapView 地图View
 @param view 取消选中的annotation views
 */
- (void)mapView:(MAMapView *)mapView didDeselectAnnotationView:(MAAnnotationView *)view {
    
}

/*!
 @brief 标注view的accessory view(必须继承自UIControl)被点击时调用此接口
 @param mapView 地图View
 @param view callout所属的标注view
 @param control 对应的control
 */
- (void)mapView:(MAMapView *)mapView annotationView:(MAAnnotationView *)view calloutAccessoryControlTapped:(UIControl *)control {
    
}

/**
 *  标注view的calloutview整体点击时调用此接口
 *
 *  @param mapView 地图的view
 *  @param view calloutView所属的annotationView
 */
- (void)mapView:(MAMapView *)mapView didAnnotationViewCalloutTapped:(MAAnnotationView *)view {
    
}

/*!
 @brief 在地图View将要启动定位时调用此接口
 @param mapView 地图View
 */
- (void)mapViewWillStartLocatingUser:(MAMapView *)mapView {
    
}

/*!
 @brief 在地图View停止定位后调用此接口
 @param mapView 地图View
 */
- (void)mapViewDidStopLocatingUser:(MAMapView *)mapView {
    
}

/*!
 @brief 位置或者设备方向更新后调用此接口
 @param mapView 地图View
 @param userLocation 用户定位信息(包括位置与设备方向等数据)
 @param updatingLocation 标示是否是location数据更新, YES:location数据更新 NO:heading数据更新
 */
- (void)mapView:(MAMapView *)mapView didUpdateUserLocation:(MAUserLocation *)userLocation updatingLocation:(BOOL)updatingLocation {
    
}

/*!
 @brief 定位失败后调用此接口
 @param mapView 地图View
 @param error 错误号，参考CLError.h中定义的错误号
 */
- (void)mapView:(MAMapView *)mapView didFailToLocateUserWithError:(NSError *)error {
    
}

/*!
 @brief 当userTrackingMode改变时调用此接口
 @param mapView 地图View
 @param mode 改变后的mode
 @param animated 动画
 */
- (void)mapView:(MAMapView *)mapView didChangeUserTrackingMode:(MAUserTrackingMode)mode animated:(BOOL)animated {
    
}

/*!
 @brief 拖动annotation view时view的状态变化，ios3.2以后支持
 @param mapView 地图View
 @param view annotation view
 @param newState 新状态
 @param oldState 旧状态
 */
- (void)mapView:(MAMapView *)mapView
        annotationView:(MAAnnotationView *)view
    didChangeDragState:(MAAnnotationViewDragState)newState
          fromOldState:(MAAnnotationViewDragState)oldState {
}

/*!
 @brief 根据overlay生成对应的Renderer
 @param mapView 地图View
 @param overlay 指定的overlay
 @return 生成的覆盖物Renderer
 */
- (MAOverlayRenderer *)mapView:(MAMapView *)mapView rendererForOverlay:(id <MAOverlay>)overlay {
    return nil;
}

/*!
 @brief 当mapView新添加overlay renderer时调用此接口
 @param mapView 地图View
 @param renderers 新添加的overlay renderers
 */
- (void)mapView:(MAMapView *)mapView didAddOverlayRenderers:(NSArray *)renderers {
    
}


#pragma mark - TMFLocationManagerDelegate

- (void)TMFLocationManager:(TMFLocationManager *)locationManager didFinishWithLocation:(CLLocation *)location {
    [self.tipsView hideAnimated:YES];
    // 标准地图转成火星地图定位
    CLLocation *lo = [TMFLocationHelper applyChinaLocationShift:location];
    [self.mapView setCenterCoordinate:lo.coordinate animated:YES];
}

- (void)TMFLocationManager:(TMFLocationManager *)locationManager didFailWithError:(NSError *)error {
    [self.tipsView hideAnimated:YES];
    
    QMUIAlertAction *action1 = [QMUIAlertAction actionWithTitle:@"知道啦" style:QMUIAlertActionStyleCancel handler:NULL];
    QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:nil message:@"定位失败了~" preferredStyle:QMUIAlertControllerStyleAlert];
    [alertController addAction:action1];
    [alertController showWithAnimated:YES];
}

@end
