//
//  LineDashPolygonController.m
//  QMapKitDemo
//
//  Created by v_hefang on 2020/12/23.
//  Copyright © 2020 tencent. All rights reserved.
//

#import "LineDashPolygonController.h"

@interface LineDashPolygonController ()

@property (nonatomic, strong) NSMutableArray<id <QOverlay>> *overlays;

@end

@implementation LineDashPolygonController

- (QOverlayView *)mapView:(QMapView *)mapView viewForOverlay:(id<QOverlay>)overlay
{
    if ([overlay isKindOfClass:[QPolygon class]])
    {
        QPolygonView *polygonView = [[QPolygonView alloc] initWithPolygon:overlay];
        polygonView.lineWidth   = 3;
        polygonView.strokeColor = [UIColor colorWithRed:70/255.0 green:119/255.0 blue:223/255.0 alpha:.8];
        polygonView.fillColor   = [[UIColor grayColor] colorWithAlphaComponent:0.2];
        
        
        if (overlay == self.overlays[0]) {
            // 实线
            polygonView.lineType = QOverlayStrokeType_Default;
        } else if (overlay == self.overlays[1]) {
            // 虚线
            polygonView.lineType = QOverlayStrokeType_Dash;
            polygonView.lineDashPattern = @[@10, @5];
        } else {
            // 点
            polygonView.lineType = QOverlayStrokeType_Dot;
            polygonView.textureGap = 10;
        }
        
        return polygonView;
    }
    
    return nil;
}

- (void)setupOverlays
{
    self.overlays = [NSMutableArray array];
    
    /* Polygon1. */
    CLLocationCoordinate2D coords[6];
    coords[0].latitude = 40.041533;
    coords[0].longitude = 116.271481;
    
    coords[1].latitude = 40.041845;
    coords[1].longitude = 116.274464;
    
    coords[2].latitude = 40.038954;
    coords[2].longitude = 116.27515;
    
    coords[3].latitude = 40.038568;
    coords[3].longitude = 116.272146;
    
    QPolygon *polygon = [QPolygon polygonWithCoordinates:coords count:4];
    [self.overlays addObject:polygon];
    
    /* Polygon1. */
    CLLocationCoordinate2D coords1[6];
    coords1[0].latitude = 40.041895;
    coords1[0].longitude = 116.274668;
    
    coords1[1].latitude = 40.042297;
    coords1[1].longitude = 116.277715;
    
    coords1[2].latitude = 40.041032;
    coords1[2].longitude = 116.277983;
    
    coords1[3].latitude = 40.04058;
    coords1[3].longitude = 116.275032;
    
    QPolygon *polygon1 = [QPolygon polygonWithCoordinates:coords1 count:4];
    [self.overlays addObject:polygon1];
    
    /* Polygon2. */
    CLLocationCoordinate2D coordinates[4];
    coordinates[0].latitude = 40.040498;
    coordinates[0].longitude = 116.275011;
    
    coordinates[1].latitude = 40.040966;
    coordinates[1].longitude = 116.278037;
    
    coordinates[2].latitude = 40.039537;
    coordinates[2].longitude = 116.278337;
    
    coordinates[3].latitude = 40.038987;
    coordinates[3].longitude = 116.275311;
    QPolygon *polygon2 = [QPolygon polygonWithCoordinates:coordinates count:4];
    
    [self.overlays addObject:polygon2];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupOverlays];
    [self.mapView addOverlays:self.overlays];
    [self.mapView setCenterCoordinate:CLLocationCoordinate2DMake(40.040531,116.274936)];
    [self.mapView setZoomLevel:16];
}

@end
