//
//  ScatterDotViewController.m
//  QMapKitDemo
//
//  Created by KeithCao on 2021/11/26.
//  Copyright © 2021 tencent. All rights reserved.
//

#import "ScatterDotViewController.h"

#import <QMapVisualPlugin/QMapPluginKit.h>

#define GlobalNavAndStatusHight (self.navigationController.navigationBar.frame.size.height + [[UIApplication sharedApplication] statusBarFrame].size.height)

static const NSInteger GlobalStyleTitleArrayCount = 9;

static NSString *const GlobalStyleTitleArray[GlobalStyleTitleArrayCount] =
{
    @"展示层级和颜色",
    @"数据空",
    @"级别+类型",
    @"透明度",
    @"可见性",
    @"3D",
    @"有数据",
    @"动画",
    @"zIndex"
};


@interface ScatterDotViewController ()

@property (nonatomic, strong) QDotScatterOverlay *layer;

@property (nonatomic, strong) QDotScatterOverlay *layer2;

@property (nonatomic, strong) NSMutableArray <QScatterNode *> *nodes;

@property (nonatomic, strong) NSMutableArray <QScatterNode *> *node2;

@end

@implementation ScatterDotViewController

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self.navigationController setToolbarHidden:NO animated:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    [self.navigationController setToolbarHidden:YES animated:animated];
}


- (void)handleSwitchAction
{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"请选择需要切换的样式" message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil]];
    
    for (int i = 0; i < GlobalStyleTitleArrayCount; i++)
    {
        [alertController addAction:[UIAlertAction actionWithTitle:GlobalStyleTitleArray[i] style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [self handleChangeAction:i];
        }]];
    }
    
    [self presentViewController:alertController animated:YES completion:nil];
}


#pragma mark -- changeAction

- (void)handleChangeAction:(NSInteger)actNum
{
    switch (actNum) {
        case 0:
            
            [self changeColorAndDisplayLevel];
            
            break;
        case 1:
            [self changeDataToEmpty];
            
            break;
            
        case 2:
            [self changeZoomAndType];
            
            break;
        case 3:
            [self changeOpacity];
            break;
            
        case 4:
            [self changeVisible];
            break;
        case 5:
            [self change3D];
            break;
        case 6:
            [self changeToHaveData];
            break;
        case 7:
            [self changeHeightAndWidth];
            break;
        case 8:
            [self changeZIndex];
            break;
        default:
            break;
    }
}

// 颜色+展示层级
- (void)changeColorAndDisplayLevel
{
    self.layer.colors = @[[UIColor yellowColor], [UIColor orangeColor], [UIColor magentaColor]];
    
    QIconScatterOverlayView *hv = (QIconScatterOverlayView *) [self.mapView viewForOverlay:self.layer];

    hv.displayLevel = QOverlayLevelAboveLabels;
    [self.layer setNeedsDisplay];
}

// 1 数据空
- (void)changeDataToEmpty
{
    self.layer.nodes = @[];
    [self.layer setNeedsDisplay];
}

// 2 type 和 最大最小
- (void)changeZoomAndType
{
    [self.layer setMinZoomLevel:10 maxZoomLevel:17];
    
}

// 3 透明
- (void)changeOpacity
{
    self.layer.opacity = self.layer.opacity == 1 ? 0.5 : 1;

}

// 4 可见性
- (void)changeVisible
{
    self.layer.visible = !self.layer.visible;
}

//5 3d
- (void)change3D
{
    self.layer.enable3D = !self.layer.enable3D;
    [self.layer setNeedsDisplay];
}

// 6 热力区间
- (void)changeToHaveData
{
    
    self.layer.nodes = self.nodes;
    [self.layer setNeedsDisplay];

}

// 7 贴图宽度、高度
- (void)changeHeightAndWidth
{
    self.layer.animated = !self.layer.animated;
    [self.layer setNeedsDisplay];
}

// 8 zindex
- (void)changeZIndex
{
    QIconScatterOverlayView *hv = (QIconScatterOverlayView *)[self.mapView viewForOverlay:self.layer];
    QIconScatterOverlayView *hv2 = (QIconScatterOverlayView *)[self.mapView viewForOverlay:self.layer2];
    
    NSLog(@"heat :%d,  heat2:%d", hv.zIndex, hv2.zIndex);
    
    hv.zIndex = 100;
    
}

#pragma mark - Setup

- (void)setupSelf
{
    self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
}

- (void)setupNavigation
{
    self.navigationController.navigationBar.translucent = YES;
    
    UIBarButtonItem *switchButton = [[UIBarButtonItem alloc] initWithTitle:@"切换样式" style:UIBarButtonItemStyleDone target:self action:@selector(handleSwitchAction)];
    
    self.navigationItem.rightBarButtonItem = switchButton;
}

- (void)setupToolbar
{
    UIBarButtonItem *flexble = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    
    UIBarButtonItem *add    = [[UIBarButtonItem alloc] initWithTitle:@"添加"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleAddAction)];
    UIBarButtonItem *remove = [[UIBarButtonItem alloc] initWithTitle:@"删除"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleRemoveAction)];
    UIBarButtonItem *reload = [[UIBarButtonItem alloc] initWithTitle:@"再加一个"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleReloadAction)];
    
    self.toolbarItems = @[flexble, add, flexble, remove, flexble, reload, flexble, flexble];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupSelf];
    [self setupNavigation];
//    [self setupStyleView];
    
    [self setupToolbar];
    
    self.mapView.delegate = self;
    self.mapView.showsScale = YES;
    
    self.mapView.overlooking = 45;
    self.mapView.zoomLevel = 10;
    
}

- (void)handleAddAction
{
    [self createlayer];
}

- (void)handleRemoveAction
{
    [self.mapView removeOverlay:self.layer];
}

- (void)handleReloadAction
{
    [self createSecondLayer];
}

- (void)createSecondLayer
{
    self.node2 = [NSMutableArray array];
    {
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"weightdot" ofType:@"data"];
        
        NSString* fileContents = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        NSArray* allLinedStrings = [fileContents componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
        
        
        [allLinedStrings enumerateObjectsUsingBlock:^(NSString *  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            
            NSArray *ar = [obj componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            if (ar.count == 3)
            {
                // 纬度.
                double lat = [ar[1] doubleValue];
                // 经度.
                double lon = [ar[0] doubleValue];
                // 权值.
                
                QScatterNode *node = [[QScatterNode alloc] init];
                node.coordinate = CLLocationCoordinate2DMake(lat, lon);
                node.type      = arc4random()%3;
                
                [self.node2 addObject:node];
            }
        }];
    }
    
    self.layer2 = [[QDotScatterOverlay alloc] init];
    
    self.layer2.nodes = self.node2;
    
    self.layer2.enable3D = YES;
    
    self.layer2.radius = 9;
    
    self.layer2.colors = @[[UIColor blackColor], [UIColor cyanColor], [UIColor grayColor]];
    
    [self.mapView addOverlay:self.layer2];
}

- (void)createlayer
{
    self.nodes = [NSMutableArray array];
    {
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"weightdot.data" ofType:nil];
        
        NSString* fileContents = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        // first, separate by new line
        NSArray* allLinedStrings = [fileContents componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
        
        
        [allLinedStrings enumerateObjectsUsingBlock:^(NSString *  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            
            NSArray *ar = [obj componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            if (ar.count == 3)
            {
                // 纬度.
                double lat = [ar[1] doubleValue];
                // 经度.
                double lon = [ar[0] doubleValue];
                // 权值.
                
                QScatterNode *node = [[QScatterNode alloc] init];
                node.coordinate = CLLocationCoordinate2DMake(lat, lon);
                node.type      = arc4random() %3;
                
                [self.nodes addObject:node];
            }
        }];
    }
    
    self.layer = [[QDotScatterOverlay alloc] init];
    
    self.layer.nodes = self.nodes;
  
    self.layer.enable3D = YES;
    
    self.layer.radius = 9;
    
    
    [self.mapView addOverlay:self.layer];
    
}

- (void)handleTestAction
{
    [self createlayer];
}

- (void)handleTest2Action
{

    
}

#pragma mark -- MapDelegate

- (QOverlayView *)mapView:(QMapView *)mapView viewForOverlay:(id<QOverlay>)overlay
{
    if ([overlay isKindOfClass:[QDotScatterOverlay class]])
    {
        QDotScatterOverlayView *view = [[QDotScatterOverlayView alloc] initWithOverlay:overlay];
        
        return view;
    }
    
    return nil;
}
@end
