//
//  ScatterIconViewController.m
//  QMapKitDemo
//
//  Created by KeithCao on 2021/11/26.
//  Copyright © 2021 tencent. All rights reserved.
//

#import "ScatterIconViewController.h"
#import <QMapVisualPlugin/QMapPluginKit.h>

#define GlobalNavAndStatusHight (self.navigationController.navigationBar.frame.size.height + [[UIApplication sharedApplication] statusBarFrame].size.height)

static const NSInteger GlobalStyleTitleArrayCount = 9;

static NSString *const GlobalStyleTitleArray[GlobalStyleTitleArrayCount] =
{
    @"展示层级和颜色",
    @"数据空",
    @"级别+类型",
    @"透明度",
    @"可见性",
    @"3D",
    @"有数据",
    @"贴图宽度、高度",
    @"zIndex"
};

@interface ScatterIconViewController () <QMapViewDelegate>

@property (nonatomic, strong) QIconScatterOverlay *iconLayer;

@property (nonatomic, strong) QIconScatterOverlay *iconLayer2;

@property (nonatomic, strong) NSMutableArray <QScatterNode *> *nodes;

@property (nonatomic, strong) NSMutableArray <QScatterNode *> *node2;

@end

@implementation ScatterIconViewController

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self.navigationController setToolbarHidden:NO animated:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    [self.navigationController setToolbarHidden:YES animated:animated];
}


#pragma mark - Action

- (void)handleSwitchAction
{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"请选择需要切换的样式" message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil]];
    
    for (int i = 0; i < GlobalStyleTitleArrayCount; i++)
    {
        [alertController addAction:[UIAlertAction actionWithTitle:GlobalStyleTitleArray[i] style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [self handleChangeAction:i];
        }]];
    }
    
    [self presentViewController:alertController animated:YES completion:nil];
}


#pragma mark -- changeAction

- (void)handleChangeAction:(NSInteger)actNum
{
    switch (actNum) {
        case 0:
            
            [self changeColorAndDisplayLevel];
            
            break;
        case 1:
            [self changeDataToEmpty];
            
            break;
            
        case 2:
            [self changeZoomAndType];
            
            break;
        case 3:
            [self changeOpacity];
            break;
            
        case 4:
            [self changeVisible];
            break;
        case 5:
            [self change3D];
            break;
        case 6:
            [self changeToHaveData];
            break;
        case 7:
            [self changeHeightAndWidth];
            break;
        case 8:
            [self changeZIndex];
            break;
        default:
            break;
    }
}

// 颜色+展示层级
- (void)changeColorAndDisplayLevel
{
    
    
    QIconScatterOverlayView *hv = (QIconScatterOverlayView *) [self.mapView viewForOverlay:self.iconLayer];

    hv.displayLevel = QOverlayLevelAboveLabels;
}

// 1 数据空
- (void)changeDataToEmpty
{
    self.iconLayer.nodes = @[];
    [self.iconLayer setNeedsDisplay];
}

// 2 type 和 最大最小
- (void)changeZoomAndType
{
    [self.iconLayer setMinZoomLevel:10 maxZoomLevel:17];
    
}

// 3 透明
- (void)changeOpacity
{
    self.iconLayer.opacity = self.iconLayer.opacity == 1 ? 0.5 : 1;

}

// 4 可见性
- (void)changeVisible
{
    self.iconLayer.visible = !self.iconLayer.visible;
}

//5 3d
- (void)change3D
{
    self.iconLayer.enable3D = !self.iconLayer.enable3D;
    [self.iconLayer setNeedsDisplay];
}

// 6 热力区间
- (void)changeToHaveData
{
    
    self.iconLayer.nodes = self.nodes;
    [self.iconLayer setNeedsDisplay];

}

// 7 贴图宽度、高度
- (void)changeHeightAndWidth
{
    self.iconLayer.size = CGSizeMake(50, 40);
    
    [self.iconLayer setNeedsDisplay];
}

// 8 zindex
- (void)changeZIndex
{
    QIconScatterOverlayView *hv = (QIconScatterOverlayView *)[self.mapView viewForOverlay:self.iconLayer];
    QIconScatterOverlayView *hv2 = (QIconScatterOverlayView *)[self.mapView viewForOverlay:self.iconLayer2];
    
    NSLog(@"heat :%d,  heat2:%d", hv.zIndex, hv2.zIndex);
    
    hv.zIndex = 100;
    
}

#pragma mark - Setup

- (void)setupSelf
{
    self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
}

- (void)setupNavigation
{
    self.navigationController.navigationBar.translucent = YES;
    
    UIBarButtonItem *switchButton = [[UIBarButtonItem alloc] initWithTitle:@"切换样式" style:UIBarButtonItemStyleDone target:self action:@selector(handleSwitchAction)];
    
    self.navigationItem.rightBarButtonItem = switchButton;
}

- (void)setupToolbar
{
    UIBarButtonItem *flexble = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    
    UIBarButtonItem *add    = [[UIBarButtonItem alloc] initWithTitle:@"添加"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleAddAction)];
    UIBarButtonItem *remove = [[UIBarButtonItem alloc] initWithTitle:@"删除"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleRemoveAction)];
    
    self.toolbarItems = @[flexble, add, flexble, remove, flexble, flexble];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupSelf];
    [self setupNavigation];
    
    [self setupToolbar];
    
    self.mapView.delegate = self;
    self.mapView.showsScale = YES;
    
    self.mapView.overlooking = 45;
    self.mapView.zoomLevel = 13;

}

- (void)handleAddAction
{
    [self createIconLayer];
}

- (void)handleRemoveAction
{
    [self.mapView removeOverlay:self.iconLayer];
}

- (void)createIconLayer
{
    self.node2 = [NSMutableArray array];
    {
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"weightdot.data" ofType:nil];
        
        NSString* fileContents = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        // first, separate by new line
        NSArray* allLinedStrings = [fileContents componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
        
        
        [allLinedStrings enumerateObjectsUsingBlock:^(NSString *  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            
            NSArray *ar = [obj componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            if (ar.count == 3)
            {
                // 纬度.
                double lat = [ar[1] doubleValue];
                // 经度.
                double lon = [ar[0] doubleValue];
                // 权值.
                
                QScatterNode *node = [[QScatterNode alloc] init];
                node.coordinate = CLLocationCoordinate2DMake(lat, lon);
                node.type      = arc4random() % 2;
                
                [self.node2 addObject:node];
            }
        }];
    }
    
    self.iconLayer2 = [[QIconScatterOverlay alloc] init];
    
    self.iconLayer2.nodes = self.node2;
    self.iconLayer2.size = CGSizeMake(30, 30);
    
    self.iconLayer2.enable3D = YES;
    
    NSString *good = [[NSBundle mainBundle] pathForResource:@"good@2x.png" ofType:nil];
    
    NSString *ball = [[NSBundle mainBundle] pathForResource:@"ball.png" ofType:nil];
    
    UIImage *goodImg = [UIImage imageWithContentsOfFile:good];
    UIImage *ballImg = [UIImage imageWithContentsOfFile:ball];
    
    self.iconLayer2.icons = [NSArray arrayWithObjects:goodImg, ballImg, nil];
    
    [self.mapView addOverlay:self.iconLayer2];
}

- (void)handleTestAction
{
    [self createIconLayer];
}

#pragma mark -- MapDelegate

- (QOverlayView *)mapView:(QMapView *)mapView viewForOverlay:(id<QOverlay>)overlay
{
    if ([overlay isKindOfClass:[QIconScatterOverlay class]])
    {
        QIconScatterOverlayView *view = [[QIconScatterOverlayView alloc] initWithOverlay:overlay];
        
        return view;
    }
    
    return nil;
}



@end
