//
//  TrailLineViewController.m
//  QMapKitDemo
//
//  Created by KeithCao on 2021/11/26.
//  Copyright © 2021 tencent. All rights reserved.
//

#import "TrailLineViewController.h"
#import <QMapVisualPlugin/QMapPluginKit.h>

#define GlobalNavAndStatusHight (self.navigationController.navigationBar.frame.size.height + [[UIApplication sharedApplication] statusBarFrame].size.height)

static const NSInteger GlobalStyleTitleArrayCount = 9;

static NSString *const GlobalStyleTitleArray[GlobalStyleTitleArrayCount] =
{
    @"展示层级和颜色",
    @"数据置nil",
    @"级别+线宽",
    @"透明度",
    @"可见性",
    @"恢复数据",
    @"速率",
    @"zIndex",
    @"高亮时间"
};

@interface TrailLineViewController () <QMapViewDelegate>

@property (nonatomic, strong) NSMutableArray <QTrailLine *> *line;

@property (nonatomic, strong) QTrailOverlay *trail;

@property (nonatomic, assign) int maxTime;

@end

@implementation TrailLineViewController

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self.navigationController setToolbarHidden:NO animated:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    [self.navigationController setToolbarHidden:YES animated:animated];
}

#pragma mark - Action

- (void)handleSwitchAction
{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"请选择需要切换的样式" message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:nil]];
    
    for (int i = 0; i < GlobalStyleTitleArrayCount; i++)
    {
        [alertController addAction:[UIAlertAction actionWithTitle:GlobalStyleTitleArray[i] style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [self handleChangeAction:i];
        }]];
    }
    
    [self presentViewController:alertController animated:YES completion:nil];
}


#pragma mark -- changeAction

- (void)handleChangeAction:(NSInteger)actNum
{
    switch (actNum) {
        case 0:
            
            [self changeColorAndDisplayLevel];
            
            break;
        case 1:
            [self changeDataToNil];
            
            break;
            
        case 2:
            [self changeZoomAndType];
            
            break;
        case 3:
            [self changeOpacity];
            break;
            
        case 4:
            [self changeVisible];
            break;
        case 5:
            [self recoverData];
            break;
        case 6:
            [self changeRatio];
            break;
        case 7:
            [self changeZIndex];
            break;
        case 8:
            [self changeHighLightDuration];
            break;
        default:
            break;
    }
}

// 颜色+展示层级
- (void)changeColorAndDisplayLevel
{
    NSArray<UIColor *> *colorDB = @[
                [UIColor purpleColor],
                [UIColor cyanColor],
                [UIColor greenColor]
    ];
    
    self.trail.style.colors = colorDB;
    
    
    [self.trail setNeedsDisplay];
    
    QTrailOverlayView *hv = (QTrailOverlayView *) [self.mapView viewForOverlay:self.trail];

    hv.displayLevel = QOverlayLevelAboveLabels;
}

// 1 半径
- (void)changeDataToNil
{

    self.trail.lines = nil;
    [self.trail setNeedsDisplay];
    
}

// 2 type 和 最大最小
- (void)changeZoomAndType
{
    [self.trail setMinZoomLevel:12 maxZoomLevel:17];
    
    self.trail.style.width = 10;
    
    [self.trail setNeedsDisplay];
    
    self.mapView.zoomLevel = 15;
}

// 3 透明
- (void)changeOpacity
{
    self.trail.opacity = self.trail.opacity == 1 ? 0.5 : 1;

}

// 4 可见性
- (void)changeVisible
{
    self.trail.visible = self.trail.visible ? NO : YES;
    
    [self.trail setNeedsDisplay];
}

// 6 热力区间
- (void)recoverData
{
    
    self.trail.lines = self.line;
    [self.trail setNeedsDisplay];

}

// 7 高度
- (void)changeRatio
{
    self.trail.style.animateRatio = 10;
    [self.trail setNeedsDisplay];
}

// 8 zindex
- (void)changeZIndex
{
    QTrailOverlayView *hv = (QTrailOverlayView *)[self.mapView viewForOverlay:self.trail];
    
    hv.zIndex = 100;
    
}

//9 高亮时间
- (void)changeHighLightDuration
{
    self.trail.style.highlightDuration = arc4random() % 2000 + 1000;
    
    [self.trail setNeedsDisplay];
}

#pragma mark - Setup

- (void)setupSelf
{
    self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
}

- (void)setupNavigation
{
    self.navigationController.navigationBar.translucent = YES;
    
    UIBarButtonItem *switchButton = [[UIBarButtonItem alloc] initWithTitle:@"切换样式" style:UIBarButtonItemStyleDone target:self action:@selector(handleSwitchAction)];
    
    self.navigationItem.rightBarButtonItem = switchButton;
}

- (void)setupToolbar
{
    UIBarButtonItem *flexble = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    
    UIBarButtonItem *add    = [[UIBarButtonItem alloc] initWithTitle:@"添加"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleAddAction)];
    UIBarButtonItem *remove = [[UIBarButtonItem alloc] initWithTitle:@"删除"  style:UIBarButtonItemStyleDone  target:self action:@selector(handleRemoveAction)];
    
    self.toolbarItems = @[flexble, add, flexble, remove, flexble];
}

-(void)handleAddAction
{
    [self constructTrail];
}

- (void)handleRemoveAction
{
    [self.mapView removeOverlays:self.mapView.overlays];
}

- (void)handleReloadAction
{
    
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupSelf];
    [self setupNavigation];
    
    [self setupToolbar];
    
    self.mapView.centerCoordinate = CLLocationCoordinate2DMake(39.792151    ,116.397607);
    
    self.mapView.zoomLevel = 10;
}

- (void)constructTrail
{
    [self buildTrailData];
    
     self.trail = [[QTrailOverlay alloc] initWithTrailData:self.line];
    
    QTrailLineStyle *style = [[QTrailLineStyle alloc] init];
    style.width = 5;
    
    self.trail.style = style;
    
    [self.mapView addOverlay:self.trail];
    
}

- (QOverlayView *)mapView:(QMapView *)mapView viewForOverlay:(id<QOverlay>)overlay
{
    if ([overlay isKindOfClass:[QTrailOverlay class]])
    {
        QTrailOverlayView *tv = [[QTrailOverlayView alloc] initWithTrailOverlay:overlay];
        
        return tv;
    }
    
    return nil;
}

- (void)buildTrailData
{
        self.line = [NSMutableArray array];
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"trailData" ofType:@"dat"];
    
            NSString* fileContents = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
    
            // first, separate by new line
            NSArray* allLinedStrings = [fileContents componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
    
            __block int lineCount = 0;
    
            __block int temp = 0;
    
            [allLinedStrings enumerateObjectsUsingBlock:^(NSString *  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                
                // 单条线的数据
                
                QTrailLine *singleLine = [[QTrailLine alloc] init];
                NSMutableArray <QTrailNode *> *nodes = [NSMutableArray array];
                NSArray *ar = [obj componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@","]];
                if (lineCount < 10)
                {
                [ar enumerateObjectsUsingBlock:^(id  _Nonnull obj2, NSUInteger idx, BOOL * _Nonnull stop) {
                    
                    NSArray *finalArray = [obj2 componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                    
                    if ((int)finalArray.count >= 3)
                    {
                        // 纬度.
                        double lat = [finalArray[0] doubleValue];
                        // 经度.
                        double lon = [finalArray[1] doubleValue];
                        // 权值.
                        double val = [finalArray[2] doubleValue];
                        
                        if ( (lat >= -90) && (lat <= 90) && (lon >= -180) && (lon <= 180) )
                        {
                            QTrailNode *node = [[QTrailNode alloc] init];
                            
                            node.coordinate = CLLocationCoordinate2DMake(lat, lon);
                            node.timeStamp = val;
                            
                            if (val > temp)
                            {
                                temp = val;
                            }
                            
                            [nodes addObject:node];
                            
                            QPointAnnotation *anno = [[QPointAnnotation alloc] init];
                            anno.coordinate = CLLocationCoordinate2DMake(lat, lon);
                        }
                        
                    }
                }];
                    singleLine.nodes = nodes;
                    [self.line addObject:singleLine];
                }
                lineCount++;
                
            }];
    
    self.maxTime = temp;
}

@end
