//
//  TMFLocationViewController.m
//  TMFDemo
//
//  Created by hauzhong on 2019/5/22.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFLocationViewController.h"
#import "QMUITips.h"

// TMFLocation
#import "TMFLocationCenter.h"
#import "TMFLocationHelper.h"
#import "CLGeocoder+TMFExtension.h"

// AMap
#import "AMapViewController.h"
// Qmap
#import "EntryViewController.h"
// TMFProfile
#import "TMFProfile.h"
#import "NSString+Localize.h"

@interface TMFLocationViewController ()

@end

@implementation TMFLocationViewController

- (void)initDataSource {
    [super initDataSource];
    QMUIOrderedDictionary<NSString *, NSString *> *dataSourceWithDetailText = [[QMUIOrderedDictionary alloc] init];
    [dataSourceWithDetailText setObject:@"QMap" forKey:[@"QMap" localizedString]];
    [dataSourceWithDetailText setObject:@"AMap" forKey:[@"AMap" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFProfile" forKey:[@"geographic reporting" localizedString]];
    self.dataSourceWithDetailText = dataSourceWithDetailText;
}

- (void)didSelectCellWithTitle:(NSString *)title {
    if ([title isEqualToString:[@"QMap" localizedString]]) {
        [self createQMapController];
    } else if ([title isEqualToString:[@"AMap" localizedString]]) {
        [self createAMapController];
    } else if ([title isEqualToString:[@"geographic reporting" localizedString]]) {
        [self requestPlacemarkAndReportProfile];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = [@"locaiton module" localizedString];
}

#pragma mark - AMap

- (void)createAMapController {
    AMapViewController *viewController = [[AMapViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - QMap

- (void)createQMapController {
    EntryViewController *viewController = [[EntryViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFProfile

- (void)requestPlacemarkAndReportProfile {
    [QMUITips showLoading:[@"getting address information" localizedString] inView:self.view];
    [[TMFLocationCenter sharedCenter] requestLocationWithSuccessHandler:^(CLLocation *_Nonnull location) {
        location = [TMFLocationHelper applyChinaLocationShift:location];

        // 根据经纬度反向地理编译出地址信息
        CLGeocoder *geocoder = [[CLGeocoder alloc] init];
        NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"zh-Hant"];
        [geocoder tmfReverseGeocodeLocation:location preferredLocale:locale
                          completionHandler:^(NSArray<CLPlacemark *> *_Nullable placemarks, NSError *_Nullable error) {
                              dispatch_async(dispatch_get_main_queue(), ^{
                                  [QMUITips hideAllTipsInView:self.view];
                                  if (error == nil && placemarks.count > 0) {
                                      CLPlacemark *placemark = placemarks.firstObject;
                                      [self reportProfileWithPlacemark:placemark];
                                  } else {
                                      [QMUITips showError:[NSString stringWithFormat:[@"Failed to get address information\nError info: %@" localizedString], error]];
                                  }
                              });
                          }];
    } failureHandler:^(NSError *_Nonnull error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [QMUITips hideAllTipsInView:self.view];
            [QMUITips showError:[NSString stringWithFormat:[@"Failed to get address information\nError info: %@" localizedString], error]];
        });
    }];
}

- (void)reportProfileWithPlacemark:(CLPlacemark *)placemark {
    NSString *country = placemark.country;              // 国家
    NSString *province = placemark.administrativeArea;  // 省份
    NSString *city = placemark.locality;                // 城市

    TMFProfileReporter *profileReporter = [TMFProfileReporter defaultReporter];
    [profileReporter updateProfileWithID:TMFProfileIDCountry stringValue:country];
    [profileReporter updateProfileWithID:TMFProfileIDProvince stringValue:province];
    [profileReporter updateProfileWithID:TMFProfileIDCity stringValue:city];

    NSString *message = [NSString stringWithFormat:[@"Country: %@\nProvince: %@\nCity: %@" localizedString], country ?: @"", province ?: @"", city ?: @""];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"report the current geographic location" localizedString] message:message
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)reportAnalyticsWithPlacemark:(CLPlacemark *)placemark {
    CLLocation *location = placemark.location;
    NSString *message =
        [NSString stringWithFormat:[@"Country: %@\nProvince: %@\nCity: %@\nLongitude: %lf\nLatitude: %lf" localizedString], placemark.country ?: @"",
                  placemark.administrativeArea ?: @"", placemark.locality ?: @"", location.coordinate.longitude, location.coordinate.latitude];

    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"report the current geographic location" localizedString] message:message
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end
