//
//  TMFOCRSystemAuthorizer.m
//  EMBC
//
//  Created by emmia on 2018/8/5.
//  Copyright © 2018年 Tencent Technology(Shenzhen) Company Limited. All rights reserved.
//

#import "TMFOCRSystemAuthorizer.h"
#import <AVFoundation/AVFoundation.h>
#import <Photos/Photos.h>

@implementation TMFOCRSystemAuthorizer

//非回调形式，不支持通知
+ (TMFOCRSystemAuthorizerStatus)getAuthorizationStatusWithType:(TMFOCRSystemAuthorizerType)type {
    __block TMFOCRSystemAuthorizerStatus status = TMFOCRSystemAuthorizerStatusAuthorized;
    switch (type) {
        case TMFOCRSystemAuthorizerTypeCamera: {
            status = (TMFOCRSystemAuthorizerStatus)[AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
        } break;
        case TMFOCRSystemAuthorizerTypePhoto: {
            status = (TMFOCRSystemAuthorizerStatus)[PHPhotoLibrary authorizationStatus];
        } break;
        case TMFOCRSystemAuthorizerTypeAudio: {
            status = (TMFOCRSystemAuthorizerStatus)[AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeAudio];
        } break;
        default: {
            status = TMFOCRSystemAuthorizerStatusAuthorized;
        } break;
    }
    return status;
}

//授权
+ (void)requestAuthorizedWithType:(TMFOCRSystemAuthorizerType)type completion:(void (^)(TMFOCRSystemAuthorizerStatus status))completion {
    switch (type) {
        case TMFOCRSystemAuthorizerTypeCamera: {
            [AVCaptureDevice requestAccessForMediaType:AVMediaTypeVideo completionHandler:^(BOOL granted) {
                TMFOCRSystemAuthorizerStatus status = (granted ? TMFOCRSystemAuthorizerStatusAuthorized : TMFOCRSystemAuthorizerStatusDenied);
                if (completion) {
                    completion(status);
                }
            }];
        } break;
        case TMFOCRSystemAuthorizerTypePhoto: {
            [PHPhotoLibrary requestAuthorization:^(PHAuthorizationStatus status) {
                if (completion) {
                    completion((TMFOCRSystemAuthorizerStatus)status);
                }
            }];
        } break;
        case TMFOCRSystemAuthorizerTypeAudio: {
            [AVCaptureDevice requestAccessForMediaType:AVMediaTypeAudio completionHandler:^(BOOL granted) {
                TMFOCRSystemAuthorizerStatus status = (granted ? TMFOCRSystemAuthorizerStatusAuthorized : TMFOCRSystemAuthorizerStatusDenied);
                if (completion) {
                    completion(status);
                }
            }];
        } break;
        default:
            break;
    }
}

@end
