//
//  TMFOCRMainViewController.m
//  EMOCRDetect
//
//  Created by emmia on 2019/4/18.
//  Copyright © 2019 Tencent Technology(Shenzhen) Company Limited. All rights reserved.
//

#import "TMFOCRMainViewController.h"
#import "TMFOCRSystemAuthorizer.h"
#import "TMFOCRProcessViewController.h"
#import "TMFOCRCameraViewController.h"
#import "UIView+EMBCFrameAdjust.h"
#import "NSString+Localize.h"

#define TMF_OCR_MAINCAMERA_DEFAULTBUTTON_TAG 1011
#define TMF_OCR_MAINCAMERA_RECT_TAG 1012

@interface TMFOCRMainViewController ()<UIImagePickerControllerDelegate, UINavigationControllerDelegate>

/**
 @brief OCR识别类型
 */
@property (nonatomic, assign) TMFOCRScanType    scanType;

@end

@implementation TMFOCRMainViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.title = @"TMFOCR";

    self.scanType = TMFOCRScanTypeBankcard;

    self.view.backgroundColor = [UIColor whiteColor];

    //类型
    UILabel *typeLabel = [[UILabel alloc] initWithFrame:CGRectMake(10 * [UIView widthScale], 100 * [UIView widthScale],
                                                            self.view.width - 20 * [UIView widthScale], 30 * [UIView widthScale])];
    typeLabel.textColor = [UIColor blackColor];
    typeLabel.text = [@"please select the type to be recognized:" localizedString];
    typeLabel.font = [UIFont systemFontOfSize:15];
    [self.view addSubview:typeLabel];

    NSArray *array = [NSArray arrayWithObjects:[@"bank card" localizedString], [@"ID card avatar page" localizedString], [@"ID card national emblem page" localizedString], nil];
    UISegmentedControl *typeSegmentControl = [[UISegmentedControl alloc] initWithItems:array];
    typeSegmentControl.frame = CGRectMake(typeLabel.minX, typeLabel.maxY, typeLabel.width, 40 * [UIView widthScale]);
    [typeSegmentControl setTitleTextAttributes:@{NSFontAttributeName:[UIFont systemFontOfSize:[NSString isCN]?15:12]} forState:UIControlStateNormal];
    typeSegmentControl.selectedSegmentIndex = 0;
    [typeSegmentControl addTarget:self action:@selector(typeSegmentControlDidChange:) forControlEvents:UIControlEventValueChanged];
    [self.view addSubview:typeSegmentControl];

    //摄像头
    UIButton *cameraButton = [UIButton buttonWithType:UIButtonTypeCustom];
    cameraButton.frame = CGRectMake(0, 0, self.view.width, 50);
    cameraButton.center = CGPointMake(self.view.center.x, self.view.center.y - 50);
    cameraButton.backgroundColor = [UIColor clearColor];
    cameraButton.tag = TMF_OCR_MAINCAMERA_DEFAULTBUTTON_TAG;
    [cameraButton setTitle:[@"camera(default)" localizedString] forState:UIControlStateNormal];
    [cameraButton setTitleColor:[UIColor blueColor] forState:UIControlStateNormal];
    [self.view addSubview:cameraButton];
    [cameraButton addTarget:self action:@selector(cameraButtonDidClick:) forControlEvents:UIControlEventTouchUpInside];

    UIButton *cameraRectButton = [UIButton buttonWithType:UIButtonTypeCustom];
    cameraRectButton.frame = CGRectMake(0, 0, self.view.width, 50);
    cameraRectButton.center = CGPointMake(self.view.center.x, self.view.center.y + 20);
    cameraRectButton.backgroundColor = [UIColor clearColor];
    cameraRectButton.tag = TMF_OCR_MAINCAMERA_RECT_TAG;
    [cameraRectButton setTitle:[@"camera(specific area)" localizedString] forState:UIControlStateNormal];
    [cameraRectButton setTitleColor:[UIColor blueColor] forState:UIControlStateNormal];
    [self.view addSubview:cameraRectButton];
    [cameraRectButton addTarget:self action:@selector(cameraButtonDidClick:) forControlEvents:UIControlEventTouchUpInside];

    //本地相册
    UIButton *photoButton = [UIButton buttonWithType:UIButtonTypeCustom];
    photoButton.frame = CGRectMake(0, 0, self.view.width, 50);
    photoButton.center = CGPointMake(self.view.center.x, self.view.center.y + 90);
    photoButton.backgroundColor = [UIColor clearColor];
    [photoButton setTitle:[@"local album" localizedString] forState:UIControlStateNormal];
    [photoButton setTitleColor:[UIColor blueColor] forState:UIControlStateNormal];
    [self.view addSubview:photoButton];
    [photoButton addTarget:self action:@selector(photoButtonDidClick:) forControlEvents:UIControlEventTouchUpInside];
}
#pragma mark - button
- (void)cameraButtonDidClick:(id)sender
{
    __weak typeof(&*self)weakSelf = self;
    [TMFOCRSystemAuthorizer requestAuthorizedWithType:TMFOCRSystemAuthorizerTypeCamera completion:^(TMFOCRSystemAuthorizerStatus status) {
        dispatch_async(dispatch_get_main_queue(), ^{
            switch (status) {
                case TMFOCRSystemAuthorizerStatusDenied:
                {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
                    UIAlertView *alterView = [[UIAlertView alloc] initWithTitle:nil message:[@"please enable camera permission" localizedString] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
                    [alterView show];
#pragma clang diagnostic pop
                }
                    break;
                case TMFOCRSystemAuthorizerStatusAuthorized:
                {
                    TMFOCRCameraViewController *vc = [[TMFOCRCameraViewController alloc] initWithScanType:weakSelf.scanType];
                    if ([(UIButton *)sender tag] == TMF_OCR_MAINCAMERA_RECT_TAG) {
                        vc.enableInterestRect = YES;
                    }
                    [weakSelf.navigationController pushViewController:vc animated:YES];
                }
                    break;
                default:
                    break;
            }
        });
    }];
}

//本地相机调取
- (void)photoButtonDidClick:(id)sender
{
    __weak typeof(&*self)weakSelf = self;
    [TMFOCRSystemAuthorizer requestAuthorizedWithType:TMFOCRSystemAuthorizerTypePhoto completion:^(TMFOCRSystemAuthorizerStatus status) {
        dispatch_async(dispatch_get_main_queue(), ^{
            switch (status) {
                case TMFOCRSystemAuthorizerStatusDenied:
                {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
                    UIAlertView *alterView = [[UIAlertView alloc] initWithTitle:nil message:[@"please enable album permission" localizedString] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
                    [alterView show];
#pragma clang diagnostic pop
                }
                    break;
                case TMFOCRSystemAuthorizerStatusAuthorized:
                {
                    //相册选取
                    UIImagePickerController *pickerVC = [[UIImagePickerController alloc] init];
                    pickerVC.delegate = weakSelf;
                    [weakSelf presentViewController:pickerVC animated:YES completion:nil];
                }
                    break;
                default:
                    break;
            }
        });
    }];
}

- (void)typeSegmentControlDidChange:(UISegmentedControl *)sender
{
    switch (sender.selectedSegmentIndex) {
        case 0:
            self.scanType = TMFOCRScanTypeBankcard;
            break;
        case 1:
            self.scanType = TMFOCRScanTypeIDcardAvatar;
            break;
        case 2:
            self.scanType = TMFOCRScanTypeIDcardNational;
            break;
        default:
            break;
    }
}

#pragma mark - image picker delegate
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *,id> *)info
{
    UIImage *pickerImage = [info objectForKey:UIImagePickerControllerOriginalImage];
    TMFOCRProcessViewController *vc = [[TMFOCRProcessViewController alloc] initWithOriginalImage:pickerImage scanType:self.scanType];
    [self.navigationController pushViewController:vc animated:NO];
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - private
- (void)readPhotoWithInfo:(NSDictionary<NSString *,id> *)info
{
    
}

@end
