//
//  TMFOCRProcessViewController.m
//  EMOCRDetect
//
//  Created by emmia on 2019/4/18.
//  Copyright © 2019 Tencent Technology(Shenzhen) Company Limited. All rights reserved.
//

#import "TMFOCRProcessViewController.h"
#import "TMFOCRDetectUtil.h"

#import <QMUIKit/QMUIKit.h>
#import "NSString+Localize.h"

@interface TMFOCRProcessViewController () <UIAlertViewDelegate>

/**
 @brief UIActivityIndicatorView
 */
@property (nonatomic, strong) UIActivityIndicatorView *activityIndicator;

/**
 @brief 原始图片
 */
@property (nonatomic, strong) UIImage *originalImage;

/**
 @brief OCR识别类型
 */
@property (nonatomic, assign) TMFOCRScanType scanType;

@end

@implementation TMFOCRProcessViewController

- (instancetype)initWithOriginalImage:(UIImage *)originalImage scanType:(TMFOCRScanType)scanType {
    self = [super init];
    if (self) {
        self.originalImage = originalImage;
        self.scanType = scanType;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.

    [self setupView];
    [self setupData];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];

    if (self.navigationController == nil && self.presentedViewController == nil) {
        if (self.delegate && [self.delegate respondsToSelector:@selector(OCRViewController:didCompleteWithResult:)]) {
            [self.delegate OCRViewController:self didCompleteWithResult:nil];
        }
    }
}

#pragma mark - setup
- (void)setupView {
    self.view.backgroundColor = [UIColor whiteColor];

    self.activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:(UIActivityIndicatorViewStyleGray)];
    self.activityIndicator.center = self.view.center;
    self.activityIndicator.color = [UIColor redColor];
    self.activityIndicator.hidesWhenStopped = NO;
    [self.view addSubview:self.activityIndicator];

    UILabel *waitingLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 200, 50)];
    waitingLabel.center = CGPointMake(self.view.center.x, self.view.center.y - 30);
    waitingLabel.textAlignment = NSTextAlignmentCenter;
    waitingLabel.textColor = [UIColor blackColor];
    waitingLabel.text = [@"processing, please wait..." localizedString];
    [self.view addSubview:waitingLabel];

    [self.activityIndicator startAnimating];
}

- (void)setupData {
    __weak typeof(&*self) weakSelf = self;
    [[TMFOCRScanService sharedService]
        scanOCRScanWithStillImage:self.originalImage
                         scanType:self.scanType
                   rectOfInterest:CGRectNull completeHandler:^(NSError *_Nullable error, NSArray<__kindof TMFOCRScanModel *> *_Nullable resultList) {
                       dispatch_async(dispatch_get_main_queue(), ^{
                           [weakSelf.activityIndicator stopAnimating];
                           NSString *result = nil;
                           if (error || [resultList count] <= 0) {
                               result = [NSString stringWithFormat:[@"failed:%ld" localizedString], (long)error.code];

                               if (self.delegate && [self.delegate respondsToSelector:@selector(OCRViewController:didCompleteWithResult:)]) {
                                   [self.delegate OCRViewController:self didCompleteWithResult:nil];
                                   //退出当前界面
                                   [self.navigationController popViewControllerAnimated:YES];
                               }
                           } else {
                               result = [TMFOCRDetectUtil ocrResultFromOcrModel:[resultList firstObject]];

                               if (self.delegate && [self.delegate respondsToSelector:@selector(OCRViewController:didCompleteWithResult:)]) {
                                   [self.delegate OCRViewController:self didCompleteWithResult:[resultList firstObject]];
                                   //退出当前界面
                                   [self.navigationController popViewControllerAnimated:YES];
                               }
                           }
                           if (!self.delegate) {
                               QMUIAlertAction *action1 =
                                   [QMUIAlertAction actionWithTitle:@"OK" style:QMUIAlertActionStyleCancel
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                //退出当前界面
                                                                [self.navigationController popViewControllerAnimated:YES];
                                                            }];
                               QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:nil message:result
                                                                                                     preferredStyle:QMUIAlertControllerStyleAlert];
                               [alertController addAction:action1];
                               [alertController showWithAnimated:YES];
                           }

                           self.delegate = nil;
                       });
                   }];
}

@end
