//
//  TMFInstructionViewController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2020/12/8.
//  Copyright © 2020 Tencent. All rights reserved.
//

#import "TMFInstructionViewController.h"
#import "TMFInstructionCenter.h"
#import "NSString+Localize.h"

#define TMF_UD_INSTRUCTION_ID_KEY @"com.tencent.tmf.tmfinstruction.defaultId"
#define TMF_INSTRUCTION_FORMATE @"Instruction ID: 【%@】"
#define TMF_INSTRUCTION_DEFAULT_ID 6238

@interface TMFInstructionViewController ()
/**
 @brief 云指令ID
 */
@property (nonatomic, copy) NSString *instructionIDString;

/**
 @brief UILabel
 */
@property (nonatomic, strong) UILabel *titleLabel;

/**
 @brief 云指令消息展示
 */
@property (nonatomic, strong) UITextView *instructionInfoView;

/**
 @brief 云指令ID容器，避免重复设置云指令监听及回调
 */
@property (nonatomic, strong) NSMutableSet *instructionIDSet;

/**
 @brief 云指令回调处理
 */
@property (nonatomic, copy)TMFInstructionHandler instructionHandler;

@end

@implementation TMFInstructionViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor colorWithWhite:0.97 alpha:1];
    NSString *insID = [[NSUserDefaults standardUserDefaults] objectForKey:TMF_UD_INSTRUCTION_ID_KEY];
    _instructionIDString = insID?:@(TMF_INSTRUCTION_DEFAULT_ID).stringValue;
    
    // Do any additional setup after loading the view.
    [self createInstructionViewControllerNavigtaions];
    [self createInstructionViewControllerSubViews];
}

- (void)createInstructionViewControllerNavigtaions{
    UIBarButtonItem *exitItem = [[UIBarButtonItem alloc] initWithTitle:@"Setting" style:UIBarButtonItemStyleDone target:self
                                                                action:@selector(clickEditButton)];
    self.navigationItem.rightBarButtonItem = exitItem;
}

- (void)createInstructionViewControllerSubViews{

    CGFloat top = CGRectGetMaxY([self.view convertRect:self.navigationController.navigationBar.bounds
                                              fromView:self.navigationController.navigationBar]);
    top += 40;
    _titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(20, top, CGRectGetWidth(self.view.bounds) - 20 - 20 - 80 , 40)];
    [self.view addSubview:_titleLabel];
    _titleLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleBottomMargin;
    _titleLabel.text = [NSString stringWithFormat:TMF_INSTRUCTION_FORMATE, _instructionIDString];
    
    UIButton *btn = [UIButton buttonWithType:UIButtonTypeSystem];
    [btn setTitle:@"Update" forState:UIControlStateNormal];
    btn.backgroundColor = [UIColor colorWithWhite:1 alpha:1];
    btn.layer.cornerRadius = 5.0;
    btn.clipsToBounds = YES;
    btn.frame = CGRectMake(CGRectGetMaxX(_titleLabel.frame) + 10, top, 70, 40);
    btn.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleBottomMargin;
    [self.view addSubview:btn];
    [btn addTarget:self action:@selector(addInstrunctionListener) forControlEvents:UIControlEventTouchUpInside];
    
    
    _instructionInfoView = [[UITextView alloc]
        initWithFrame:CGRectMake(20, top + 60,
                                 CGRectGetWidth(self.view.bounds) - 20 - 20, CGRectGetHeight(self.view.bounds) - top - 60 - 60)];
    [self.view addSubview:_instructionInfoView];
    _instructionInfoView.backgroundColor = [UIColor colorWithWhite:0.91 alpha:1];
    _instructionInfoView.editable = NO;
    _instructionInfoView.font = [UIFont systemFontOfSize:17];
    _instructionInfoView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleHeight;
}

#pragma mark - Edit
- (void)clickEditButton{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"set monitor instruction id" localizedString] message:@"Instruction ID" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = @"instruction id";
        textField.text = self.instructionIDString;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        
        NSString *insID = alertController.textFields[0].text;
        self.instructionIDString = insID;
        self.titleLabel.text = [NSString stringWithFormat:TMF_INSTRUCTION_FORMATE, insID];
        
        [[NSUserDefaults standardUserDefaults] setObject:insID forKey:TMF_UD_INSTRUCTION_ID_KEY];
        
        [self addInstrunctionListener];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)addInstrunctionListener{
    [self appendInstructionInfoViewMessage:@"observed updates ..."];
    if ([self.instructionIDSet containsObject:self.instructionIDString]) {
        [[TMFInstructionCenter defaultCenter] checkForUpdates];
        return;
    }
    
    [self.instructionIDSet addObject:self.instructionIDString];
    [[TMFInstructionCenter defaultCenter] observeID:self.instructionIDString.integerValue handler:self.instructionHandler];
    [[TMFInstructionCenter defaultCenter] checkForUpdates];
}


- (NSMutableSet *)instructionIDSet{
    if (!_instructionIDSet) {
        _instructionIDSet = [[NSMutableSet alloc] init];
    }
    return _instructionIDSet;
}

- (TMFInstructionHandler)instructionHandler{
    if (!_instructionHandler) {
        __weak typeof(self)weakSelf = self;
        TMFInstructionHandler handler = ^(TMFInstruction * _Nonnull instruction) {
            if (instruction.instructionID != weakSelf.instructionIDString.integerValue) {
                return;
            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
                formatter.dateFormat = @"yyyy-MM-dd HH:mm:ss";
                NSString *timeString = [formatter stringFromDate:[NSDate date]];
                NSMutableString *resultString = [[NSMutableString alloc] initWithFormat:@"【%@】\n\n",timeString];
                [resultString appendString:@"【Result】:\n"];
                [resultString appendFormat:@"TaskID: %lld\n",instruction.taskID];
                [resultString appendFormat:@"TaskSeqno: %lld\n",instruction.taskSeqNo];
                [resultString appendFormat:@"CmdId: %ld\n",(long)instruction.instructionID];
                [resultString appendString:@"Params:\n"];
                for (NSArray *value in instruction.values) {
                    [resultString appendFormat:@"  arg:%@\n",value];
                }
                [weakSelf appendInstructionInfoViewMessage:resultString];
            });
        };
        _instructionHandler = [handler copy];
    }
    return _instructionHandler;
}

- (void)appendInstructionInfoViewMessage:(NSString *)message{
    NSString *string = self.instructionInfoView.text;
    string = [string stringByAppendingString:@"\n"];
    string = [string stringByAppendingString:message];
    self.instructionInfoView.text = string;
}

@end
