//
//  TMFProfileViewController.m
//  TMFDemo
//
//  Created by bentonxiu on 2019/8/21.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFProfileViewController.h"
#import "TMFInstructionViewController.h"
#import "TMFPush.h"
#import "TMFProfile.h"
#import "NSString+Localize.h"

#define ROW_COMMON              [@"common report" localizedString]
#define ROW_USERINFO            [@"user info report" localizedString]
#define ROW_CUSTOMTAG           [@"custom tag report" localizedString]
#define ROW_INSTRUCTION         [@"pull command" localizedString]
#define ROW_COMMON_DETAIL       @"Common"
#define ROW_USERINFO_DETAIL     @"UserInfo"
#define ROW_CUSTOMTAG_DETAIL    @"CustomTag"
#define ROW_INSTRUCTION_DETAIL  @"Instruction"

#define kCustomTagKey(idx) [NSString stringWithFormat:@"tag%d",idx]
#define kCustomTagValue(idx) [NSString stringWithFormat:@"tag%d_value%d",idx,idx]

@interface TMFProfileViewController ()
@end

@implementation TMFProfileViewController

- (instancetype)init
{
    self = [super init];
    if (self) {
        
        [self initDataSource];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(profileDidReport:)
                                                     name:TMFProfileDidReportNotification
                                                   object:nil];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
}

#pragma mark - Data Functions

- (void)initDataSource
{
    [super initDataSource];
    
    QMUIOrderedDictionary<NSString *, NSString *> *dataSourceWithDetailText = [[QMUIOrderedDictionary alloc] init];
    [dataSourceWithDetailText setObject:ROW_COMMON_DETAIL forKey:ROW_COMMON];
    [dataSourceWithDetailText setObject:ROW_USERINFO_DETAIL forKey:ROW_USERINFO];
    [dataSourceWithDetailText setObject:ROW_CUSTOMTAG_DETAIL forKey:ROW_CUSTOMTAG];
    [dataSourceWithDetailText setObject:ROW_INSTRUCTION_DETAIL forKey:ROW_INSTRUCTION];
    
    self.dataSourceWithDetailText = dataSourceWithDetailText;
}

- (void)reloadTableView
{
    [self initDataSource];
    [self.tableView reloadData];
}

- (void)didSelectCellWithTitle:(NSString *)title
{
    
    if ([title isEqualToString:ROW_COMMON]) {
        [self updateCommonProfiles];
    } else if ([title isEqualToString:ROW_USERINFO]) {
        [self updateUserInfos];
    } else if ([title isEqualToString:ROW_CUSTOMTAG]) {
        [self updateCustomTags];
    } else if ([title isEqualToString:ROW_INSTRUCTION]) {
        [self createInstructionViewController];
    }
}

- (void)setupNavigationItems
{
    [super setupNavigationItems];
    self.title = @"TMFProfile";
}

#pragma mark - Private Functions

- (void)updateCommonProfiles
{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_COMMON message:nil preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = [@"country" localizedString];
        textField.text = @"中国";
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = [@"province" localizedString];
        textField.text = @"广东";
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = [@"city" localizedString];
        textField.text = @"深圳";
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = [@"department" localizedString];
        textField.text = @"研发部";
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:[@"cancel" localizedString] style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:[@"update" localizedString] style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        NSString *country = alertController.textFields[0].text;
        NSString *province = alertController.textFields[1].text;
        NSString *city = alertController.textFields[2].text;
        NSString *department = alertController.textFields[3].text;
        [[TMFProfileReporter defaultReporter] updateProfileWithID:TMFProfileIDCountry stringValue:country];
        [[TMFProfileReporter defaultReporter] updateProfileWithID:TMFProfileIDProvince stringValue:province];
        [[TMFProfileReporter defaultReporter] updateProfileWithID:TMFProfileIDCity stringValue:city];
        [[TMFProfileReporter defaultReporter] updateProfileWithID:TMFProfileIDDepartment stringValue:department];
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateUserInfos
{
    NSString *userID = @"x3dsw23lh";
    NSString *deviceToken = [TMFPushTokenManager defaultTokenManager].deviceTokenString;
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_USERINFO message:nil preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = [@"custom user ID: " localizedString];
        textField.text = userID;
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.placeholder = [@"remote push Token: " localizedString];
        textField.text = deviceToken;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:[@"cancel" localizedString] style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:[@"update" localizedString] style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        NSString *uid = alertController.textFields[0].text;
        NSString *token = alertController.textFields[1].text;
        [[TMFProfileReporter defaultReporter] updateCustomizedUserID:uid];
        [[TMFProfileReporter defaultReporter] updateRemoteNotificationToken:token];
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateCustomTags {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_CUSTOMTAG message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    for (int index = 1; index < 4; index ++) {
        [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
            textField.placeholder = [NSString stringWithFormat:[@"custom tag type %d: " localizedString], index];
            textField.text = kCustomTagKey(index);
        }];
        [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
            textField.placeholder = [NSString stringWithFormat:[@"custom tag %d: " localizedString], index];
            textField.text = kCustomTagValue(index);
        }];
    }
    [alertController addAction:[UIAlertAction actionWithTitle:[@"cancel" localizedString] style:UIAlertActionStyleCancel handler:nil]];

    [alertController addAction:[UIAlertAction actionWithTitle:[@"update" localizedString] style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        NSOperationQueue *mainOperationQueue = [NSOperationQueue mainQueue];
        NSMutableString *alertMessage = [[NSMutableString alloc] init];
        NSInvocationOperation *alertOperation = [[NSInvocationOperation alloc] initWithTarget:self selector:@selector(showAlertMessage:)
                                                                                       object:alertMessage];
        NSLock *lock = [[NSLock alloc] init];
        for (int index = 0; index < 3; index ++) {
            __block NSString *msg = nil;
            NSBlockOperation *blockOperation = [NSBlockOperation blockOperationWithBlock:^{
                [lock lock];
                [alertMessage appendString:msg];
                [lock unlock];
            }];
            [alertOperation addDependency:blockOperation];
            
            NSString *key = alertController.textFields[index * 2].text;
            NSString *value = alertController.textFields[index * 2 + 1].text;
            if (key.length > 0) {
                [[TMFProfileReporter defaultReporter] updateCustomizedTagWithKey:key value:value responseHandler:^(NSError *_Nullable error) {
                    NSLog(@"【上报结束%d Key:%@, Value:%@, Error:%@", index + 1, key, value, error);
                    if (error) {
                        msg = [NSString stringWithFormat:@"【自定义标签%d】key:%@, value:%@ 上报失败：%@\n", index + 1, key, value, error.localizedDescription];
                    } else {
                        msg = [NSString stringWithFormat:@"【自定义标签%d】key:%@, value:%@ 上报成功\n", index + 1, key, value];
                    }
                    [blockOperation start];
                }];
            }
        }
        [mainOperationQueue addOperation:alertOperation];
    }]];

    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)createInstructionViewController{
    TMFInstructionViewController *instructionVC = [[TMFInstructionViewController alloc] init];
    [self.navigationController pushViewController:instructionVC animated:YES];
}

#pragma mark - Notification Functions

- (void)profileDidReport:(NSNotification *)notification {
    dispatch_async(dispatch_get_main_queue(), ^{
        NSDictionary *errorInfo = notification.userInfo[TMFProfileDidReportErrorKey];
        NSDictionary *reportedInfo = notification.userInfo[TMFProfileDidReportItemsKey];

        UIAlertController *alertController = nil;
        // 处理服务器错误问题
        if ([errorInfo isKindOfClass:[NSError class]]) {
            alertController = [UIAlertController alertControllerWithTitle:@"Error" message:((NSError *)errorInfo).domain
                                                           preferredStyle:UIAlertControllerStyleAlert];
        } else {
            NSMutableString *reportedResult = [NSMutableString string];
            if ([errorInfo isKindOfClass:[NSDictionary class]] && errorInfo.count > 0) {
                for (NSString *key in errorInfo.allKeys) {
                    [reportedResult appendFormat:@"【Error】%@\n", errorInfo[key]];
                    [reportedResult appendFormat:@"【%@】%@\n\n", key, reportedInfo[key]];
                }
                alertController = [UIAlertController alertControllerWithTitle:[@"report failed" localizedString] message:reportedResult
                                                               preferredStyle:UIAlertControllerStyleAlert];
            } else if ([reportedInfo isKindOfClass:[NSDictionary class]] && reportedInfo.count > 0) {
                for (NSString *key in reportedInfo.allKeys) {
                    [reportedResult appendFormat:@"【%@】%@\n", key, reportedInfo[key]];
                }
                alertController = [UIAlertController alertControllerWithTitle:[@"report successed" localizedString] message:reportedResult
                                                               preferredStyle:UIAlertControllerStyleAlert];
            }
        }

        if (alertController) {
            [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
            [self presentViewController:alertController animated:YES completion:nil];
        }
    });
}
- (void)showAlertMessage:(NSString *)message{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
    [alert addAction:action];
    [self presentViewController:alert animated:YES completion:nil];
}


@end
