//
//  TMFURLRouterTest1VC.m
//  TMFDemo
//
//  Created by gavinjwxu on 2022/4/11.
//  Copyright © 2022 Tencent. All rights reserved.
//

#import "TMFURLRouterTest1VC.h"
#import "TMFRouter.h"
#import "NSString+Localize.h"

#define TMFRouter_TestVC_1 @"tmf://router/test/1"
#define TMFRouter_TestVC_1_Push @"tmf://router/test/1/push"

@interface TMFURLRouterTest1VC ()<TMFURLRouterProtocol>

@property (nonatomic, copy) NSString *titleStr;
@property (nonatomic, copy) void(^testBlock)(void);
@property (nonatomic, copy) void(^completion)(id);
@property (nonatomic, copy) void(^failure)(id);
@property (nonatomic,assign) BOOL isAppVC;

@end

@implementation TMFURLRouterTest1VC

+ (void)registerRouter {
    [TMFURLRouter registerURLPattern:TMFRouter_TestVC_1 toObjectHandler:^id(NSDictionary *routerParameters) {
        TMFURLRouterTest1VC *vc = [[TMFURLRouterTest1VC alloc] init];
        vc.titleStr = routerParameters[@"titleStr"];
        return vc;
    }];
    
    [TMFURLRouter registerURLPattern:TMFRouter_TestVC_1_Push toHandler:^(NSDictionary *routerParameters) {
        TMFURLRouterTest1VC *vc = [[TMFURLRouterTest1VC alloc] init];
        vc.hidesBottomBarWhenPushed = YES;
        if (routerParameters[@"titleStr"]) {
            vc.titleStr = routerParameters[@"titleStr"];
        } else {
            vc.titleStr = routerParameters.routerUserInfo[@"titleStr"];
        }
        vc.testBlock = routerParameters.routerUserInfo[@"testBlock"];
        vc.completion = routerParameters[TMFRouterParameterCompletion];
        vc.failure = routerParameters[TMFRouterParameterFailure];
        [[UIViewController currentNavigationViewController] pushViewController:vc animated:YES];
//        UINavigationController *navi = [[UINavigationController alloc] initWithRootViewController:vc];
//        [[UIViewController currentNavigationViewController] presentViewController:vc animated:YES completion:nil];
    }];
    
    [TMFURLRouter registerURLPattern:@"tmf://router/microApp/1" toObjectHandler:^id(NSDictionary *routerParameters) {
        TMFURLRouterTest1VC *vc = [[TMFURLRouterTest1VC alloc] init];
        vc.titleStr = routerParameters[@"titleStr"];
        vc.isAppVC = YES;
        vc.hidesBottomBarWhenPushed = YES;
        [[UIViewController currentNavigationViewController] pushViewController:vc animated:YES];
        return vc;
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.title = self.titleStr;
    [self.navigationController.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName : [UIColor redColor]}];
    
    if (self.testBlock) {
        self.testBlock();
    }
    if (self.completion) {
        self.completion(@(YES));
    }
    
    if (self.isAppVC) {
        QMUIButton *button = [[QMUIButton alloc] initWithFrame:CGRectMake(20, 100, 300, 30)];
        [button setTitle:[@"open the second VC of this MicroApp" localizedString] forState:UIControlStateNormal];
        button.titleLabel.font = UIFontMake(16);
        [button setTitleColor:UIColor.darkGrayColor forState:UIControlStateNormal];
        button.backgroundColor = TableViewCellSelectedBackgroundColor;
        button.qmui_automaticallyAdjustTouchHighlightedInScrollView = YES;
        [self.view addSubview:button];
        [button addTarget:self action:@selector(pushSecondVC:) forControlEvents:UIControlEventTouchUpInside];
    }
}

- (void)pushSecondVC:(id)sender {
    [TMFURLRouter openURL:[NSString stringWithFormat:@"tmf://router/test/2/push?isAppVC=1&titleStr=%@",self.titleStr]];
}

- (void)dealloc {
    NSLog(@"dealloc");
}

@end
