//
//  TMFSSLTester.m
//  TMFDemo
//
//  Created by klaudz on 16/5/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFSSLTester.h"

#import "TMFSM2Coder.h"
#import "TMFSM3Coder.h"
#import "TMFSM4Coder.h"

#import "NSData+MQQHexString.h"
#import "NSString+Localize.h"

@implementation TMFSSLTester

+ (NSString *)testSM2Sign {
//    NSString *msg = @"SM2 Test: sign";
//    NSString *userId = @"Admin";
    
    NSString *msg = @"qwer";
    NSString *userId = @"1234567890";
    
    NSData *msgData = [msg dataUsingEncoding:NSUTF8StringEncoding];
    NSData *userData = [userId dataUsingEncoding:NSUTF8StringEncoding];
    
    NSData *privateKey = [NSData mqqDataWithHexString:@"8A0CEF075216F8E150604B2D627CF973CA274420964B648C6F6F77580467AE6A"];
    NSData *publicKey_X = [NSData mqqDataWithHexString:@"0128BEA05C9F9D10B5B65C8E3A1AAC16CA4812758908D55DDE44979BA6ABB4EF"];
    NSData *publicKey_Y = [NSData mqqDataWithHexString:@"1950D9A40735BDD6FF11F455C83B8B79F2B68EF91BC9FC446B8301D3DF7E5366"];
    
    NSData *rData = nil;
    NSData *sData = nil;
    
    TMFSMStatus result = TMFSM2Sign(msgData, userData, publicKey_X, publicKey_Y, privateKey, &rData, &sData);
    
    NSLog(@"签名结果：%ld",(long)result);

    NSLog(@"密文r str=%@",[rData mqqHexString]);

    NSLog(@"密文s str=%@",[sData mqqHexString]);
    
    return [[rData mqqHexString] stringByAppendingString:[sData mqqHexString]];
}

+ (NSString *)testSM2Verify:(NSString *)signStr {
//    NSString *msg = @"SM2 Test: sign";
//    NSString *userId = @"Admin";
    
    NSString *msg = @"qwer";
    NSString *userId = @"1234567890";
    
    NSData *msgData = [msg dataUsingEncoding:NSUTF8StringEncoding];
    NSData *userData = [userId dataUsingEncoding:NSUTF8StringEncoding];
    
    NSData *publicKey_X = [NSData mqqDataWithHexString:@"0128BEA05C9F9D10B5B65C8E3A1AAC16CA4812758908D55DDE44979BA6ABB4EF"];
    NSData *publicKey_Y = [NSData mqqDataWithHexString:@"1950D9A40735BDD6FF11F455C83B8B79F2B68EF91BC9FC446B8301D3DF7E5366"];
    
    NSString *rStr = [signStr substringToIndex:signStr.length/2];
    NSString *sStr = [signStr substringFromIndex:signStr.length/2];
    
    NSData *rData = [NSData mqqDataWithHexString:rStr];
    NSData *sData = [NSData mqqDataWithHexString:sStr];
    
    TMFSMStatus result = TMFSM2Verify(msgData, userData, publicKey_X, publicKey_Y, rData, sData);
    
    NSString *log = nil;
    if (result == 0) {
        log = [@"verify success" localizedString];
    } else {
        log = [NSString stringWithFormat:[@"verify failed, error code: %ld" localizedString],(long)result];
    }
    return log;
}

+ (NSString *)testSM2
{
    NSString *srcString = @"SM2 Test: encrypt, decrypt, sign and verify.";
    NSString *tstString = nil;
    BOOL isEqual = NO;
    
    NSData *srcData = [srcString dataUsingEncoding:NSUTF8StringEncoding];
    NSData *privateKey = [NSData mqqDataWithHexString:@"8A0CEF075216F8E150604B2D627CF973CA274420964B648C6F6F77580467AE6A"];
    NSData *publicKey_X = [NSData mqqDataWithHexString:@"0128BEA05C9F9D10B5B65C8E3A1AAC16CA4812758908D55DDE44979BA6ABB4EF"];
    NSData *publicKey_Y = [NSData mqqDataWithHexString:@"1950D9A40735BDD6FF11F455C83B8B79F2B68EF91BC9FC446B8301D3DF7E5366"];
    NSData *tstData = nil;
    
    NSData *c1 = nil;
    NSData *c2 = nil;
    NSData *c3 = nil;
    if (/* 加密 */ TMFSM2Encrypt(srcData, publicKey_X, publicKey_Y, &c1, &c3, &c2) == 0 &&
        /* 解密 */ TMFSM2Decrypt(c1, c3, c2, privateKey, &tstData) == 0) {
        tstString = [[NSString alloc] initWithData:tstData encoding:NSUTF8StringEncoding];
        isEqual = [srcString isEqualToString:tstString];
    }
    
    NSLog(@"[SM2]      srcString : %@", srcString);
    NSLog(@"[SM2]      tstString : %@", tstString);
    NSLog(@"[SM2]      result    : %@", isEqual ? @"Verified" : @"Failed");
    
    NSMutableString *log = [NSMutableString string];
    [log appendFormat:[@"origin data: %@\n" localizedString], srcString];
    [log appendFormat:[@"decrypted data: %@\n" localizedString], tstString];
    [log appendFormat:[@"test result: %@" localizedString], isEqual ? [@"passed" localizedString] : [@"failed" localizedString]];
    return log;
}

+ (NSString *)testSM3
{
//    NSString *srcString = @"SM3 Test: encrypt.你好";
    NSString *srcString = @"qwer";
    NSData *srcData = [srcString dataUsingEncoding:NSUTF8StringEncoding];
    
    NSData *result = nil;
    
    TMFSM3(srcData, &result);
    
    NSString *str = [result mqqHexString];
    
    NSLog(@"%@",str);
    
    return [NSString stringWithFormat:[@"encrypt success, the result HexString: %@" localizedString],str];
}

+ (NSString *)testSM3HMac
{
//    NSString *srcString = @"SM3 HMac Test: encrypt.你好";
    NSString *srcString = @"qwer";
    NSData *srcData = [srcString dataUsingEncoding:NSUTF8StringEncoding];
    
    NSString *keyString = @"123456789";
    NSData *keyData = [keyString dataUsingEncoding:NSUTF8StringEncoding];
    
    NSData *result = nil;
    
    TMFSM3HMac(srcData, keyData, &result);
    
    NSString *str = [result mqqHexString];
    
    NSLog(@"%@",str);
    
    return [NSString stringWithFormat:[@"encrypt success, the result HexString: %@" localizedString],str];
}

+ (NSString *)testSM4ECB
{
    NSString *srcString = @"SM4(ECB) Test: encrypt, decrypt, sign and verify.";
    NSString *tstString = nil;
    BOOL isEqual = NO;
    
    NSData *srcData = [srcString dataUsingEncoding:NSUTF8StringEncoding];
    NSData *key = [NSData mqqDataWithHexString:@"9874561230ABCDEF3698521470BFEDCA"];
    NSData *tstData = nil;
    
    NSData *encryptedData = nil;
    if (/* 加密 */ TMFSM4EncryptECB(srcData, key, &encryptedData) == 0 &&
        /* 解密 */ TMFSM4DecryptECB(encryptedData, key, &tstData) == 0) {
        tstString = [[NSString alloc] initWithData:tstData encoding:NSUTF8StringEncoding];
        isEqual = [srcString isEqualToString:tstString];
    }
    
    NSLog(@"[SM4(ECB)] srcString : %@", srcString);
    NSLog(@"[SM4(ECB)] tstString : %@", tstString);
    NSLog(@"[SM4(ECB)] result    : %@", [srcString isEqualToString:tstString] ? @"Verified" : @"Failed");
    
    NSMutableString *log = [NSMutableString string];
    [log appendFormat:[@"origin data: %@\n" localizedString], srcString];
    [log appendFormat:[@"decrypted data: %@\n" localizedString], tstString];
    [log appendFormat:[@"test result: %@" localizedString], isEqual ? [@"passed" localizedString] : [@"failed" localizedString]];
    return log;
}

+ (NSString *)testSM4CBC
{
    NSString *srcString = @"SM4(CBC) Test: encrypt, decrypt, sign and verify.";
    NSString *tstString = nil;
    BOOL isEqual = NO;
    
    NSData *srcData = [srcString dataUsingEncoding:NSUTF8StringEncoding];
    NSData *key = [NSData mqqDataWithHexString:@"9874561230ABCDEF3698521470BFEDCA"];
    const uint8_t iv[] = { 0x98, 0x74, 0x56, 0x12, 0x30, 0xAB, 0xCD, 0xEF,
        0x36, 0x98, 0x52, 0x14, 0x70, 0xBF, 0xED, 0xCA };
    NSData *ivData = [NSData dataWithBytes:iv length:sizeof(iv)];
    NSData *tstData = nil;
    
    NSData *encryptedData = nil;
    if (/* 加密 */ TMFSM4EncryptCBC(srcData, key, ivData, &encryptedData) == 0 &&
        /* 解密 */ TMFSM4DecryptCBC(encryptedData, key, ivData, &tstData) == 0) {
        tstString = [[NSString alloc] initWithData:tstData encoding:NSUTF8StringEncoding];
        isEqual = [srcString isEqualToString:tstString];
    }
    
    NSLog(@"[SM4(CBC)] srcString : %@", srcString);
    NSLog(@"[SM4(CBC)] tstString : %@", tstString);
    NSLog(@"[SM4(CBC)] result    : %@", [srcString isEqualToString:tstString] ? @"Verified" : @"Failed");
    
    NSMutableString *log = [NSMutableString string];
    [log appendFormat:[@"origin data: %@\n" localizedString], srcString];
    [log appendFormat:[@"decrypted data: %@\n" localizedString], tstString];
    [log appendFormat:[@"test result: %@" localizedString], isEqual ? [@"passed" localizedString] : [@"failed" localizedString]];
    return log;
}

@end



