//
//  TMFSSLViewController.m
//  TMFDemo
//
//  Created by klaudz on 20/5/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFSSLViewController.h"
#import "TMFSSLTester.h"
#import "NSString+Localize.h"

@interface TMFSSLViewController ()

@property (nonatomic,copy) NSString *signStr;

@end

@implementation TMFSSLViewController

- (void)awakeFromNib {
    [super awakeFromNib];
    
    [self initDataSource];
}

#define ROW_SM2_SIGN [@"SM2 sign" localizedString]
#define ROW_SM2_VERIFY [@"SM2 verify" localizedString]
#define ROW_SM2     [@"SM2 encryption and decryption" localizedString]
#define ROW_SM3     [@"SM3 encryption" localizedString]
#define ROW_SM3_HMAC [@"SM3 HMac encryption" localizedString]
#define ROW_SM4_ECB @"SM4 (ECB)"
#define ROW_SM4_CBC @"SM4 (CBC)"

- (void)initDataSource {
    [super initDataSource];
    
    self.dataSource = @[
        ROW_SM2_SIGN,
        ROW_SM2_VERIFY,
        ROW_SM2,
        ROW_SM3,
        ROW_SM3_HMAC,
        ROW_SM4_ECB,
        ROW_SM4_CBC,
    ];
}

- (void)didSelectCellWithTitle:(NSString *)title {
    if ([title isEqualToString:ROW_SM2_SIGN]) {
        [self testSM2Sign];
    } else if ([title isEqualToString:ROW_SM2_VERIFY]) {
        [self testSM2Verify];
    } else if ([title isEqualToString:ROW_SM2]) {
        [self testSM2];
    } else if ([title isEqualToString:ROW_SM3]) {
        [self testSM3];
    } else if ([title isEqualToString:ROW_SM3_HMAC]) {
        [self testSM3HMac];
    } else if ([title isEqualToString:ROW_SM4_ECB]) {
        [self testSM4ECB];
    } else if ([title isEqualToString:ROW_SM4_CBC]) {
        [self testSM4CBC];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"TMFSSL";
}

#pragma mark -

- (void)testSM2Sign {
    NSString *log = [TMFSSLTester testSM2Sign];
    self.signStr = [log copy];
    log = [NSString stringWithFormat:[@"sign success, signed string: %@" localizedString],log];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM2_SIGN message:log preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)testSM2Verify {
    NSString *log = [TMFSSLTester testSM2Verify:self.signStr];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM2_VERIFY message:log preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)testSM2 {
    NSString *log = [TMFSSLTester testSM2];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM2 message:log preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)testSM3 {
    NSString *log = [TMFSSLTester testSM3];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM3 message:log preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)testSM3HMac {
    NSString *log = [TMFSSLTester testSM3HMac];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM3_HMAC message:log preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)testSM4ECB {
    NSString *log = [TMFSSLTester testSM4ECB];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM4_ECB message:log
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)testSM4CBC {
    NSString *log = [TMFSSLTester testSM4CBC];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_SM4_CBC message:log
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end
