//
//  SKGetKeyPositionDataNetImpl.m
//  safe_keyboard
//
//  Created by mac on 2020/4/28.
//  Copyright © 2020 tencent. All rights reserved.
//

#import "SKGetKeyDataImpl.h"
// Shark
#import "TMFSharkCenter.h"
#import "TMFSharkErrors.h"
#import "TMFSM4Coder.h"
#import "SKEEncryptType.h"
#import "SKKeyboardPosition.h"

@interface SKGetKeyDataImpl ()
/**
 @brief 映射请求数据
 */
@property (atomic, strong)  SKCSGetKeyPosition * requestData;

/**
 @brief 数据请求成功回调
 */
@property (atomic, copy)  SKGetKeyPositionCallback callbackRef;
@end

@implementation SKGetKeyDataImpl

- (id)copyWithZone:(NSZone *)zone {
    return [SKGetKeyDataImpl new];
}

- (void)setRequest:(SKCSGetKeyPosition *)requestDataModel {
    self.requestData = requestDataModel;
}

- (void)execute {
    TMFOCSharkCmdSettings *cmdSettings = [TMFOCSharkCmdSettings settings];
    [cmdSettings csSetCmdID: 1007 objectClass:[SKCSGetKeyPosition class]];
    [cmdSettings scSetCmdID:11007 objectClass:[SKSCGetKeyPosition class]];
    
    __weak typeof(self) weakself = self;
    [[TMFSharkCenter masterCenter]
        requestWithCmdSettings:cmdSettings
                      csObject:self.requestData completionHandler:^(__kindof SKSCGetKeyPosition *_Nullable resp, NSError *_Nullable error) {
                          bool failed = error || [error code] != 0;
                          if (failed) {
                              NSLog(@"request keyboard position data failed,  desc: %@", error != nil ? [error description] : @"unknown error!");
                              return;
                          }

                          if (!resp || resp.tars_ret != 0) {
                              NSLog(@"request keyboard position data failed, resp return failed,  retCode: %d", resp ? resp.tars_ret : -1);
                              return;
                          }

                          NSLog(@"request keyboard position data success");
                          if (weakself.callbackRef) {
                              weakself.callbackRef(resp);
                          }
                      }];
}

#pragma mark - NSData 非空判断
- (BOOL) checkNotNull:(NSData *) param {
    return param && param.length > 0;
}

- (void)setCallback:(SKGetKeyPositionCallback)callback {
    self.callbackRef = callback;
}

- (void)dealloc
{
    NSLog(@"get key position data, dealloc");
}
@end
