//
//  TMFSafeKeyboardPasswordViewController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2021/3/23.
//  Copyright © 2021 Tencent. All rights reserved.
//

#import "TMFSafeKeyboardPasswordViewController.h"

#import <TMFSafeKeyboard/TMFSafeKeyboard.h>
#import <TMFSafeKeyboard/TMFSafeKeyboardWrapperApi.h>
#import <TMFSafeKeyboard/TMFSafeKeyboardCheckApi.h>

#import <TMFShark/TMFSharkCenter.h>
#import "SKReqRecoverInputText.h"
#import "SKRespRecoverInputText.h"
#import "NSString+Localize.h"

@interface TMFSafeKeyboardPasswordViewController () <TMFSafeKeyboardDelegate, UITextFieldDelegate>

/**
 @brief user Text Field
 */
@property (nonatomic, strong) UITextField *userTextF;

/**
 @brief pasword Text Field
 */
@property (nonatomic, strong) UITextField *passwordTextF;

/**
 @brief repeat password Text Field
 */
@property (nonatomic, strong) UITextField *repeatPasswordTextF;

/**
 @brief confirm Button
 */
@property (nonatomic, strong) UIButton *confirmBtn;

/**
 @brief confirm Button 2
 */
@property (nonatomic, strong) UIButton *confirmBtn2;

/**
 @brief label1
 */
@property (nonatomic, strong) UILabel *label1;

/**
 @brief 密码强度设置
 */
@property (nonatomic, strong) UISlider *slider1;

/**
 @brief label2
 */
@property (nonatomic, strong) UILabel *label2;

/**
 @brief 密码连续相同位数设置
 */
@property (nonatomic, strong) UISlider *slider2;

@end

#define _kStandardFrame CGRectMake(20, y, w - 40, 40)

@implementation TMFSafeKeyboardPasswordViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.

    UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithTitle:[@"illustrate" localizedString] style:UIBarButtonItemStyleDone target:self action:@selector(showDocument)];
    self.navigationItem.rightBarButtonItem = item;

    CGFloat y = 120.f;
    CGFloat w = CGRectGetWidth(self.view.bounds);

    _userTextF = [[UITextField alloc] initWithFrame:_kStandardFrame];
    _userTextF.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    _userTextF.placeholder = [@"normal keyboard" localizedString];
    _userTextF.backgroundColor = [UIColor colorWithWhite:0.98 alpha:1];
    [self.view addSubview:_userTextF];

    y += 60.f;
    _passwordTextF = [[UITextField alloc] initWithFrame:_kStandardFrame];
    _passwordTextF.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    _passwordTextF.placeholder = [@"safety keyboard 1" localizedString];
    _passwordTextF.backgroundColor = [UIColor colorWithWhite:0.98 alpha:1];
    _passwordTextF.delegate = self;
    [self.view addSubview:_passwordTextF];

    y += 60.f;
    _repeatPasswordTextF = [[UITextField alloc] initWithFrame:_kStandardFrame];
    _repeatPasswordTextF.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    _repeatPasswordTextF.placeholder = [@"safety keyboard 2" localizedString];
    _repeatPasswordTextF.backgroundColor = [UIColor colorWithWhite:0.98 alpha:1];
    _repeatPasswordTextF.delegate = self;
    [self.view addSubview:_repeatPasswordTextF];

    y += 60.f;
    _confirmBtn = [UIButton buttonWithType:UIButtonTypeSystem];
    _confirmBtn.frame = _kStandardFrame;
    [_confirmBtn setTitle:@"check" forState:UIControlStateNormal];
    [_confirmBtn addTarget:self action:@selector(sendInfo:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_confirmBtn];
    _confirmBtn.tag = 10001;

    y += 60.f;
    _confirmBtn2 = [UIButton buttonWithType:UIButtonTypeSystem];
    _confirmBtn2.frame = _kStandardFrame;
    [_confirmBtn2 setTitle:@"double check" forState:UIControlStateNormal];
    [_confirmBtn2 addTarget:self action:@selector(sendInfo:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_confirmBtn2];
    _confirmBtn2.tag = 10002;

    y += 60.f;
    _label1 = [[UILabel alloc] initWithFrame:_kStandardFrame];
    [self.view addSubview:_label1];
    y += 40;
    _slider1 = [[UISlider alloc] initWithFrame:_kStandardFrame];
    _slider1.minimumValue = 0;
    _slider1.maximumValue = 4;
    _slider1.value = 4;
    [self.view addSubview:_slider1];
    [_slider1 addTarget:self action:@selector(valueChange:) forControlEvents:UIControlEventValueChanged];

    y += 60.f;
    _label2 = [[UILabel alloc] initWithFrame:_kStandardFrame];
    [self.view addSubview:_label2];
    y += 40;
    _slider2 = [[UISlider alloc] initWithFrame:_kStandardFrame];
    _slider2.minimumValue = 2;
    _slider2.maximumValue = 6;
    _slider2.value = 3;
    [self.view addSubview:_slider2];
    [_slider2 addTarget:self action:@selector(valueChange:) forControlEvents:UIControlEventValueChanged];
    [self valueChange:_slider1];
    [self valueChange:_slider2];

    TMFSafeKeyboard *keyboard = [[TMFSafeKeyboard alloc] init];
    keyboard.delegate = self;
    TMFSafeKeyboardWrapper *wrapper = [[TMFSafeKeyboardWrapper alloc] initWithKeyboard:keyboard];

    TMFSafeKeyboard *keyboard2 = [[TMFSafeKeyboard alloc] init];
    keyboard.delegate = self;
    TMFSafeKeyboardWrapper *wrapper2 = [[TMFSafeKeyboardWrapper alloc] initWithKeyboard:keyboard2];

    [TMFSafeKeyboardWrapperApi customKeyboardWithWrapper:wrapper inputView:_passwordTextF completeCallback:^(NSData *context, NSString *text) {
        NSLog(@"%s: %@", __func__, text);
    }];
    [TMFSafeKeyboardWrapperApi customKeyboardWithWrapper:wrapper2 inputView:_repeatPasswordTextF completeCallback:^(NSData *context, NSString *text) {
        NSLog(@"%s: %@", __func__, text);
    }];
}

- (void)sendInfo:(UIButton *)button {
    NSInteger pwdStrength = self.slider1.value;
    int repeatNum = self.slider2.value;

    // 客户端判断内容是否相同
    TMFSafeKeyboardCheckContent *content1 = [TMFSafeKeyboardCheckContent contentWithText:_passwordTextF.text
                                                                                 context:_passwordTextF.safeKeyboardWrapper.context];
    TMFSafeKeyboardCheckContent *content2 = [TMFSafeKeyboardCheckContent contentWithText:_repeatPasswordTextF.text
                                                                                 context:_repeatPasswordTextF.safeKeyboardWrapper.context];

    if (button.tag == 10001) {
        if (_passwordTextF.text.length == 0) {
            [self showAlertTitle:[@"error" localizedString] msg:[@"please enter password" localizedString]];
            return;
        }
        [TMFSafeKeyboardCheckApi textWeakCheckForContent:content1 passwordStrength:pwdStrength repeatNumber:repeatNum
                                              completion:^(BOOL pass, NSError *_Nullable error) {
                                                  NSLog(@"[TMFSafeKeyboard-Check]%d, %@", pass, error);
                                                  [self showAlertTitle:pass ? [@"passed" localizedString] : [@"failed" localizedString]
                                                                   msg:[NSString stringWithFormat:[@"textfield 1 weak password %@" localizedString], pass ? [@"passed" localizedString] : [@"failed" localizedString]]];
                                              }];
    } else if (button.tag == 10002) {
        if (_passwordTextF.text.length == 0 && _repeatPasswordTextF.text.length == 0) {
            [self showAlertTitle:[@"error" localizedString] msg:[@"please enter password twice" localizedString]];
            return;
        }
        [TMFSafeKeyboardCheckApi textDoubleCheckForContent:content1 secondContent:content2 passwordStrength:pwdStrength repeatNumber:repeatNum
                                                completion:^(BOOL pass, NSError *_Nullable error) {
                                                    NSLog(@"[TMFSafeKeyboard-Check2]%d, %@", pass, error);
                                                    NSString *message;
                                                    if (error && error.code == 7) {
                                                        message = [@"the two passwords entered are inconsistent" localizedString];
                                                    } else if (!error && pass == NO) {
                                                        message = [@"weak password" localizedString];
                                                    }
                                                    [self showAlertTitle:pass ? [@"passed" localizedString] : [@"failed" localizedString] msg:message];
                                                }];
    }
}

#pragma mark - TMFSafeKeyboardDelegate
- (BOOL)safeKeyboardShouldClose:(TMFSafeKeyboard *)safeKeyboard {
    return YES;
}

#pragma mark - UITextFieldDelegate
- (void)textFieldDidBeginEditing:(UITextField *)textField {
    NSLog(@"%s: %@", __func__, textField.text);
}
- (void)textFieldDidEndEditing:(UITextField *)textField {
    NSLog(@"%s: %@", __func__, textField.text);
    //[self testRecoverTextWhileCompleteInput:textField.safeKeyboardWrapper.context text:textField.text];
}

#pragma mark - Slider Function
- (void)valueChange:(UISlider *)slider {
    if (slider == self.slider1) {
        self.label1.text = [NSString stringWithFormat:[@"password strength:%d" localizedString], (int)slider.value];
    } else if (slider == self.slider2) {
        self.label2.text = [NSString stringWithFormat:[@"consecutive repeat limit:%d" localizedString], (int)slider.value];
    }
}

- (void)showDocument {
    NSString *msg = [@"password strength illustrate" localizedString];

    [self showAlertTitle:[@"illustrate" localizedString] msg:msg];
}

#pragma mark - UIAlert
- (void)showAlertTitle:(NSString *)title msg:(NSString *)msg {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:title message:msg preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alert animated:YES completion:nil];
}

@end
