//
//  TMFSafeKeyboardViewController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2020/8/4.
//  Copyright © 2020 Tencent. All rights reserved.
//

#import "TMFSafeKeyboardViewController.h"

#import "TMFSKCustomInputView.h"
#import "TMFSafeKeyboard.h"
#import "SKKeyboardConfigViewController.h"
// Shark
#import "TMFSharkCenter.h"
#import "TMFSafeKeyboardWrapper.h"
#import "UIView+TMFSafeKeyboard.h"
#import "TMFSKDefaultKeyPositionProvider.h"
#import "TMFSKDefaultKeyboardLayoutProvider.h"
#import "IQKeyboardManager.h"
#import "SKReqRecoverInputText.h"
#import "SKRespRecoverInputText.h"
#import "QMUISegmentedControl.h"
#import "TMFSafeKeyboardWrapperApi.h"
#import "TMFSKWeakSelf.h"
#import "TMFNetErrorCode.h"
#import "NSString+Localize.h"

static NSNotificationName TMFLogNotification = @"log";


@interface TMFSafeKeyboardViewController ()<TMFSKCustomInputViewDelegate,TMFSafeKeyboardDelegate>

/**
 @brief 主滑动视图，用来处理内容上下超出屏幕问题
 */
@property (nonatomic, strong) UIScrollView *mainScrollView;

/**
 @brief 键盘设置
 */
@property (nonatomic, strong) TMFSafeKeyboardWrapper *codeViewKeyboardWrapper;

/**
 @brief 自定义输入框
 */
@property (nonatomic, strong) TMFSKCustomInputView *codeView;                       
 
/**
 @brief 子滑动视图，用来处理内容左右超出屏幕问题
 */
@property(nonatomic, strong) UIScrollView *scrollView;

/**
 @brief switchKeyinputTypeLabel
 */
@property(nonatomic, strong) UILabel * switchKeyinputTypeLabel;

/**
 @brief 键盘类型[默认、数字、身份证]
 */
@property(nonatomic, strong) QMUISegmentedControl * allInputTypeSwitcher;

/**
 @brief 调试日志
 */
@property(nonatomic, strong) UITextView *logTextView;

typedef NS_ENUM(NSInteger, ViewId) {
    ViewIdOptionShowBorder,
    ViewIdOptionBorderWidth,
    ViewIdOptionBorderRaduis,
    ViewIdOptionBorderColor,
    ViewIdOptionBorderHideRadiusSide,
    ViewIdOptionCharGap,
    ViewIdOptionTextSize,
    ViewIdOptionTextLength,
    ViewIdOptionShowPlainText,
    ViewIdOptionTextColor
};


@end

@implementation TMFSafeKeyboardViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // IQKeyboardManager不是必须组件
    [IQKeyboardManager sharedManager].enable = YES;
    self.view.backgroundColor = UIColor.whiteColor;
    
    // 1.notification
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShowNotification:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidShowNotification:) name:UIKeyboardDidShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textFieldViewDidBeginEditing:) name:UITextFieldTextDidBeginEditingNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(logNotification:) name:TMFLogNotification object:nil];
    
     //监听是否重新进入程序程序.
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationDidBecomeActive:) name:UIApplicationDidBecomeActiveNotification object:nil];
    
    
    // 2.navigation bar
    UIBarButtonItem *trashItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemTrash target:self action:@selector(trashBarButtonItemClicked:)];
    
    UIBarButtonItem *setupItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCompose target:self action:@selector(setupBarButtonItemClicked:)];
    
    self.navigationItem.rightBarButtonItems = @[setupItem,trashItem];

    CGFloat itemHeight  = 30.0;
    CGFloat topOffset   = 10;
    CGFloat leftOffset  = 20.0;
    CGFloat verticalGap = 40.0;
    CGFloat totalWidth  = CGRectGetWidth(self.view.frame);
    
    self.mainScrollView = [[UIScrollView alloc] initWithFrame:self.view.bounds];
    self.mainScrollView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.mainScrollView.contentSize = self.mainScrollView.frame.size;
    self.mainScrollView.decelerationRate = 0.1;
    [self.view addSubview:self.mainScrollView];
    
    int counter = 0;
    UIScrollView *topScrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(0, topOffset, totalWidth, verticalGap)];
    topScrollView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    topScrollView.bounces = NO;
    topScrollView.decelerationRate = 0.1;
    [self.mainScrollView addSubview:topScrollView];
    UIView *showBorderOption = [self switchOption:leftOffset top:0 optionTitle:[@"hide border:" localizedString] defValue:NO tagId:ViewIdOptionShowBorder viewWidth:150 viewHeight:itemHeight];
    [topScrollView addSubview:showBorderOption];
    
    UIView *showPlainTextOption = [self switchOption:CGRectGetMaxX(showBorderOption.frame) + 10 top:0 optionTitle:[@"show plaintext:" localizedString] defValue:NO tagId:ViewIdOptionShowPlainText viewWidth:180 viewHeight:itemHeight];
    [topScrollView addSubview:showPlainTextOption];
    topScrollView.contentSize = CGSizeMake(MAX(totalWidth, CGRectGetMaxX(showPlainTextOption.frame) + leftOffset), verticalGap);
    
    counter ++;
  
    UIView *optionBorderWidth = [self sliderOption:leftOffset top: topOffset + counter * verticalGap optionTitle:[@"border width:" localizedString]  minValue:0 maxValue:4 tagId:ViewIdOptionBorderWidth viewHeight:itemHeight];
    optionBorderWidth.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:optionBorderWidth];
    counter ++;
    
    UIView *optionBorderRadius = [self sliderOption:leftOffset top: topOffset + counter * verticalGap optionTitle:[@"border raduis:" localizedString]  minValue:0 maxValue:10 tagId:ViewIdOptionBorderRaduis viewHeight:itemHeight];
    optionBorderRadius.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:optionBorderRadius];
    counter ++;
    
    UIView *optionGap = [self sliderOption:leftOffset top: topOffset + counter * verticalGap optionTitle:[@"char gap:" localizedString]  minValue:0 maxValue:10 tagId:ViewIdOptionCharGap viewHeight:itemHeight];
    optionGap.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:optionGap];
    counter ++;
    
    UIView *optionTextSize = [self sliderOption:leftOffset top: topOffset + counter * verticalGap optionTitle:[@"text size:" localizedString]  minValue:24 maxValue:40 tagId:ViewIdOptionTextSize viewHeight:itemHeight];
    optionTextSize.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:optionTextSize];
    counter ++;
    
    UIView *inputNumSize = [self sliderOption:leftOffset top: topOffset + counter * verticalGap  optionTitle:[@"text length:" localizedString]  minValue:6 maxValue:10 tagId:ViewIdOptionTextLength viewHeight:itemHeight];
    inputNumSize.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:inputNumSize];
    counter ++;
    
    UIView *optionTextColor = [self sliderOption:leftOffset top: topOffset + counter * verticalGap  optionTitle:[@"text color:" localizedString]  minValue:0.0 maxValue:1.0 tagId:ViewIdOptionTextColor viewHeight:itemHeight];
    optionTextColor.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:optionTextColor];
    
    counter ++;
    self.switchKeyinputTypeLabel = [[UILabel alloc] initWithFrame:CGRectMake(leftOffset, topOffset + counter * verticalGap, 0, 0)];
    self.switchKeyinputTypeLabel.textAlignment = NSTextAlignmentLeft;
    self.switchKeyinputTypeLabel.text = [@"keyboard type:" localizedString];
    self.switchKeyinputTypeLabel.numberOfLines = 0;
    [self.switchKeyinputTypeLabel sizeToFit];
    [self.mainScrollView addSubview: self.switchKeyinputTypeLabel];
    
    CGFloat left = CGRectGetMaxX(self.switchKeyinputTypeLabel.frame) + 10;
    self.scrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(left, topOffset + counter * verticalGap, totalWidth - left - leftOffset - 10, itemHeight)];
    self.scrollView.keyboardDismissMode = UIScrollViewKeyboardDismissModeInteractive;
    self.scrollView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview:self.scrollView];
    
    self.allInputTypeSwitcher = [self generateSegmentedControlWithItems:@[[@"default" localizedString], [@"only number" localizedString], [@"only ID card" localizedString]]];
    self.scrollView.contentSize = CGSizeMake(CGRectGetWidth(self.allInputTypeSwitcher.frame), itemHeight);
    self.switchKeyinputTypeLabel.center = CGPointMake(self.switchKeyinputTypeLabel.center.x, CGRectGetMidY(self.scrollView.frame));     // segment view的高度比label打，以segment的frame为准

    [self.scrollView addSubview:self.allInputTypeSwitcher];
    counter ++;
    
    
    // 默认值的设置
    self.allInputTypeSwitcher.selectedSegmentIndex = 0;
    
    TMFSKCustomInputView *inputView = [self CreateInputView: leftOffset top:itemHeight + counter * verticalGap height:50];
    //    [self.codeView setTextWidth:40];
    
    {
        self.codeViewKeyboardWrapper =
        [[TMFSafeKeyboardWrapper alloc] initWithKeyboard:TMFSafeKeyboard.new];
        
        [self.codeViewKeyboardWrapper setKeyboardDelegate:self];
        
        @TMFSKWeakify(self)
        
        [TMFSafeKeyboardWrapperApi customKeyboardWithWrapper:self.codeViewKeyboardWrapper inputView:self.codeView completeCallback:^(NSData *context, NSString *text) {

            @TMFSKStrongify(self)
            if (!self) return;
            
            [self testRecoverTextWhileCompleteInput:context text:text];
        }];
        // 写法2
//        [self.codeView setupSafeKeyboardWithCompleteCallback:^(NSData *context, NSString *text) {
//            [weakself testRecoverTextWhileCompleteInput:context text:text];
//        }];
//        self.codeViewKeyboardWrapper = self.codeView.safeKeyboardWrapper;
        
        // 写法3
//        TMFSafeKeyboard *keyboard       = TMFSafeKeyboard.new;
//
//        TMFSafeKeyboardWrapper *wrapper   = TMFSafeKeyboardWrapper.new;
//        [wrapper setInputType:SKInputTypeNone];
//        [wrapper setSessionId:self.codeView.hash];
//        [wrapper setCustomKeyboard:keyboard];
//        [wrapper customKeyboardLayoutProvider:TMFSKDefaultKeyboardLayoutProvider.new];
//        [wrapper setKeyboardDataConfigMode:TMFConfigModeOnline];
//
//        self.codeView.inputView             = keyboard.inputView;
//        self.codeView.inputAccessoryView    = keyboard.inputAccessoryView;
//        self.codeView.delegate              = self;
//
//        self.codeViewKeyboard               = keyboard;
//        self.codeViewKeyboardWrapper        = wrapper;
    }
    inputView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [self.mainScrollView addSubview:inputView];
    
    UITextView *logTextView = [self logTextView];
    _logTextView.frame = CGRectMake(leftOffset, CGRectGetMaxY(self.codeView.frame) + 30, CGRectGetWidth(self.view.frame) - (leftOffset * 2), 0);
    logTextView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [self.mainScrollView addSubview: logTextView];
    
}

- (QMUISegmentedControl *)generateSegmentedControlWithItems:(NSArray<NSString *> *)items {
    QMUISegmentedControl *segmentedControl = [[QMUISegmentedControl alloc] initWithItems:items];
    segmentedControl.tintColor = [UIColor blueColor];
    [segmentedControl sizeToFit];
    segmentedControl.frame = CGRectMake(0, 0, self.scrollView.frame.size.width, self.scrollView.frame.size.height);// 统一按照最长的来就行啦
    segmentedControl.transform = CGAffineTransformMakeScale(.8, .8);
    [segmentedControl sizeToFit];
    [segmentedControl addTarget:self action:@selector(handleSegmentedControlEvent:) forControlEvents:UIControlEventValueChanged];
    segmentedControl.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    return segmentedControl;
}

- (void)handleSegmentedControlEvent:(QMUISegmentedControl *)segmentedControl {
    if (segmentedControl == self.allInputTypeSwitcher) {
        if (segmentedControl.selectedSegmentIndex == 0) {
            [self.codeView resignFirstResponder];
            [self.codeViewKeyboardWrapper setInputType:SKInputTypeNone];
            [self.codeView becomeFirstResponder];
        } else if (segmentedControl.selectedSegmentIndex == 1) {
            [self.codeView resignFirstResponder];
            [self.codeViewKeyboardWrapper setInputType:SKInputTypeNumeric];
            [self.codeView becomeFirstResponder];
        } else if (segmentedControl.selectedSegmentIndex == 2) {
            [self.codeView resignFirstResponder];
            [self.codeViewKeyboardWrapper setInputType:SKInputTypeChinaIDCard];
            [self.codeView becomeFirstResponder];
        }
    }
}

- (void)provideKeyPosition:(TMFNetErrorCode)errorCode positionProvider:(TMFSKAbstractKeyPositionProvider *)positionProvider {
    NSLog(@"provideKeyPosition, received return data.");
    NSLog(@"%s",__func__);
}

- (void)doRelayout {
    NSLog(@"%s",__func__);
    [self.codeView clearText];
}

- (TMFSKCustomInputView *) CreateInputView:(CGFloat) left top:(CGFloat) top height:(CGFloat) height {
    if (!_codeView) {
        _codeView = [[TMFSKCustomInputView alloc] initWithFrame:CGRectMake(left, top, CGRectGetWidth(self.view.frame) - 2 * left, height) inputLength:6];
        _codeView.delegate = self;
//        [_codeView setBackgroundColor:[UIColor redColor]];
//        [_codeView setBorderHideRaduisSide:TMFSKHideRadiusBottomSide];
    }
    return _codeView;
}

-(UIView *) switchOption:(CGFloat) margin top:(CGFloat) top optionTitle:(NSString *) title defValue:(BOOL) defValue tagId:(NSInteger) tagId viewWidth:(CGFloat)width viewHeight:(CGFloat) height {
    UIView *root = [[UIView alloc] initWithFrame:CGRectMake(margin, top, width, height)];
    [self.view addSubview:root];
    UILabel *optionLabel = [UILabel new];
    optionLabel.textAlignment = NSTextAlignmentCenter;
    optionLabel.text = title;
    optionLabel.numberOfLines = 0;
    [optionLabel sizeToFit];
    [root addSubview:optionLabel];
    
    UISwitch *showBorderSwitch = [[UISwitch alloc] initWithFrame:CGRectMake(optionLabel.frame.size.width + 10, 0, 0, 0)];
    showBorderSwitch.tag = tagId;
    [showBorderSwitch setOn:NO];
    [showBorderSwitch addTarget:self action:@selector(switchValueChanged:) forControlEvents:UIControlEventValueChanged | UIControlEventTouchDragExit];
    [root addSubview:showBorderSwitch];
    [showBorderSwitch sizeToFit];
//    if(width <= 0) {
//       [root sizeToFit];
//    }
    optionLabel.frame = CGRectMake(0, (showBorderSwitch.frame.size.height - optionLabel.frame.size.height) / 2, optionLabel.frame.size.width, optionLabel.frame.size.height);
    return root;
}

-(UIView *) sliderOption: (CGFloat) margin top: (CGFloat) top optionTitle:(NSString *)title minValue:(CGFloat) min maxValue:(CGFloat) max tagId:(NSInteger) tagId viewHeight:(CGFloat) height
{
    UIView *root = [[UIView alloc] initWithFrame:CGRectMake(margin, top, CGRectGetWidth(self.view.frame) - margin * 2, height)];
    [self.view addSubview:root];
            
    UILabel *optionLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 0, height)];
    optionLabel.textAlignment = NSTextAlignmentLeft;
    optionLabel.text = title;
    optionLabel.numberOfLines = 0;
    [optionLabel sizeToFit];
    [root addSubview:optionLabel];
            
    // 最小值
    UILabel *minLabel = [[UILabel alloc] initWithFrame:CGRectMake(optionLabel.frame.origin.x + optionLabel.frame.size.width + 10, optionLabel.frame.origin.y, 0, height)];
    minLabel.textAlignment = NSTextAlignmentRight;
    if (tagId == ViewIdOptionTextColor) {
        minLabel.text = @"darkGray";
    } else if (tagId == ViewIdOptionTextLength) {
        minLabel.text = [NSString stringWithFormat:@"%.0f", min];
    } else {
        minLabel.text = [NSString stringWithFormat:@"%.1fpt", min];
    }
    minLabel.numberOfLines = 0;
    [minLabel sizeToFit];
    minLabel.autoresizingMask = UIViewAutoresizingFlexibleRightMargin;
    [root addSubview:minLabel];
    
    // 最大值
    UILabel *maxLable = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 0, height)];
    maxLable.textAlignment = NSTextAlignmentLeft;
    if (tagId == ViewIdOptionTextColor) {
        maxLable.text = @"red";
    } else if (tagId == ViewIdOptionTextLength){
        maxLable.text = [NSString stringWithFormat:@"%.0f位", max];
    } else {
        maxLable.text = [NSString stringWithFormat:@"%.1fpt", max];
    }
    maxLable.numberOfLines = 0;
    [maxLable sizeToFit];
    maxLable.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
    [root addSubview:maxLable];
    
    CGFloat sliderWidth = root.frame.size.width - minLabel.frame.origin.x - minLabel.frame.size.width - maxLable.frame.size.width - 20;
    UISlider *slider = [[UISlider alloc] initWithFrame:CGRectMake(minLabel.frame.origin.x + minLabel.frame.size.width + 10, optionLabel.frame.origin.y, sliderWidth, height)];
    slider.minimumValue = min;
    slider.maximumValue = max;
    slider.continuous = YES;
    slider.tag = tagId;
    [slider addTarget:self action:@selector(sliderValueChanged:) forControlEvents:UIControlEventValueChanged];
    [root addSubview:slider];
    slider.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    maxLable.frame = CGRectMake(slider.frame.origin.x + slider.frame.size.width + 10, slider.frame.origin.y, maxLable.frame.size.width, height);
    
    return root;
}

-(void)sliderValueChanged:(id)sender {
    UISlider *slider = (UISlider *)sender;
    switch (slider.tag) {
        case ViewIdOptionCharGap:
            [self.codeView setCharGap:slider.value];
            break;
        case ViewIdOptionTextSize:
            [self.codeView setTextSize:slider.value];
            break;
        case ViewIdOptionTextLength:
            [self.codeView setInputLength:slider.value];
            break;
        case ViewIdOptionBorderWidth:
            [self.codeView setBorderWidth:slider.value];
            break;
        case ViewIdOptionBorderRaduis:
            [self.codeView setBorderCornerRadius:slider.value];
            break;
        case ViewIdOptionTextColor: {
            NSLog(@"ratio === %f", slider.value);
            UIColor * textColor = [self transitionColorWithStartValue:slider.value startColor:[UIColor darkGrayColor] endColor:[UIColor redColor]];
            [self.codeView setTextColor:textColor];
            break;
        }
        default:
            break;
    }
}

-(UIColor *) transitionColorWithStartValue:(CGFloat) ratio startColor:(UIColor *) startColor endColor:(UIColor *) endColor {
    // 获取开始位置颜色的 R、G、B、A值
    CGFloat startR, startG, startB, startA;
    if (![startColor getRed:&startR green: &startG blue:&startB alpha:&startA]) {
        return nil;
    }
    // 获取结束位置颜色的 R、G、B、A值
    CGFloat endR, endG, endB, endA;
    if (![endColor getRed:&endR green: &endG blue:&endB alpha:&endA]) {
        return nil;
    }
    // 开始颜色与结束的色值差
    CGFloat R_D_value = endR - startR;
    CGFloat G_D_value = endG - startG;
    CGFloat B_D_value = endB - startB;
    CGFloat A_D_value = endA - startA;
    // (开始值 + 差值*coe)/255.0 = 最终色值
    CGFloat R = startR + R_D_value * ratio;
    CGFloat G = startG + G_D_value * ratio;
    CGFloat B = startB + B_D_value * ratio;
    CGFloat A = startA + A_D_value * ratio;
    return [UIColor colorWithRed:R green:G blue:B alpha:A];
}

-(void)switchValueChanged:(id)sender {
    UISwitch *switcher = (UISwitch *) sender;
    if (switcher.tag == ViewIdOptionShowBorder) {
        [self.codeView setShowBorder:!switcher.isOn];
    } else if (switcher.tag == ViewIdOptionShowPlainText) {
        [self.codeView setShowPlainText:switcher.isOn];
    }
}

#pragma mark 输入框开始编辑
- (void)textFieldViewDidBeginEditing:(NSNotification *)noti
{
    
}

- (void) testRecoverTextWhileCompleteInput:(NSData *) context text:(NSString *) text {
    
//    [self clearLog];
    
    TMFOCSharkCmdSettings *cmdSettings = [TMFOCSharkCmdSettings settings];
    [cmdSettings csSetCmdID: 1008 objectClass:[SKReqRecoverInputText class]];
    [cmdSettings scSetCmdID:11008 objectClass:[SKRespRecoverInputText class]];
    
    SKReqRecoverInputText * req = [SKReqRecoverInputText object];
    req.tars_context = context;
    req.tars_inputText = text;
    
    [[TMFSharkCenter masterCenter] requestWithCmdSettings:cmdSettings csObject: req completionHandler:^(__kindof SKRespRecoverInputText * _Nullable resp, NSError * _Nullable error) {
        bool failed = error || [error code] != 0;
        if (failed) {
            [self log:@"recover input text failed,  desc: %@", error != nil ? [error description] : @"unknown error!"];
            return;
        }
        
        if (!resp || resp.tars_ret != 0) {
            [self log:@"recover input text failed, resp return failed,  retCode: %d", resp ? resp.tars_ret : -1];
            return;
        }
        
        [self log:@"recover input text: %@", resp.tars_recoveredText];
    }];
}

- (void)inputViewDidChange:(TMFSKCustomInputView *)inputView {
    NSLog(@"inputViewDidChange === %@", inputView.text);
}

- (void)inputViewDidDeleteBackward:(TMFSKCustomInputView *)inputView {
    NSLog(@"inputViewDidDeleteBackward === %@", inputView.text);
}

- (void)inputViewEndInput:(TMFSKCustomInputView *)inputView {
    [self log:[@"input ended, the input content is: %@" localizedString], inputView.text];
    NSLog(@"inputViewEndInput === %@", inputView.text);
}

- (void)inputViewBeginInput:(TMFSKCustomInputView *)inputView {
    NSLog(@"inputViewBeginInput === %@", inputView.text);
    
//    __weak typeof(self) weakself = self;
//
//    [self.codeViewKeyboardWrapper wrapWithRelayoutCallback:^{
//        NSLog(@"clear text...");
//        [weakself.codeView clearText];
//    }];
}

- (void)inputViewCompleteInput:(TMFSKCustomInputView *)inputView {
    NSLog(@"inputViewCompleteInput === %@", inputView.text);
//    [self testRecoverTextWhileCompleteInput:self.codeViewKeyboardWrapper.context text:inputView.text];
}

#pragma mark - 日志
/**
 *  如果想在其他地方发送日志内容，就使用以下通知方法
 *  [[NSNotificationCenter defaultCenter] postNotificationName:@"log" object:[NSString stringWithFormat:@"%@\n",@"log example"]];
 */
- (void)log:(NSString *)format, ... NS_FORMAT_FUNCTION(1,2)
{
    va_list ap;
    va_start(ap, format);
    
    NSString *string =
    [[NSString alloc] initWithFormat:format arguments:ap];
    
    va_end(ap);
    
    NSLog(@"%@",string);
    
    [self appendLog:string];
}

- (void)appendLog:(NSString *)log
{
    [self appendLog:log color:[UIColor whiteColor]];
}

- (void)appendLog:(NSString *)log color:(UIColor *)color
{
    NSMutableDictionary *attrs = NSMutableDictionary.new;
    
    [attrs setValue:[UIFont systemFontOfSize:16] forKey:NSFontAttributeName];
    [attrs setValue:color forKey:NSForegroundColorAttributeName];
    
    [self.logTextView.textStorage appendAttributedString:[[NSMutableAttributedString alloc] initWithString:log attributes:attrs]];
    [self.logTextView.textStorage appendAttributedString:[[NSMutableAttributedString alloc] initWithString:@"\n" attributes:attrs]];
}

- (void)logNotification:(NSNotification *)noti
{
    [self appendLog:noti.object];
}

- (void)clearLog
{
    self.logTextView.text = @"";
}

- (UITextView *)logTextView
{
    if (!_logTextView)
    {
        _logTextView = [UITextView new];
        _logTextView.backgroundColor = [UIColor darkGrayColor];
        _logTextView.selectable = NO;
    }
    return _logTextView;
}

- (void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];

    CGRect logTextFrame = _logTextView.frame;
    if (@available(iOS 11.0, *)) {
        logTextFrame.size.height = MAX(180, CGRectGetHeight(self.mainScrollView.frame) - CGRectGetMinY(logTextFrame) - self.mainScrollView.adjustedContentInset.top - self.mainScrollView.adjustedContentInset.bottom) - CGRectGetMinX(_logTextView.frame);
    } else {
        logTextFrame.size.height = MAX(180, CGRectGetHeight(self.mainScrollView.frame) - CGRectGetMinY(logTextFrame) - self.mainScrollView.contentInset.top - self.mainScrollView.contentInset.bottom) - CGRectGetMinX(_logTextView.frame);
    }
    _logTextView.frame = logTextFrame;
    
    self.mainScrollView.contentSize = CGSizeMake(self.mainScrollView.contentSize.width, CGRectGetMaxY(_logTextView.frame) + CGRectGetMinX(_logTextView.frame));
}

- (void)viewSafeAreaInsetsDidChange{
    [super viewSafeAreaInsetsDidChange];
 
    CGRect mainScrollFrame = self.mainScrollView.frame;
    mainScrollFrame.size.width = CGRectGetWidth(self.view.frame) - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right;
    mainScrollFrame.origin.x = self.view.safeAreaInsets.left;
    self.mainScrollView.frame = mainScrollFrame;
    
    CGSize mainScrollSize = self.mainScrollView.contentSize;
    mainScrollSize.width  = mainScrollFrame.size.width;
    self.mainScrollView.contentSize = mainScrollSize;
}

- (void)trashBarButtonItemClicked:(UIBarButtonItem *)sender
{
    [self.codeView clearText];
    [self clearLog];
}

- (void)setupBarButtonItemClicked:(UIBarButtonItem *)sender
{
    [self.navigationController pushViewController:SKKeyboardConfigViewController.new animated:YES];
}

- (void)keyboardWillShowNotification:(NSNotification *)noti
{
//    NSLog(@"KB BEGIN:%@",[noti.userInfo valueForKey:UIKeyboardFrameBeginUserInfoKey]);
}
- (void)keyboardDidShowNotification:(NSNotification *)noti
{
//    NSLog(@"KB END:%@",[noti.userInfo valueForKey:UIKeyboardFrameEndUserInfoKey]);
    
}

- (void)applicationDidBecomeActive:(NSNotification *)notification
{
    
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [super touchesBegan:touches
              withEvent:event];
    
    [self.view endEditing:YES];
    
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}


@end
