//
//  TMFShareViewController.m
//  TMFDemo
//
//  Created by hauzhong on 2019/5/28.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFShareViewController.h"

#import "TMFShareManager.h"
#import "NSString+Localize.h"

// 要分享到什么地方
typedef NS_ENUM(NSInteger, TMFShareDestination) {
    TMFShareDestinationNone      = 0,
    TMFShareToWeChatContacts     = 200,    // 分享给微信联系人
    TMFShareToWeChatMoments      = 201,    // 分享到微信朋友圈
    TMFShareToQQAll              = 300,    // 分享到QQ，会到达“发送到”界面，用户可以选择好友、群、讨论组、QQ空间、电脑、收藏来进行分享
    TMFShareToAliPay             = 400,    // 分享到支付宝
    TMFShareToWeibo              = 500,    // 分享到微博
    TMFShareToSMS                = 600,    // 分享到短信
    TMFShareToDDing              = 700,    // 分享到钉钉
    TMFShareToWXWork             = 701,    // 分享到企业微信

};

@interface TMFShareViewController ()
// Share object
@property(nonatomic,strong) TMFShareObject *shareObject;

@end

@implementation TMFShareViewController

- (void)dealloc {
    self.shareObject = nil;
}

- (void)initDataSource {
    [super initDataSource];
    QMUIOrderedDictionary<NSString *, NSString *> *dataSourceWithDetailText = [[QMUIOrderedDictionary alloc] init];
    [dataSourceWithDetailText setObject:@"TMFShareToWeChatContacts" forKey:[@"share to WeChat contact" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToWeChatMoments" forKey:[@"share to WeChat moments" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToQQAll" forKey:[@"share to QQ" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToAliPay" forKey:[@"share to Alipay" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToWeibo" forKey:[@"share to Weibo" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToSMS" forKey:[@"share to Messages" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToDDing" forKey:[@"share to DingTalk" localizedString]];
    [dataSourceWithDetailText setObject:@"TMFShareToWXWork" forKey:[@"share to WeCom" localizedString]];
    self.dataSourceWithDetailText = dataSourceWithDetailText;
}

- (void)didSelectCellWithTitle:(NSString *)title {
    if ([title isEqualToString:[@"share to WeChat contact" localizedString]]) {
        [self showDestinationContentView:TMFShareToWeChatContacts];
    } else if ([title isEqualToString:[@"share to WeChat moments" localizedString]]) {
        [self showDestinationContentView:TMFShareToWeChatMoments];
    } else if ([title isEqualToString:[@"share to QQ" localizedString]]) {
        [self showDestinationContentView:TMFShareToQQAll];
    } else if ([title isEqualToString:[@"share to Alipay" localizedString]]) {
        [self showDestinationContentView:TMFShareToAliPay];
    } else if ([title isEqualToString:[@"share to Weibo" localizedString]]) {
        [self showDestinationContentView:TMFShareToWeibo];
    } else if ([title isEqualToString:[@"share to Messages" localizedString]]) {
        [self showDestinationContentView:TMFShareToSMS];
    } else if ([title isEqualToString:[@"share to DingTalk" localizedString]]) {
        [self showDestinationContentView:TMFShareToDDing];
    }else if ([title isEqualToString:[@"share to WeCom" localizedString]]) {
        [self showDestinationContentView:TMFShareToWXWork];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = [NSString stringWithFormat:@"%@-TMFShare",[@"share" localizedString]];
}

#pragma mark - Share Object

- (void)showDestinationContentView:(TMFShareDestination)destination {
    NSUInteger sharingCount = [[self shareDestinationsThatCanShare] count];

    if (sharingCount > 0) {
        QMUIAlertAction *action1 = [QMUIAlertAction actionWithTitle:[@"cancel" localizedString] style:QMUIAlertActionStyleCancel
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                // Cancel
                                                            }];
        QMUIAlertAction *action2 = [QMUIAlertAction actionWithTitle:[@"share picture" localizedString] style:QMUIAlertActionStyleDefault
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                [self shareObjectWithDestination:destination type:TMFShareTypeImage];
                                                            }];
        QMUIAlertAction *action3 = [QMUIAlertAction actionWithTitle:[@"share link" localizedString] style:QMUIAlertActionStyleDefault
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                [self shareObjectWithDestination:destination type:TMFShareTypeWebPage];
                                                            }];
        QMUIAlertAction *action4 = [QMUIAlertAction actionWithTitle:[@"share text" localizedString] style:QMUIAlertActionStyleDefault
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                [self shareObjectWithDestination:destination type:TMFShareTypeText];
                                                            }];
        QMUIAlertAction *action5 = [QMUIAlertAction actionWithTitle:[@"share file" localizedString] style:QMUIAlertActionStyleDefault
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                [self shareObjectWithDestination:destination type:TMFShareTypeFile];
                                                            }];
        QMUIAlertAction *action6 = [QMUIAlertAction actionWithTitle:[@"share video" localizedString] style:QMUIAlertActionStyleDefault
                                                            handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                [self shareObjectWithDestination:destination type:TMFShareTypeVideo];
                                                            }];
        QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:nil message:nil
                                                                              preferredStyle:QMUIAlertControllerStyleActionSheet];
        [alertController addAction:action1];
        [alertController addAction:action2];
        [alertController addAction:action3];
        [alertController addAction:action4];
        if (TMFShareToWXWork == destination){
            [alertController addAction:action5];
            [alertController addAction:action6];
        }
        [alertController showWithAnimated:YES];

    } else {
        QMUIAlertAction *action1 = [QMUIAlertAction actionWithTitle:@"OK" style:QMUIAlertActionStyleCancel handler:NULL];
        QMUIAlertController *alertController = [QMUIAlertController
            alertControllerWithTitle:nil
                             message:[@"the phone does not have QQ/WeChat/Alipay/Weibo installed, or the sharing scheme is not configured in the Info.plist, see the documentation for details" localizedString]
                      preferredStyle:QMUIAlertControllerStyleAlert];
        [alertController addAction:action1];
        [alertController showWithAnimated:YES];
    }
}

- (void)shareObjectWithDestination:(TMFShareDestination)destination type:(TMFShareType)shareType {
    TMFShareObject *shareObject = [[TMFShareObject alloc] init];  // Share object
    shareObject.shareType = shareType;
    shareObject.title = @"TMF分享测试标题";
    shareObject.text = @"TMF分享测试描述";

    NSString *thumbnailPath = [[NSBundle mainBundle] pathForResource:@"tmf-thumbnail.png" ofType:nil];
    UIImage *thumbnail = [UIImage imageNamed:thumbnailPath];
    if (TMFShareTypeWebPage == shareType) {
        shareObject.webPageURL = [NSURL URLWithString:@"https://m.qq.com"];
        shareObject.thumbnail = thumbnail;  // thumbnail IfNeeded
    } else if (TMFShareTypeImage == shareType) {
        shareObject.image = thumbnail;
    }else if (TMFShareTypeFile == shareType) {
        shareObject.fileName = @"test file";
        shareObject.filePath = [[NSBundle mainBundle] pathForResource:@"tmf-share-file.pdf" ofType:nil];
    }else if (TMFShareTypeVideo == shareType) {
        shareObject.fileName = @"test video";
        shareObject.filePath = [[NSBundle mainBundle] pathForResource:@"tmf-share-video.mp4" ofType:nil];
    } else {
        shareObject.thumbnail = thumbnail;  // thumbnail IfNeeded
    }

    if (destination == TMFShareToQQAll) {
        // 仅在分享到QQ时，强制使用图片URL的方式来作为QQ内的预览图。
        NSString *thumbnailURL = @"http://a-thumbnail-been-setted-here.png";
        shareObject.previewImageURL = [NSURL URLWithString:thumbnailURL];
    }
    
    if (destination == TMFShareToWXWork && TMFShareTypeImage == shareType) {
        shareObject.fileName = @"test.gif";
        shareObject.filePath = [[NSBundle mainBundle] pathForResource:@"tmf-testGif.gif" ofType:nil];
    }

    self.shareObject = shareObject;
    [self shareToDestination:destination];
}

#pragma mark - ShareAPI

+ (BOOL)canShareToDestination:(TMFShareDestination)destination {
    TMFShareApp app = [self appFromDestination:destination];
    return [[TMFShareManager shareManager] isAppInstalled:app];
}

+ (NSArray *)shareDestinationsThatCanShare:(NSArray *)sourceShareDestinations {
    if (sourceShareDestinations) {
        NSMutableArray *shareDestinationsThatCanShare = [NSMutableArray array];
        for (NSNumber *elem in sourceShareDestinations) {
            TMFShareDestination destination = [elem integerValue];
            if ([self canShareToDestination:destination]) {
                [shareDestinationsThatCanShare addObject:elem];
            }
        }
        return [NSArray arrayWithArray:shareDestinationsThatCanShare];
    } else {
        return nil;
    }
}

+ (NSArray *)allShareDestinations {
    return @[
        @(TMFShareToWeChatContacts),
        @(TMFShareToWeChatMoments),
        @(TMFShareToQQAll),
        @(TMFShareToAliPay),
        @(TMFShareToWeibo),
        @(TMFShareToSMS),
        @(TMFShareToDDing),
        @(TMFShareToWXWork),
    ];
}

- (NSArray *)shareDestinationsThatCanShare {
    return [[self class] shareDestinationsThatCanShare:[[self class] allShareDestinations]];
}

- (void)shareToDestination:(TMFShareDestination)destination {
    if ([[self class] canShareToDestination:destination]  // Make sure that's posible to share
        && self.shareObject) {
        [self doShareToDestination:destination];
    } else {
        QMUIAlertAction *action1 = [QMUIAlertAction actionWithTitle:@"OK" style:QMUIAlertActionStyleCancel handler:NULL];
        QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:nil message:[@"the shared app is not installed" localizedString]
                                                                              preferredStyle:QMUIAlertControllerStyleAlert];
        [alertController addAction:action1];
        [alertController showWithAnimated:YES];
    }
}

- (void)doShareToDestination:(TMFShareDestination)destination {
    TMFShareApp app = [[self class] appFromDestination:destination];
    TMFShareScene scene = [[self class] sceneFromDestination:destination];

    self.shareObject.scene = scene;  // 通过 destination 转换成 SDK 中的 scene，设置分享场景
    [[TMFShareManager shareManager] shareToApp:app withObject:self.shareObject completionHandler:^(TMFShareResponse *_Nonnull response) {
        [self didShareToApp:app respCode:response.retCode];
    }];
}

#pragma mark - 类型转换

// TMFShareDestination -> TMFShareApp
+ (TMFShareApp)appFromDestination:(TMFShareDestination)destination {
    if (destination == TMFShareToWeChatContacts || destination == TMFShareToWeChatMoments) {
        return TMFShareAppWeChat;
    } else if (destination == TMFShareToQQAll) {
        return TMFShareAppQQ;
    } else if (destination == TMFShareToAliPay) {
        return TMFShareAppAlipay;
    } else if (destination == TMFShareToWeibo) {
        return TMFShareAppWeibo;
    } else if (destination == TMFShareToSMS) {
        return TMFShareAppSMS;
    } else if (destination == TMFShareToDDing) {
        return TMFShareAppDDing;
    } else if (destination == TMFShareToWXWork) {
        return TMFShareAppWXWork;
    } else {
        return TMFShareAppNone;
    }
}

// TMFShareDestination -> TMFShareScene
+ (TMFShareScene)sceneFromDestination:(TMFShareDestination)destination {
    if (destination == TMFShareToWeChatMoments) {
        // 朋友圈，如微信朋友圈、支付宝好友动态
        return TMFShareSceneTimeline;
    } else {
        // 默认为好友消息
        return TMFShareSceneMessage;
    }
}

#pragma mark - 分享回调结果处理
/*
 - 下面的协议用来监听分享App的回调
 */
- (void)didShareToApp:(TMFShareApp)app respCode:(NSInteger)resp {
    NSLog(@"stone share didShareToApp:%ld", resp);
    NSString *message = nil;
    // -4是QQ取消
    // -2是AliPay/短信取消
    // -1是Weibo取消
    if ((TMFShareAppQQ == app && -4 == resp) || ((TMFShareAppAlipay == app | TMFShareAppSMS == app) && -2 == resp)
        || (TMFShareAppWeibo == app && -1 == resp)) {
        message = [@"share canceled" localizedString];
    }

    // 0是成功
    else if (0 == resp) {
        message = [@"share succeeded" localizedString];
    }

    // 其他原因
    else {
        /* 微博Weibo 回调时候 响应代码 说明
         Code Success               = 0,    //成功
         Code UserCancel            = -1,   //用户取消发送
         Code SentFail              = -2,   //发送失败
         Code AuthDeny              = -3,   //授权失败
         Code UserCancelInstall     = -4,   //用户取消安装微博客户端
         Code PayFail               = -5,   //支付失败
         Code ShareInSDKFailed      = -8,   //分享失败 详情见TMFShareResponse errorMessage
         Code Unsupport             = -99,  //不支持的请求
         Code Unknown               = -100或-999,
         */
        message = [NSString stringWithFormat:[@"share failed，code=%ld" localizedString], (long)resp];
    }

    dispatch_async(dispatch_get_main_queue(), ^{
        QMUIAlertAction *action1 = [QMUIAlertAction actionWithTitle:@"OK" style:QMUIAlertActionStyleCancel handler:NULL];
        QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:[@"share result" localizedString] message:message
                                                                              preferredStyle:QMUIAlertControllerStyleAlert];
        [alertController addAction:action1];
        [alertController showWithAnimated:YES];
    });
}
@end
