//
//  TMFSharkViewController.m
//  TMFDemo
//
//  Created by klaudz on 16/4/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFSharkViewController.h"
#import "TMFSharkDemoUserConfiguration.h"
#import "TMFSharkCenter.h"
#import "TMFSharkCenter+Profile.h"
#import "TMFSharkCenter.h"
#import "TMFSharkCenterConfiguration.h"
#import "TMFProfile.h"
#import "TMFSharkEnvironmentSelectViewController.h"

#import "QMUITips.h"
#import "NSURL+MQQExtended.h"
#import "NSString+Localize.h"

@implementation TMFSharkViewController {
    NSString *_customizedUserID;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        [self initDataSource];

        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(sharkCenterDidRegisterGUID:)
                                                     name:TMFSharkCenterDidRegisterGUIDNotification
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(tmfProfileDidReport:) name:TMFProfileDidReportNotification
                                                   object:nil];
    }
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];

    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Reset" style:UIBarButtonItemStylePlain target:self
                                                                          action:@selector(resetEnvironmentConfiguration)];
    self.navigationItem.rightBarButtonItem = rightBarButtonItem;
}

#define ROW_PRODUCT_KEY      @"ProductKey"
#define ROW_PRODUCT_ID       @"ProductID"
#define ROW_HTTP_URL         @"HTTPURL"
#define ROW_PUBLIC_KEY       @"PublicKey"
#define ROW_GUID             @"GUID"
#define ROW_VID              @"VID"

#define ROW_CMD              @"Cmd (String)"
#define ROW_CMD_ID           @"CmdID (Integer)"
#define ROW_REQUEST_HEADERS  @"RequestHeaders"
#define ROW_REQUEST_COOKIES  @"RequestCookies"
#define ROW_REQUEST_QUERIES  @"RequestQueries"
#define ROW_REQUEST_DATA     @"RequestData"
#define ROW_TEST_REQUEST     @"TestRequest >>"

#define ROW_USERID           @"CustomizedUserID"
#define ROW_UPDATE_USERID    @"Update >>"

- (void)initDataSource {
    [super initDataSource];

    TMFSharkCenter *center = [TMFSharkCenter masterCenter];
    TMFSharkCenterConfiguration *configuration = center.configuration;

    QMUIOrderedDictionary<NSString *, NSString *> *dataSourceWithDetailText = [[QMUIOrderedDictionary alloc] init];

    [dataSourceWithDetailText setObject:(configuration.productKey ?: @"") forKey:ROW_PRODUCT_KEY];
    [dataSourceWithDetailText setObject:[NSString stringWithFormat:@"%ld", (long)configuration.productID] forKey:ROW_PRODUCT_ID];
    [dataSourceWithDetailText setObject:(configuration.HTTPURL ?: @"") forKey:ROW_HTTP_URL];
    [dataSourceWithDetailText setObject:(configuration.RSAPublicKey ?: @"") forKey:ROW_PUBLIC_KEY];
    [dataSourceWithDetailText setObject:(center.GUID ?: @"") forKey:ROW_GUID];
    [dataSourceWithDetailText setObject:@"Click to fetch VID" forKey:ROW_VID];

    [dataSourceWithDetailText setObject:@"" forKey:@"------------------------- API Test"];

    [dataSourceWithDetailText setObject:(TMFSharkDemoUserConfiguration.u_cmd ?: @"") forKey:ROW_CMD];
    [dataSourceWithDetailText setObject:(TMFSharkDemoUserConfiguration.u_cmdIDString ?: @"") forKey:ROW_CMD_ID];
    [dataSourceWithDetailText setObject:(TMFSharkDemoUserConfiguration.u_requestHeadersJSONString ?: @"") forKey:ROW_REQUEST_HEADERS];
    [dataSourceWithDetailText setObject:(TMFSharkDemoUserConfiguration.u_requestCookiesJSONString ?: @"") forKey:ROW_REQUEST_COOKIES];
    [dataSourceWithDetailText setObject:(TMFSharkDemoUserConfiguration.u_requestQueriesString ?: @"") forKey:ROW_REQUEST_QUERIES];
    [dataSourceWithDetailText setObject:(TMFSharkDemoUserConfiguration.u_requestDataString ?: @"") forKey:ROW_REQUEST_DATA];
    [dataSourceWithDetailText setObject:@"" forKey:ROW_TEST_REQUEST];

    [dataSourceWithDetailText setObject:@"" forKey:@"------------------------- Customized User ID"];

    [dataSourceWithDetailText setObject:(_customizedUserID ?: @"") forKey:ROW_USERID];
    [dataSourceWithDetailText setObject:@"" forKey:ROW_UPDATE_USERID];

    self.dataSourceWithDetailText = dataSourceWithDetailText;
}

- (void)reloadTableView {
    [self initDataSource];
    [self.tableView reloadData];
}

- (void)didSelectCellWithTitle:(NSString *)title {
    if ([title isEqualToString:ROW_PRODUCT_KEY]) {
        //[self updateConfiguration];
    } else if ([title isEqualToString:ROW_PRODUCT_ID]) {
        //[self updateConfiguration];
    } else if ([title isEqualToString:ROW_HTTP_URL]) {
        //[self updateConfiguration];
    } else if ([title isEqualToString:ROW_PUBLIC_KEY]) {
        //[self updateConfiguration];
    } else if ([title isEqualToString:ROW_GUID]) {
        [self copyGUID];
    } else if ([title isEqualToString:ROW_VID]) {
        [self fetchVID];
    }

    else if ([title isEqualToString:ROW_CMD]) {
        [self updateCmd];
    } else if ([title isEqualToString:ROW_CMD_ID]) {
        [self updateCmdID];
    } else if ([title isEqualToString:ROW_REQUEST_HEADERS]) {
        [self updateRequestHeaders];
    } else if ([title isEqualToString:ROW_REQUEST_COOKIES]) {
        [self updateRequestCookies];
    } else if ([title isEqualToString:ROW_REQUEST_QUERIES]) {
        [self updateRequestQueries];
    } else if ([title isEqualToString:ROW_REQUEST_DATA]) {
        [self updateRequestData];
    } else if ([title isEqualToString:ROW_TEST_REQUEST]) {
        [self testSharkRequest];
    }

    else if ([title isEqualToString:ROW_UPDATE_USERID]) {
        [self updateCustomizedUserID];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"TMFShark";
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

#pragma mark -

- (void)sharkCenterDidRegisterGUID:(NSNotification *)notification {
    [self reloadTableView];
}

/*
- (void)updateConfiguration {
    TMFSharkCenter *center = [TMFSharkCenter masterCenter];
    TMFSharkCenterConfiguration *configuration = center.configuration;

    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Configuration" message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = configuration.productKey;
        textField.placeholder = ROW_PRODUCT_KEY;
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = [NSString stringWithFormat:@"%ld", (long)configuration.productID];
        textField.placeholder = ROW_PRODUCT_ID;
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = configuration.HTTPURL;
        textField.placeholder = ROW_HTTP_URL;
    }];
    //    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
    //        textField.text = configuration.RSAPublicKey;
    //        textField.placeholder = ROW_PUBLIC_KEY;
    //    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        UIAlertController *alertController2 = [UIAlertController alertControllerWithTitle:@"Configuration"
                                                                                  message:@"Demo 将会重置数据并退出，重启 Demo 后生效"
                                                                           preferredStyle:UIAlertControllerStyleAlert];
        [alertController2 addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
        [alertController2 addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wundeclared-selector"
            // 清空 Shark 数据的私有接口，仅供 demo 使用
            [[TMFSharkCenter masterCenter] performSelector:@selector(_reset)];
#pragma clang diagnostic pop

            TMFSharkDemoUserConfiguration.u_productKey = alertController.textFields[0].text;
            TMFSharkDemoUserConfiguration.u_productIDString = alertController.textFields[1].text;
            TMFSharkDemoUserConfiguration.u_HTTPURL = alertController.textFields[2].text;
            // TMFSharkDemoUserConfiguration.u_publicKey = alertController.textFields[3].text;

            [TMFSharkDemoUserConfiguration synchronize];

            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                abort();
            });
        }]];
        [self presentViewController:alertController2 animated:YES completion:nil];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}
*/
 
- (void)copyGUID {
    NSString *GUID = [TMFSharkCenter masterCenter].GUID;
    if (GUID) {
        [[UIPasteboard generalPasteboard] setString:GUID];
        NSLog(@"[TMFShark] GUID: %@", GUID);
        [QMUITips showSucceed:[@"GUID has copied to clipboard" localizedString] inView:self.view hideAfterDelay:2.0];
    }
}

- (void)fetchVID {
    [QMUITips showLoading:nil detailText:nil inView:self.view];
    [[TMFSharkCenter masterCenter] fetchVID:^(NSString *_Nullable VID, NSError *_Nullable error) {
        [QMUITips hideAllTipsInView:self.view];
        if (VID) {
            [[UIPasteboard generalPasteboard] setString:VID];
            NSLog(@"[TMFShark] VID: %@", VID);
            UIAlertController *alertController =
                [UIAlertController alertControllerWithTitle:nil message:[NSString stringWithFormat:[@"VID: %@\nhas copied to clipboard" localizedString], VID]
                                             preferredStyle:UIAlertControllerStyleAlert];
            [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
            [self presentViewController:alertController animated:YES completion:nil];
        } else {
            UIAlertController *alertController =
                [UIAlertController alertControllerWithTitle:nil message:[NSString stringWithFormat:[@"get VID fail\nError: %@" localizedString], error]
                                             preferredStyle:UIAlertControllerStyleAlert];
            [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
            [self presentViewController:alertController animated:YES completion:nil];
        }
    }];
}

- (void)updateCmd {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_CMD message:nil preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = TMFSharkDemoUserConfiguration.u_cmd;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        TMFSharkDemoUserConfiguration.u_cmd = alertController.textFields.firstObject.text;
        [self reloadTableView];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateCmdID {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_CMD_ID message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = TMFSharkDemoUserConfiguration.u_cmdIDString;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        TMFSharkDemoUserConfiguration.u_cmdIDString = alertController.textFields.firstObject.text;
        [self reloadTableView];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateRequestHeaders {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_REQUEST_HEADERS message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = TMFSharkDemoUserConfiguration.u_requestHeadersJSONString;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        TMFSharkDemoUserConfiguration.u_requestHeadersJSONString = alertController.textFields.firstObject.text;
        [self reloadTableView];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateRequestCookies {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_REQUEST_COOKIES message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = TMFSharkDemoUserConfiguration.u_requestCookiesJSONString;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        TMFSharkDemoUserConfiguration.u_requestCookiesJSONString = alertController.textFields.firstObject.text;
        [self reloadTableView];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateRequestQueries {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_REQUEST_QUERIES message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = TMFSharkDemoUserConfiguration.u_requestQueriesString;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        TMFSharkDemoUserConfiguration.u_requestQueriesString = alertController.textFields.firstObject.text;
        [self reloadTableView];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)updateRequestData {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:ROW_REQUEST_DATA message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = TMFSharkDemoUserConfiguration.u_requestDataString;
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        TMFSharkDemoUserConfiguration.u_requestDataString = alertController.textFields.firstObject.text;
        [self reloadTableView];
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)resetEnvironmentConfiguration {
    [self.navigationController pushViewController:[TMFSharkEnvironmentSelectViewController new] animated:YES];
}

#pragma mark - Shark Sample

- (void)testSharkRequest {
    [QMUITips showLoading:nil detailText:nil inView:self.view];

    TMFOCSharkCmdSettings *cmdSettings = [TMFOCSharkCmdSettings settings];
    cmdSettings.cmd = TMFSharkDemoUserConfiguration.u_cmd;
    [cmdSettings csSetCmdID:[TMFSharkDemoUserConfiguration.u_cmdIDString integerValue] objectClass:Nil];

    // Request Header
    TMFSharkRequestHeaders *requestHeaders = [TMFSharkRequestHeaders headers];
    if (TMFSharkDemoUserConfiguration.u_requestHeadersJSONString) {
        NSData *data = [TMFSharkDemoUserConfiguration.u_requestHeadersJSONString dataUsingEncoding:NSUTF8StringEncoding];
        NSDictionary *dict = [NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:NULL];
        if ([dict isKindOfClass:[NSDictionary class]]) {
            [requestHeaders.headers setValuesForKeysWithDictionary:dict];
        }
    }
    if (TMFSharkDemoUserConfiguration.u_requestCookiesJSONString) {
        NSData *data = [TMFSharkDemoUserConfiguration.u_requestCookiesJSONString dataUsingEncoding:NSUTF8StringEncoding];
        NSDictionary *dict = [NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:NULL];
        if ([dict isKindOfClass:[NSDictionary class]]) {
            [requestHeaders.cookies setValuesForKeysWithDictionary:dict];
        }
    }
    if (TMFSharkDemoUserConfiguration.u_requestQueriesString) {
        NSURL *URL = [NSURL URLWithString:[@"tmf://shark?" stringByAppendingString:TMFSharkDemoUserConfiguration.u_requestQueriesString]];
        NSDictionary *dict = [URL mqqQueryDictionary];
        if (dict) {
            [requestHeaders.queries setValuesForKeysWithDictionary:dict];
        }
    }

    // Request Data
    NSData *requestData = [TMFSharkDemoUserConfiguration.u_requestDataString dataUsingEncoding:NSUTF8StringEncoding];

    // Request
    [[TMFSharkCenter masterCenter] requestWithCmdSettings:cmdSettings requestHeaders:requestHeaders requestData:requestData completionHandler:^(
        TMFSharkResponseHeaders *_Nullable responseHeaders, NSData *_Nullable responseData, NSError *_Nullable error) {
        NSString *responseString = nil;
        if (responseData) {
            responseString = [[NSString alloc] initWithData:responseData encoding:NSUTF8StringEncoding];
        }

        [QMUITips hideAllTipsInView:self.view];

        UIAlertController *alertController = nil;
        if (error == nil) {
            NSMutableString *message = [NSMutableString string];
            [message appendFormat:@"【HTTPStatusCode】\n%ld\n", (long)responseHeaders.HTTPStatusCode];
            [message appendFormat:@"【ResponseHeaders】\n%@\n\n", responseHeaders.headers];
            [message appendFormat:@"【ResponseData】\n%@\n", responseString];
            const NSUInteger messageLimitedLength = 2000;
            if (message.length > messageLimitedLength) {
                [message deleteCharactersInRange:NSMakeRange(messageLimitedLength, message.length - messageLimitedLength)];
                [message appendString:@"\n......"];
            }

            alertController = [UIAlertController alertControllerWithTitle:@"Success" message:message preferredStyle:UIAlertControllerStyleAlert];

            NSMutableParagraphStyle *paragraphStyle = [[NSMutableParagraphStyle alloc] init];
            paragraphStyle.alignment = NSTextAlignmentLeft;
            NSMutableAttributedString *atrStr = [[NSMutableAttributedString alloc] initWithString:message attributes:@ {
                NSParagraphStyleAttributeName: paragraphStyle,
                NSFontAttributeName: [UIFont preferredFontForTextStyle:UIFontTextStyleFootnote],
            }];
            [alertController setValue:atrStr forKey:@"attributedMessage"];
        } else {
            alertController = [UIAlertController alertControllerWithTitle:@"Error"
                                                                  message:[NSString stringWithFormat:@"%@, %ld", error.domain, (long)error.code]
                                                           preferredStyle:UIAlertControllerStyleAlert];
        }
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    }];

    // Cancel
    //    dispatch_async(dispatch_get_main_queue(), ^{
    //        [[TMFSharkCenter masterCenter] cancelRequest:requestID];
    //        NSLog(@"Did cancel request, requestID: %lu", (unsigned long)requestID);
    //    });
}

- (void)updateCustomizedUserID {
    NSString *userID = @"x3dsw23lh";
    [[TMFProfileReporter defaultReporter] updateCustomizedUserID:userID];
}

- (void)tmfProfileDidReport:(NSNotification *)notification {
    NSDictionary *reportItems = notification.userInfo[TMFProfileDidReportItemsKey];
    NSString *stringKey = [NSString stringWithFormat:@"%ld", (long)TMFProfileIDUserID];
    _customizedUserID = reportItems[stringKey];

    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertController *alertController =
            [UIAlertController alertControllerWithTitle:nil
                                                message:[NSString stringWithFormat:@"CustomizedUserID is set to %@", self->_customizedUserID]
                                         preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];

        [self reloadTableView];
    });
}

@end
