//
//  TMFStorageViewController.m
//  TMFDemo
//
//  Created by gavinjwxu on 2022/2/14.
//  Copyright © 2022 Tencent. All rights reserved.
//

#import "TMFStorageViewController.h"
#import "TMFStorage.h"
#import "TestDBModel.h"
#import "TMFStorageDBContext+Test.h"
#import "NSString+Localize.h"

@interface TMFStorageViewController ()

@property (strong,nonatomic) TMFStorageManager *manager;
@property (strong,nonatomic) NSString *userId;
@property (strong,nonatomic) NSString *fileName;
@property (strong,nonatomic) NSData *fileData;

@end

@implementation TMFStorageViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.manager = [TMFStorageManager sharedManager];
    self.userId = @"testUser";
    self.manager.currentUserId = self.userId;
    self.fileName = @"testFile.txt";
    NSString *testFilePath = [[NSBundle mainBundle] pathForResource:self.fileName ofType:nil];
    self.fileData = [NSData dataWithContentsOfFile:testFilePath];
    
    [self.manager.commonDBContext setTables:@[TestDBModel.class]];
    [self.manager.userDBContext setTables:@[TestDBModel.class]];// 在设置 currentUserId 后，为了确保表初始化，应重新设置tables
}

- (void)initDataSource {
    [super initDataSource];
    self.dataSource = @[ [@"Storage path" localizedString],
                         [@"KV save-global" localizedString],
                         [@"KV read-global" localizedString],
                         [@"KV clear-global" localizedString],
                         [@"KV save-user" localizedString],
                         [@"KV read-user" localizedString],
                         [@"KV clear-user" localizedString],
                         [@"KV save-temp" localizedString],
                         [@"KV read-temp" localizedString],
                         [@"KV clear-temp" localizedString],
                         [@"File save-global" localizedString],
                         [@"File read-global" localizedString],
                         [@"File clear-global" localizedString],
                         [@"File -user-omit" localizedString],
                         [@"File -temp-omit" localizedString],
                         [@"Database -create-global" localizedString],
                         [@"Database -delete-global" localizedString],
                         [@"Database -update-global" localizedString],
                         [@"Database -read-global" localizedString],
                         [@"Database -create(custom)-global" localizedString],
                         [@"Database -batch operation(async)-global" localizedString],
                         [@"Database -batch operation(sync)-global" localizedString],
                         [@"Database -clear tables-global" localizedString],
                         [@"Database -user-omit" localizedString],
                         [@"Database -temp-omit" localizedString],
    ];
}

- (void)didSelectCellWithTitle:(NSString *)title {
    if ([title isEqualToString:[@"Storage path" localizedString]]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Storage path" localizedString]
                                                                                 message:[NSString stringWithFormat:@"%@：%@",[@"Storage path" localizedString],[self.manager storageRootPath]]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV save-global" localizedString]]) {
        [self.manager.commonKVManager setString:@"testString" forKey:@"testKey"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV save-global" localizedString]
                                                                                 message:[@"“testString” has been saved to key “testKey”" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV read-global" localizedString]]) {
        NSString *value = [self.manager.commonKVManager getStringForKey:@"testKey" defaultValue:@"123"];
        NSString *msg = nil;
        if ([self.manager.commonKVManager containsKey:@"testKey"]) {
            msg = [NSString stringWithFormat:[@"The key “testKey” has been read, the value is %@" localizedString],value];
        } else {
            msg = [NSString stringWithFormat:[@"The key “testKey” is not read, the default value is %@" localizedString],value];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV read-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV clear-global" localizedString]]) {
        [self.manager.commonKVManager removeValueForKey:@"testKey"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV clear-global" localizedString]
                                                                                 message:[@"The key “testKey” has been cleared" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV save-user" localizedString]]) {
        [self.manager.userKVManager setString:@"testString" forKey:@"testKey"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV save-user" localizedString]
                                                                                 message:[@"“testString” has been saved to key “testKey”" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV read-user" localizedString]]) {
        NSString *value = [self.manager.userKVManager getStringForKey:@"testKey" defaultValue:@"123"];
        NSString *msg = nil;
        if ([self.manager.userKVManager containsKey:@"testKey"]) {
            msg = [NSString stringWithFormat:[@"The key “testKey” has been read, the value is %@" localizedString],value];
        } else {
            msg = [NSString stringWithFormat:[@"The key “testKey” is not read, the default value is %@" localizedString],value];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV read-user" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV clear-user" localizedString]]) {
        [self.manager.userKVManager removeValueForKey:@"testKey"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV clear-user" localizedString]
                                                                                 message:[@"The key “testKey” has been cleared" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV save-temp" localizedString]]) {
        [self.manager.tempKVManager setString:@"testString" forKey:@"testKey"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV save-temp" localizedString]
                                                                                 message:[@"“testString” has been saved to key “testKey”" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV read-temp" localizedString]]) {
        NSString *value = [self.manager.tempKVManager getStringForKey:@"testKey" defaultValue:@"123"];
        NSString *msg = nil;
        if ([self.manager.tempKVManager containsKey:@"testKey"]) {
            msg = [NSString stringWithFormat:[@"The key “testKey” has been read, the value is %@" localizedString],value];
        } else {
            msg = [NSString stringWithFormat:[@"The key “testKey” is not read, the default value is %@" localizedString],value];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV read-temp" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"KV clear-temp" localizedString]]) {
        [self.manager.tempKVManager removeValueForKey:@"testKey"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"KV clear-temp" localizedString]
                                                                                 message:[@"The key “testKey” has been cleared" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    }
    
    else if ([title isEqualToString:[@"File save-global" localizedString]]) {
        [self.manager.commonFileManager setFileWithData:self.fileData fileName:self.fileName key:@"98745612301"];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"File save-global" localizedString]
                                                                                 message:[@"The file has been saved" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"File read-global" localizedString]]) {
        NSData *fileData = [self.manager.commonFileManager getFile:self.fileName key:@"98745612301"];
        NSString *msg = nil;
        if (fileData) {
            NSString *content = [[NSString alloc] initWithData:fileData encoding:NSUTF8StringEncoding];
            msg = [NSString stringWithFormat:[@"The file %@ has been readed, the decrypted string is: %@" localizedString],self.fileName,content];
        } else {
            msg = [NSString stringWithFormat:[@"The file %@ not read" localizedString],self.fileName];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"File read-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"File clear-global" localizedString]]) {
        [self.manager.commonFileManager removeFile:self.fileName];
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"File clear-global" localizedString]
                                                                                 message:[NSString stringWithFormat:[@"The file %@ has been cleared" localizedString],self.fileName]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    }
    
    else if ([title isEqualToString:[@"Database -create-global" localizedString]]) {
        TestDBModel *testModel = [[TestDBModel alloc] init];
        testModel.key = @"1";
        testModel.test = @"this is a test";
        
        BOOL result = [self.manager.commonDBContext insertObject:testModel];
        NSString *msg = nil;
        if (result) {
            msg = [@"Data added successfully" localizedString];
        } else {
            msg = [@"Data addition failed because there is already duplicate data" localizedString];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -create-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -delete-global" localizedString]]) {
        //删除的方法一般需要自己实现，可以查看TMFStorageDBContext+Test 中的实现方式
        BOOL result = [self.manager.commonDBContext deleteModelWithKey:@"1"];
        NSString *msg = nil;
        if (result) {
            msg = [@"Data deleted successfully" localizedString];
        } else {
            msg = [@"Data deletion failed" localizedString];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -delete-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -update-global" localizedString]]) {
        TestDBModel *testModel = [[TestDBModel alloc] init];
        testModel.key = @"1";
        testModel.test = @"this is a test for update data";
        
        BOOL result = [self.manager.commonDBContext insertOrReplaceObject:testModel];
        NSString *msg = nil;
        if (result) {
            msg = [@"Data updated successfully" localizedString];
        } else {
            msg = [@"Data update failed" localizedString];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -update-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -read-global" localizedString]]) {
        TestDBModel *testModel = [self.manager.commonDBContext getOneObjectOfClass:TestDBModel.class];
        NSString *msg = nil;
        if (testModel) {
            msg = [NSString stringWithFormat:[@"Database query succeeded, the result is %@" localizedString],[testModel description]];
        } else {
            msg = [@"Database query did not find the data" localizedString];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -read-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -create(custom)-global" localizedString]]) {
        TestDBModel *testModel = [self.manager.commonDBContext selectTestModelWithTest:@"this is a test for update data"];
        NSString *msg = nil;
        if (testModel) {
            msg = [NSString stringWithFormat:[@"Database query(custom) succeeded, the result is %@" localizedString],[testModel description]];
        } else {
            msg = [@"Database query(custom) did not find the data" localizedString];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -create(custom)-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -batch operation(async)-global" localizedString]]) {
        TestDBModel *testModel = [[TestDBModel alloc] init];
        testModel.key = @"2";
        testModel.test = @"this is a test";
        [self.manager.commonDBContext performBlock:^(TMFStorageDBContext *db) {
            [db insertObject:testModel];
            [db deleteModelWithKey:@"2"];
            NSLog(@"111");
        }];
        NSLog(@"222");
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -batch operation(async)-global" localizedString]
                                                                                 message:[@"The processing is complete, please pay attention to the order in which the console logs are printed" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -batch operation(sync)-global" localizedString]]) {
        TestDBModel *testModel = [[TestDBModel alloc] init];
        testModel.key = @"2";
        testModel.test = @"this is a test";
        [self.manager.commonDBContext performBlockAndWait:^(TMFStorageDBContext *db) {
            [db insertObject:testModel];
            [db deleteModelWithKey:@"2"];
            NSLog(@"111");
        }];
        NSLog(@"222");
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -batch operation(sync)-global" localizedString]
                                                                                 message:[@"The processing is complete, please pay attention to the order in which the console logs are printed" localizedString]
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    } else if ([title isEqualToString:[@"Database -clear tables-global" localizedString]]) {
        BOOL result = [self.manager.commonDBContext clearTable:NSStringFromClass(TestDBModel.class)];
        NSString *msg = nil;
        if (result) {
            msg = [@"Database cleanup succeeded" localizedString];
        } else {
            msg = [@"Database cleanup failed" localizedString];
        }
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"Database -clear tables-global" localizedString]
                                                                                 message:msg
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"TMFStorage";
}

@end
