//
//  TMFPickerController.h
//  TMFDemo
//
//  Created by v_zwtzzhou on 2021/3/3.
//  Copyright © 2021 Tencent. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger, TMFPickerControllerSourceType) {
    TMFPickerControllerSourceTypePhotoLibrary,
    TMFPickerControllerSourceTypeCamera,
    TMFPickerControllerSourceTypeSavedPhotosAlbum,
};

@class TMFPickerController,TMFMedia;
@protocol TMFPickerControllerDelegate <NSObject>

- (void)tmfPickerController:(TMFPickerController *)pickerController didFinishPickingMedia:(NSArray <TMFMedia*> *)medias;
- (void)tmfPickerControllerDidCancel:(TMFPickerController *)pickerController;

@end

/**
 @brief 媒体选择控制器
 */
@interface TMFPickerController : NSObject

/**
 @brief 媒体选择 [PHPickerConfiguration、UIImagePickerController]
 */
@property (nonatomic, strong, readonly) UIViewController *picker;

/**
 @brief 数据来源
 */
@property (nonatomic, assign) TMFPickerControllerSourceType sourceType;

/**
 @brief 是否使用编辑功能
 */
@property (nonatomic, assign) BOOL allowsEditing;

/**
 @brief TMFPickerControllerDelegate
 */
@property (nonatomic, weak) id<TMFPickerControllerDelegate> delegate;

@end

#pragma mark - Media Model
typedef NS_ENUM(NSInteger, TMFCustomMediaType) {
    TMFCustomMediaTypeUnknown   = -1,
    TMFCustomMediaTypeImage     = 0,
    TMFCustomMediaTypeVideo     = 1,
};


/**
 @brief 封装的媒体对象
 */
@interface TMFMedia : NSObject
@property (nonatomic, assign) TMFCustomMediaType type;
@end

/**
 @brief 图片类型媒体对象
 */
@interface TMFMediaImage : TMFMedia

/**
 @brief UIImage
 */
@property (nonatomic, strong) UIImage *image;
@end

/**
 @brief 视频类型媒体对象
 */
@interface TMFMediaVideo : TMFMedia

/**
 @brief 视频URL
 */
@property (nonatomic, strong) NSURL *URL;
@end

NS_ASSUME_NONNULL_END
