//
//  TMFPickerController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2021/3/3.
//  Copyright © 2021 Tencent. All rights reserved.
//

#import "TMFPickerController.h"

#import <AssetsLibrary/AssetsLibrary.h>
#import <MobileCoreServices/MobileCoreServices.h>

#import <Photos/Photos.h>
#import <PhotosUI/PhotosUI.h>



@interface TMFPickerController ()<UIImagePickerControllerDelegate,UINavigationControllerDelegate, PHPickerViewControllerDelegate>
@property (nonatomic, strong, readwrite) UIViewController *picker;

@end

@implementation TMFPickerController

- (instancetype)init
{
    self = [super init];
    if (self) {
        [self installPicker];
    }
    return self;
}

- (void)installPicker{
    if (@available(iOS 14, *)) {
        PHPickerConfiguration *configuration = [[PHPickerConfiguration alloc] initWithPhotoLibrary:[PHPhotoLibrary sharedPhotoLibrary]];
        PHPickerViewController *pickerController = [[PHPickerViewController alloc] initWithConfiguration:configuration];
        pickerController.delegate = self;
        self.picker = pickerController;
    } else {
        // Fallback on earlier versions
        UIImagePickerController *pickerController = [[UIImagePickerController alloc] init];
        pickerController.delegate = self;
        [pickerController setMediaTypes: @[( NSString *)kUTTypeMovie, (NSString *)kUTTypeImage]];
        self.picker = pickerController;
    }
}


#pragma mark - UIImagePickerControllerDelegate
- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker{
    if (self.delegate && [self.delegate respondsToSelector:@selector(tmfPickerControllerDidCancel:)]) {
        [self.delegate tmfPickerControllerDidCancel:self];
    }
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<UIImagePickerControllerInfoKey,id> *)info{
    
    TMFMedia *media = nil;
    if ([info[UIImagePickerControllerMediaType] isEqualToString: (NSString *)kUTTypeImage] ) {
        UIImage *pickerImage = [info objectForKey:UIImagePickerControllerOriginalImage];
        TMFMediaImage *imageMedia = [[TMFMediaImage alloc] init];
        imageMedia.type = TMFCustomMediaTypeImage;
        imageMedia.image = pickerImage;
        media = imageMedia;
    }else if ([info[UIImagePickerControllerMediaType] isEqualToString: (NSString *)kUTTypeMovie]) {
        TMFMediaVideo *videoMedia = [[TMFMediaVideo alloc] init];
        videoMedia.type = TMFCustomMediaTypeVideo;
        videoMedia.URL = [info objectForKey:@"UIImagePickerControllerMediaURL"];
        media = videoMedia;
    }
    if (self.delegate && [self.delegate respondsToSelector:@selector(tmfPickerController:didFinishPickingMedia:)]) {
        [self.delegate tmfPickerController:self didFinishPickingMedia:@[media]];
    }
}

#pragma mark - PHPickerViewControllerDelegate
- (void)picker:(PHPickerViewController *)picker didFinishPicking:(NSArray<PHPickerResult *> *)results API_AVAILABLE(ios(14)){
    if (results.count == 0) {
        if (self.delegate && [self.delegate respondsToSelector:@selector(tmfPickerControllerDidCancel:)]) {
            [self.delegate tmfPickerControllerDidCancel:self];
        }
    }else{
        PHPickerResult *result = results.firstObject;
        if ([result.itemProvider canLoadObjectOfClass:[UIImage class]]) {
            [result.itemProvider loadDataRepresentationForTypeIdentifier:(NSString *)kUTTypeImage completionHandler:^(NSData * _Nullable data, NSError * _Nullable error) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (error) {
                        if ([self.delegate respondsToSelector:@selector(tmfPickerController:didFinishPickingMedia:)]) {
                            [self.delegate tmfPickerController:self didFinishPickingMedia:@[]];
                        }
                        return;
                    }
                    TMFMediaImage *imageMedia  = [[TMFMediaImage alloc] init];
                    imageMedia.type = TMFCustomMediaTypeImage;
                    imageMedia.image = [UIImage imageWithData:data];
                    if ([self.delegate respondsToSelector:@selector(tmfPickerController:didFinishPickingMedia:)]) {
                        [self.delegate tmfPickerController:self didFinishPickingMedia:@[imageMedia]];
                    }
                });
            }];
        } else {
            [result.itemProvider loadItemForTypeIdentifier:(NSString *)kUTTypeMovie options:@{NSItemProviderPreferredImageSizeKey:[NSValue valueWithCGSize:CGSizeMake(150, 150)]} completionHandler:^(__kindof id<NSSecureCoding>  _Nullable item, NSError * _Null_unspecified error) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (error || ![item isKindOfClass:[NSURL class]]) {
                        if ([self.delegate respondsToSelector:@selector(tmfPickerController:didFinishPickingMedia:)]) {
                            [self.delegate tmfPickerController:self didFinishPickingMedia:@[]];
                        }
                        return;
                    }
                    TMFMediaVideo *videoMedia = videoMedia = [[TMFMediaVideo alloc] init];
                    videoMedia.type = TMFCustomMediaTypeVideo;
                    videoMedia.URL = (NSURL *)item;
                    if ([self.delegate respondsToSelector:@selector(tmfPickerController:didFinishPickingMedia:)]) {
                        [self.delegate tmfPickerController:self didFinishPickingMedia:@[videoMedia]];
                    }
                });
            }];
        }
    }
}

#pragma mark - Setter & Getter
- (BOOL)allowsEditing{
    if ([self.picker isKindOfClass:[UIImagePickerController class]]) {
        return ((UIImagePickerController *)self.picker).allowsEditing;
    }else{
        return YES;
    }
}

- (void)setAllowsEditing:(BOOL)allowsEditing{
    if ([self.picker isKindOfClass:[UIImagePickerController class]]) {
        ((UIImagePickerController *)self.picker).allowsEditing = allowsEditing;
    }
}

- (TMFPickerControllerSourceType)sourceType{
    if ([self.picker isKindOfClass:[UIImagePickerController class]]) {
        return (TMFPickerControllerSourceType)((UIImagePickerController *)self.picker).sourceType;
    }else{
        return 1;
    }
}

- (void)setSourceType:(TMFPickerControllerSourceType)sourceType{
    if ([self.picker isKindOfClass:[UIImagePickerController class]]) {
        ((UIImagePickerController *)self.picker).sourceType = (UIImagePickerControllerSourceType)sourceType;
    }
}

@end




@implementation TMFMedia
@end
@implementation TMFMediaImage
@end
@implementation TMFMediaVideo
@end
