//
//  TMFUploadTableCell.m
//  TMFDemo
//
//  Created by hauzhong on 2019/7/22.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFUploadTableCell.h"

#import <Masonry/Masonry.h>
#import "NSString+Localize.h"

@interface TMFUploadTableCell ()
@property (strong) UILabel *fileLabel;
@property (strong) UILabel *URLLabel;
@property (strong) UIButton *opButton;
@property (strong) UIProgressView *progressView;
@property (assign) TMFUploadTableCellState state;
@end

@implementation TMFUploadTableCell

- (void)dealloc {
    self.fileLabel = nil;
    self.URLLabel = nil;
    self.opButton = nil;
    self.progressView = nil;
}

- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
}

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(nullable NSString *)reuseIdentifier {
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        [self makeSubViews];
        [self setAutoLayout];
        self.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    return self;
}

- (void)makeSubViews {
    _fileLabel = [[UILabel alloc] init];
    _fileLabel.textAlignment = NSTextAlignmentLeft;
    _fileLabel.textColor = UIColor.blackColor;
    _fileLabel.font = [UIFont systemFontOfSize:15];
    
    _URLLabel = [[UILabel alloc] init];
    _URLLabel.textAlignment = NSTextAlignmentLeft;
    _URLLabel.textColor = UIColor.blueColor;
    _URLLabel.font = [UIFont systemFontOfSize:13];
    
    _opButton = [UIButton buttonWithType:UIButtonTypeSystem];
    _opButton.backgroundColor = UIColor.redColor;
    [_opButton setTitleColor:UIColor.whiteColor forState:UIControlStateNormal];
    [_opButton addTarget:self action:@selector(resume) forControlEvents:UIControlEventTouchUpInside];
    
    _progressView = [[UIProgressView alloc] initWithProgressViewStyle:UIProgressViewStyleDefault];
    
    [self.contentView addSubview:_opButton];
    [self.contentView addSubview:_fileLabel];
    [self.contentView addSubview:_URLLabel];
    [self.contentView addSubview:_progressView];
}

- (void)setAutoLayout {
    
    
    CGFloat width = CGRectGetWidth([[UIScreen mainScreen] bounds]);
    CGRect rect = CGRectMake(0, 0, width, 44.0);
    self.frame = rect;
    
    __weak typeof(self) weakSelf = self;
    [self.opButton mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(weakSelf.contentView.mas_top).offset(0);
        make.left.equalTo(weakSelf.contentView.mas_left).offset(0);
        make.width.mas_equalTo(44);
        make.bottom.equalTo(weakSelf.mas_bottom).offset(0);
    }];
    
    [self.fileLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(weakSelf.opButton).offset(0);
        make.left.equalTo(weakSelf.opButton.mas_right).offset(5);
        make.width.mas_equalTo(280);
        make.height.mas_equalTo(24);
    }];
    
    [self.URLLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(weakSelf.fileLabel.mas_bottom).offset(1);
        make.left.equalTo(weakSelf.fileLabel);
        make.width.mas_equalTo(280);
        make.height.mas_equalTo(15);
    }];
    
    [self.progressView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(weakSelf.contentView.mas_bottom).offset(0);
        make.right.equalTo(weakSelf.contentView.mas_right).offset(0);
        make.width.equalTo(weakSelf.contentView);
        make.height.mas_equalTo(5);
    }];
}

- (void)reloadDataWithFile:(NSString *)file {
    self.fileLabel.text = file;
    self.URLLabel.text = nil;
    self.progressView.progress = 0.0;
    self.accessoryType = UITableViewCellAccessoryNone;
    self.state = TMFUploadTableCellStateSuspended;
    [self.opButton setTitle:[@"start" localizedString] forState:UIControlStateNormal];
}

- (void)reloadDataWithResult:(NSString *)result accessoryType:(UITableViewCellAccessoryType)accessoryType {
    
    if (accessoryType == UITableViewCellAccessoryCheckmark) {
        self.progressView.progress = 1.0;
        self.URLLabel.text = result;
        self.URLLabel.textColor = [UIColor blueColor];
        self.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        self.URLLabel.text = result;
        self.URLLabel.textColor = [UIColor redColor];
        self.accessoryType = accessoryType;
    }
}

- (void)reloadState:(TMFUploadTableCellState)state {
    if (state == TMFUploadTableCellStateRunning) {
        [self.opButton setTitle:[@"stop" localizedString] forState:UIControlStateNormal];
    } else if (state == TMFUploadTableCellStateSuspended) {
        [self.opButton setTitle:[@"start" localizedString] forState:UIControlStateNormal];
    } else {
        [self.opButton setTitle:[@"end" localizedString] forState:UIControlStateNormal];
    }
    self.state = state;
}

- (void)resume {
    !self.handler ? : self.handler(self.state);
    if (self.state == TMFUploadTableCellStateSuspended) {
        self.state = TMFUploadTableCellStateRunning;
        [self.opButton setTitle:[@"stop" localizedString] forState:UIControlStateNormal];
    } else if (self.state == TMFUploadTableCellStateRunning) {
        self.state = TMFUploadTableCellStateSuspended;
        [self.opButton setTitle:[@"start" localizedString] forState:UIControlStateNormal];
    } else {
        [self.opButton setTitle:[@"end" localizedString] forState:UIControlStateNormal];
    }
}

@end
