//
//  TMFUploadViewController.m
//  TMFDemo
//
//  Created by hauzhong on 2019/7/18.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFUploadViewController.h"

#import "TMFUploadTableCell.h"
#import "TMFUploadViewModel.h"
#import "TMFPickerController.h"
#import <AVKit/AVKit.h>
#import "NSString+Localize.h"

@interface TMFUploadViewController () <TMFPickerControllerDelegate>
@property (strong) TMFUploadViewModel *viewModel;
@property (strong) TMFPickerController *pickerController;;

@end

@implementation TMFUploadViewController

- (void)dealloc {
    self.viewModel = nil;
    self.pickerController = nil;
}

- (void)initDataSource {
    [super initDataSource];
    self.viewModel = [[TMFUploadViewModel alloc] init];
      
    self.pickerController = [[TMFPickerController alloc] init];
    self.pickerController.delegate = self;
    
    [self.tableView registerClass:[TMFUploadTableCell class] forCellReuseIdentifier:NSStringFromClass(TMFUploadTableCell.class)];
    
    [self loadInCompleteUploadfiles];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.viewModel.files ? self.viewModel.files.count : 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    TMFUploadTableCell *cell = [tableView dequeueReusableCellWithIdentifier:NSStringFromClass(TMFUploadTableCell.class) forIndexPath:indexPath];
    
    NSString *path = [self.viewModel.files objectAtIndex:indexPath.row];
    if ([self.viewModel containTaskWithFile:path]) { // 当前文件路径已经创建了上传任务，无需再次创建!!!
        return cell;
    }
    [cell reloadDataWithFile:[path lastPathComponent]];
    
    // Note: 根据demo演示的需要，通过 `TableView` 来添加文件上传任务，每个cell对应一个文件，因此在cell生成的时候，同时分配上传任务以及控制器，实际接入时无需按照下面的步骤!!!
    // 一般上传操作流程是 分配 `TMFUploadTask` 任务的实例对象，然后调用实例对象的 `-resume` 即可!!!
    // Step 1. 分配上传任务
//    NSDictionary *param = @{@"tenantId":@"photo",@"type":@"jpg",@"fileName":@"testImg"};
//    TMFUploadFileURI *fileURI = [TMFUploadFileURI URIWithFilePath:path businessId:@"busId1" params:param];

    TMFUploadFileURI *fileURI = [TMFUploadFileURI URIWithFilePath:path businessId:@"busId1"];
//    TMFDemo测试使用
//    TMFUploadTask *uploadTask = [[TMFUploadManager defaultManager] uploadTaskWithFileURI:fileURI progressHandler:^(NSProgress * _Nullable progress) {
//        NSLog(@"%@",progress);
//        dispatch_async(dispatch_get_main_queue(), ^{
//            cell.progressView.observedProgress = progress;
//            [cell reloadState:TMFUploadTableCellStateRunning];
//        });
//    } completionHandler:^(NSString * _Nullable URL, NSError * _Nullable error) {
//        NSLog(@"URL: %@, error : %@",URL, error);
//        dispatch_async(dispatch_get_main_queue(), ^{
//            if (!error) {
//                [[NSFileManager defaultManager] removeItemAtPath:path error:nil]; // demo演示时，避免tmp文件过多，在上传文件成功后，就删除掉本地缓存，实际接入时续根据需要谨慎操作!!!
//                [cell reloadState:TMFUploadTableCellStateCompleted];
//                [cell reloadDataWithResult:URL accessoryType:UITableViewCellAccessoryCheckmark];
//            } else {
//                [cell reloadState:TMFUploadTableCellStateSuspended];
//                [cell reloadDataWithResult:[error description] accessoryType:UITableViewCellAccessoryNone];
//            }
//        });
//    }];
    
    TMFUploadTask *uploadTask = [[TMFUploadManager defaultManager] uploadTaskWithFileURI:fileURI progressHandler:^(NSProgress * _Nullable progress) {
        NSLog(@"%@",progress);
        dispatch_async(dispatch_get_main_queue(), ^{
            cell.progressView.observedProgress = progress;
            [cell reloadState:TMFUploadTableCellStateRunning];
        });
    } completionOriginalHandler:^(NSData * _Nullable result, NSError * _Nullable error) {
        NSString *resString = [[NSString alloc] initWithData:result encoding:NSUTF8StringEncoding];
        NSLog(@"result: %@, error : %@", resString, error);
        dispatch_async(dispatch_get_main_queue(), ^{
            if (!error) {
                [[NSFileManager defaultManager] removeItemAtPath:path error:nil]; // demo演示时，避免tmp文件过多，在上传文件成功后，就删除掉本地缓存，实际接入时续根据需要谨慎操作!!!
                [cell reloadState:TMFUploadTableCellStateCompleted];
                [cell reloadDataWithResult:resString accessoryType:UITableViewCellAccessoryCheckmark];
            } else {
                [cell reloadState:TMFUploadTableCellStateSuspended];
                [cell reloadDataWithResult:[error description] accessoryType:UITableViewCellAccessoryNone];
            }
        });
    }];
    
    // Step 2. 缓存上传任务
    if (uploadTask) { // 上传时，传入的文件本地沙盒路径不存在时，生成的uploadTask将为nil !!!
        [self.viewModel addUploadTask:uploadTask];
    }
    
    // Step 3. 分配cell单元格控制上传任务
    cell.handler = ^(TMFUploadTableCellState state) {
        if (state == TMFUploadTableCellStateRunning) {
            [uploadTask cancel];
        } else if (state == TMFUploadTableCellStateSuspended) {
            [uploadTask resume];
        }
    };
    
    return cell;
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"TMFUploader";
    UIBarButtonItem *addfileButton = [UIBarButtonItem qmui_itemWithTitle:[@"add" localizedString] target:self action:@selector(addEvent)];
    UIBarButtonItem *uploadfileButton = [UIBarButtonItem qmui_itemWithTitle:[@"upload" localizedString] target:self action:@selector(uploadEvent)];
    
    self.navigationItem.rightBarButtonItems = @[uploadfileButton, addfileButton];
}

- (void)addEvent {
    // 重新添加文件时，需要将之前上传列表中已经上传成功的文件移除掉!!!
    [self reloadData];
    // 添加文件
    self.pickerController.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    self.pickerController.allowsEditing = NO;
    [self presentViewController:self.pickerController.picker animated:YES completion:NULL];
}

- (void)uploadEvent {
    [self.viewModel resume];
}

- (void)reloadData {
    [self.viewModel reloadData];
    [self.tableView reloadData];
}

- (void)loadInCompleteUploadfiles {
    [self.viewModel reloadInCompleteUploadfiles];
    [self.tableView reloadData];
}

- (void)addUploadFile:(NSString *)path {
    [self.viewModel addUploadFile:path];
    [self.tableView reloadData];
}

#pragma mark - UIImagePickerControllerDelegate

- (void)tmfPickerController:(TMFPickerController *)pickerController didFinishPickingMedia:(nonnull NSArray<TMFMedia *> *)medias {
    UIWindow *mainWindow = [UIApplication sharedApplication].keyWindow ?: [[UIApplication sharedApplication] windows].firstObject;
    if (medias.count == 0) {
        [pickerController.picker dismissViewControllerAnimated:YES completion:nil];
        [QMUITips showLoading:[@"upload cancel" localizedString] inView:mainWindow hideAfterDelay:3];
        return;
    }
    TMFMedia *media = medias.firstObject;
    if (media.type == TMFCustomMediaTypeImage) {
        QMUITips *tips = [QMUITips showLoading:[@"loading, please wait" localizedString] inView:mainWindow hideAfterDelay:15];
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            NSString *updateFilePath;
            NSTimeInterval timestamp = CFAbsoluteTimeGetCurrent() + kCFAbsoluteTimeIntervalSince1970;
            NSString *name = [NSString stringWithFormat:@"%lf.png", timestamp];
            updateFilePath = [self.viewModel.temporaryPath stringByAppendingPathComponent:name];
            NSData *imageData = UIImagePNGRepresentation(((TMFMediaImage *)media).image);
            [imageData writeToFile:updateFilePath atomically:YES];
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [tips hideAnimated:YES];
                [self addUploadFile:updateFilePath];
                [pickerController.picker dismissViewControllerAnimated:YES completion:nil];
            });
        });
    }else if (media.type == TMFCustomMediaTypeVideo){
        
        QMUITips *tips = [QMUITips showLoading:[@"processing video, please wait" localizedString] inView:mainWindow hideAfterDelay:15];
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            NSString *updateFilePath;
            NSTimeInterval timestamp = CFAbsoluteTimeGetCurrent() + kCFAbsoluteTimeIntervalSince1970;
            NSString *name = [NSString stringWithFormat:@"%lf.mp4", timestamp];
            updateFilePath = [self.viewModel.temporaryPath stringByAppendingPathComponent:name];
            [self startExportVideoWithVideoPath:((TMFMediaVideo *)media).URL outputPath:updateFilePath completion:^{
                dispatch_async(dispatch_get_main_queue(), ^{
                    [tips hideAnimated:YES];
                    [self addUploadFile:updateFilePath];
                    [pickerController.picker dismissViewControllerAnimated:YES completion:nil];
                });
            }];
        });
    }
}

- (void)tmfPickerControllerDidCancel:(TMFPickerController *)pickerController {
    [pickerController.picker dismissViewControllerAnimated:YES completion:NULL];
}

- (void)startExportVideoWithVideoPath:(NSURL *)URL outputPath:(NSString *)outpath completion:(void (^)(void))completion{
    
    AVURLAsset *asset = [AVURLAsset assetWithURL:URL];
    AVAssetExportSession *session = [[AVAssetExportSession alloc] initWithAsset:asset presetName:AVAssetExportPreset1280x720];
    session.outputURL = [NSURL fileURLWithPath:outpath];
    session.outputFileType = AVFileTypeMPEG4;
    [session exportAsynchronouslyWithCompletionHandler:^{
        if (completion) {
            completion();
        }
    }];
    if (session.error) {
        NSLog(@"session error:%@", session.error);
    }
}

@end
