//
//  TMFUploadViewModel.m
//  TMFDemo
//
//  Created by hauzhong on 2019/7/22.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFUploadViewModel.h"

@interface TMFUploadViewModel ()
@property (copy) NSString *temporaryPath;
@property (strong) NSMutableArray<NSString *> *files;
@property (strong) NSMutableArray<TMFUploadTask *> *tasks;
@end

@implementation TMFUploadViewModel

- (void)dealloc {
    [self abort];
    self.files = nil;
    self.tasks = nil;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        self.files = [NSMutableArray<NSString *> array];
        self.tasks = [NSMutableArray<TMFUploadTask *> array];
        
        // demo演示的需要，选取的照片临时缓存在 `tmp/TMFUpload` 下。实际使用时请根据需要放置!!!!
        self.temporaryPath = [NSTemporaryDirectory() stringByAppendingPathComponent:@"TMFUpload"];
        if (![[NSFileManager defaultManager] fileExistsAtPath:self.temporaryPath]) {
            [[NSFileManager defaultManager] createDirectoryAtPath:self.temporaryPath withIntermediateDirectories:YES attributes:nil error:nil];
        }
    }
    return self;
}

- (void)addUploadFile:(NSString *)path {
    [self.files addObject:path];
}

- (void)reloadInCompleteUploadfiles {
    // demo演示的需要，从 `tmp/TMFUpload` 中加载未上传完的图片。实际使用时请根据需要操作!!!!
    NSString *path = self.temporaryPath;
    NSArray *files = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:path error:nil];
    for (NSString *file in files) {
        [self.files addObject:[path stringByAppendingPathComponent:file]];
    }
}

- (void)addUploadTask:(TMFUploadTask *)task {
    [self.tasks addObject:task];
}

- (void)resume {
    for (TMFUploadTask *uploadTask in self.tasks) {
        if (uploadTask.state != TMFUploadTaskStateRunning) {
            [uploadTask resume];
        }
    }
}

- (BOOL)containTaskWithFile:(NSString *)file {
    BOOL ret = NO;
    for (TMFUploadTask *uploadTask in self.tasks) {
        if ([uploadTask.filePath containsString:[file lastPathComponent]]) {
            ret = YES;
            break;
        }
    }
    return ret;
}

- (void)resumeWithFile:(NSString *)file {
    for (TMFUploadTask *uploadTask in self.tasks) {
        if ([uploadTask.filePath containsString:[file lastPathComponent]]) {
            if (uploadTask.state != TMFUploadTaskStateRunning) {
                [uploadTask resume];
            }
            break;
        }
    }
}

- (void)cancelWithFile:(NSString *)file {
    for (TMFUploadTask *uploadTask in self.tasks) {
        if ([uploadTask.filePath containsString:[file lastPathComponent]]) {
            if (uploadTask.state == TMFUploadTaskStateRunning) {
                [uploadTask cancel];
            }
            break;
        }
    }
}

- (void)abort {
    [self.files removeAllObjects];
    [self.tasks enumerateObjectsUsingBlock:^(TMFUploadTask * _Nonnull uploadTask, NSUInteger idx, BOOL * _Nonnull stop) {
        if (uploadTask.state == TMFUploadTaskStateRunning) {
            [uploadTask cancel];
        }
    }];
    [self.tasks removeAllObjects];
}

- (void)reloadData {
    [self abort];
    [self reloadInCompleteUploadfiles];
}

@end
