//
//  TMFWebOllinePresetListViewController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2021/7/21.
//  Copyright © 2021 Tencent. All rights reserved.
//

#import "TMFWebOllinePresetListViewController.h"

@interface TMFWebOllinePresetListViewController ()

@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray<TMFWebOfflinePackageInfo *> *selectedPackages;
@property (nonatomic, strong) NSMutableArray<TMFWebOfflinePackageInfo *> *otherPackages;

@end

@implementation TMFWebOllinePresetListViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.selectedPackages = [[NSMutableArray alloc] init];
    self.otherPackages = [[NSMutableArray alloc] initWithArray:self.allSource];

    self.tableView = [[UITableView alloc] initWithFrame:self.view.bounds style:UITableViewStylePlain];
    [self.view addSubview:self.tableView];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = UIView.new;
    self.tableView.rowHeight = 55;
    self.tableView.sectionHeaderHeight = 44;

    [self.tableView setEditing:YES];
    self.navigationItem.rightBarButtonItem = self.editButtonItem;

    UIButton *doButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [doButton setTitle:@"生成预置离线包" forState:UIControlStateNormal];
    doButton.bounds = CGRectMake(0, 0, 200, 40);
    doButton.center = CGPointMake(CGRectGetWidth(self.view.bounds) / 2 + 60, CGRectGetHeight(self.view.bounds) - 80);
    doButton.backgroundColor = [UIColor colorWithRed:94 / 255.0 green:196 / 255.0 blue:230 / 255.0 alpha:1];
    doButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin;
    [doButton addTarget:self action:@selector(clickDoButton:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:doButton];

    UIButton *allButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [allButton setTitle:@"选择全部" forState:UIControlStateNormal];
    [allButton setTitle:@"取消选择全部" forState:UIControlStateSelected];
    [allButton setTitleColor:UIColor.redColor forState:UIControlStateSelected];
    allButton.bounds = CGRectMake(0, 0, 200, 40);
    allButton.center = CGPointMake(CGRectGetWidth(self.view.bounds) / 2 + 60, CGRectGetMinY(doButton.frame) - 40);
    allButton.backgroundColor = [UIColor colorWithRed:94 / 255.0 green:196 / 255.0 blue:230 / 255.0 alpha:1];
    allButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin;
    [allButton addTarget:self action:@selector(clickAllButton:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:allButton];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0) {
        return self.selectedPackages.count;
    } else {
        return self.otherPackages.count;
    }
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    return section == 0 ? @"选中离线包" : @"忽略离线包";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"UITableViewCell"];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"UITableViewCell"];
    }
    TMFWebOfflinePackageInfo *packageInfo;
    if (indexPath.section == 0) {
        packageInfo = self.selectedPackages[indexPath.row];
    } else {
        packageInfo = self.otherPackages[indexPath.row];
    }

    cell.textLabel.text = packageInfo.BID;
    cell.detailTextLabel.text = @(packageInfo.version).stringValue;

    return cell;
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
    return indexPath.section == 0 ? UITableViewCellEditingStyleDelete : UITableViewCellEditingStyleInsert;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    TMFWebOfflinePackageInfo *packageInfo;
    if (indexPath.section == 0) {
        packageInfo = self.selectedPackages[indexPath.row];
    } else {
        packageInfo = self.otherPackages[indexPath.row];
    }

    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [self.selectedPackages removeObject:packageInfo];
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];

        [self.otherPackages insertObject:packageInfo atIndex:0];
        [tableView insertRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:0 inSection:1]] withRowAnimation:UITableViewRowAnimationAutomatic];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        [self.selectedPackages addObject:packageInfo];
        [tableView insertRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:self.selectedPackages.count - 1 inSection:0]]
                         withRowAnimation:UITableViewRowAnimationAutomatic];

        [self.otherPackages removeObject:packageInfo];
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    return NO;
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
}

#pragma makr - clickButton
- (void)clickDoButton:(UIButton *)button {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self.doButtonHandle) {
            self.doButtonHandle(self.selectedPackages);
        }
    });
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)clickAllButton:(UIButton *)button {
    if (button.selected) {
        button.selected = NO;
        [self.selectedPackages removeAllObjects];
        [self.otherPackages setArray:self.allSource];
        [self.tableView reloadData];
    } else {
        button.selected = YES;
        [self.selectedPackages setArray:self.allSource];
        [self.otherPackages removeAllObjects];
        [self.tableView reloadData];
    }
}

@end
