//
//  TMFWebOllinePresetViewController.m
//  TMFDemo
//
//  Created by v_zwtzzhou on 2021/7/21.
//  Copyright © 2021 Tencent. All rights reserved.
//

#import "TMFWebOllinePresetViewController.h"
#import "TMFWebOfflineService.h"
#import "TMFWebOllinePresetListViewController.h"
#import "TMFWebOfflinePresetLogViewController.h"
#import "SSZipArchive.h"

@interface TMFWebOllinePresetViewController () {
    dispatch_queue_t _queue;
}
@property (nonatomic, strong) TMFWebOfflinePresetLogViewController *logVC;
@property (nonatomic, strong) NSMutableString *logMessage;
@property (nonatomic, strong) NSLock *lock;
@property (nonatomic, assign) BOOL isBusy;
@property (nonatomic, strong) NSArray *allPackages;

@end

@implementation TMFWebOllinePresetViewController


- (instancetype)init{
    self = [super init];
    if (self) {
        _queue = dispatch_queue_create("demo.weboffline.download", DISPATCH_QUEUE_SERIAL);
        self.lock = [[NSLock alloc] init];
        _isBusy = NO;
    }
    return self;
}

- (void)dealloc {
    for (TMFWebOfflinePackageInfo *info in self.allPackages) {
        [TMFWebOfflineService cancelDownloadPackageWithInfo:info];
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    TMFWebOfflinePresetLogViewController *subVC = [[TMFWebOfflinePresetLogViewController alloc] init];
    [self addChildViewController:subVC];
    self.logVC = subVC;
    __weak typeof(self) weakSelf = self;
    subVC.closeButtonHandle = ^{
        [weakSelf.logVC.view removeFromSuperview];
        weakSelf.tableView.scrollEnabled = YES;
    };
    
    self.logMessage = [[NSMutableString alloc] initWithString:@"预置离线包日志"];
}

- (void)initDataSource {
    [super initDataSource];
    self.dataSource = @[@"拉取离线包信息",
                        ];
}

- (void)didSelectCellWithTitle:(NSString *)title {
    [self.lock lock];
    if (self.isBusy) {
        [self addLogMsg:@"离线包制作中，请勿重复操作！！"];
        [self.lock unlock];
        return;
    }
    [self.lock unlock];

    if ([title isEqualToString:@"拉取离线包信息"]) {
        TMFWebOfflineServiceOptions *options = [TMFWebOfflineServiceOptions options];
        options.ignoresDelay = YES;
        options.ignoresFrequency = YES;
        [self clearAllLocalPackage];
        [self addLogMsg:@"清理本地离线包缓存..."];
        __weak typeof(self) weakSelf = self;
        [TMFWebOfflineService checkAllAvailablePackageInfosWithOptions:options completionHandler:^(
            NSArray<TMFWebOfflinePackageInfo *> *_Nonnull packageInfos, NSError *_Nullable error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                for (TMFWebOfflinePackageInfo *info in packageInfos) {
                    [weakSelf addLogMsg:[NSString stringWithFormat:@"拉取到离线包信息，name: %@, version: %ld", info.BID, info.version]];
                }
                TMFWebOllinePresetListViewController *vc = [[TMFWebOllinePresetListViewController alloc] init];
                vc.allSource = packageInfos;
                vc.doButtonHandle = ^(NSArray<TMFWebOfflinePackageInfo *> *_Nonnull packages) {
                    [weakSelf makepackages:packages];
                };
                [weakSelf.navigationController pushViewController:vc animated:YES];
            });
        }];
    }
}

#pragma mark - mark Package
- (void)clearAllLocalPackage {
    [TMFWebOfflineService removeAllLocalPackage];

    NSString *documentPath;
#if DEBUG
    documentPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
#else
    documentPath = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) objectAtIndex:0];
#endif
    documentPath = [documentPath stringByAppendingPathComponent:@"TMF/TMFWebOffline"];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if (![fileManager fileExistsAtPath:documentPath]) {
        return;
    }
    NSArray *items = [fileManager subpathsOfDirectoryAtPath:documentPath error:nil];
    for (NSString *directories in items) {
        NSString *directoryPath = [documentPath stringByAppendingPathComponent:directories];
        NSArray *items = [fileManager subpathsOfDirectoryAtPath:directoryPath error:nil];
        for (NSString *subPath in items) {
            [fileManager removeItemAtPath:[directoryPath stringByAppendingPathComponent:subPath] error:nil];
        }
    }
}

- (void)makepackages:(NSArray<TMFWebOfflinePackageInfo *> *)packages {
    if (!packages || 0 == packages.count) {
        [self addLogMsg:@"预置数量0！"];
        return;
    }
    [self addLogMsg:@"开始制作预置离线包"];
    self.tableView.scrollEnabled = NO;
    [self.lock lock];
    self.isBusy = YES;
    [self.lock unlock];

    self.allPackages = packages;
    NSMutableArray *bids = [[NSMutableArray alloc] init];

    /// 1. 下载离线包文件
    TMFWebOfflineServiceOptions *options = [TMFWebOfflineServiceOptions options];
    options.ignoresDelay = YES;
    options.ignoresFrequency = YES;
    options.cachePolicy = TMFWebOfflineServiceOptionsCachePolicyRemoveAndDownload;

    dispatch_semaphore_t s = dispatch_semaphore_create(0);
    [self addLogMsg:@">>>>"];
    dispatch_async(_queue, ^{
        for (TMFWebOfflinePackageInfo *package in packages) {
            [self addLogMsg:@"    ----"];
            [self addLogMsg:[NSString stringWithFormat:@"    开始下载离线包，name: %@, version: %ld", package.BID, package.version]];
            [TMFWebOfflineService downloadPackageWithInfo:package options:options completionHandler:^(NSError *_Nullable error) {
                if (error) {
                    [self addLogMsg:[NSString stringWithFormat:@"    离线包下载失败，name: %@, version: %ld, error:%@", package.BID, package.version,
                                              error]];
                } else {
                    if (![bids containsObject:package.BID]) {
                        [bids addObject:package.BID];
                    }
                    [self addLogMsg:[NSString stringWithFormat:@"    离线包下载成功，name: %@, version: %ld", package.BID, package.version]];
                }
                dispatch_semaphore_signal(s);
            }];
            dispatch_semaphore_wait(s, DISPATCH_TIME_FOREVER);
        }

        /// 2.解压离线包文件
        [TMFWebOfflineService uncompressPackagesIfNeeded];

        /// 3.创建临时目录进行离线包合并
        [self createWebappCachein:bids];

        self.allPackages = nil;
        [self.lock lock];
        self.isBusy = NO;
        [self.lock unlock];
    });
}

#define ERROR_CONTINUE do {\
    if (error) {\
        [self addLogMsg:[NSString stringWithFormat:@"   文件操作失败: %@", error]];\
        continue;\
    }\
} while (0);


- (void)createWebappCachein:(NSArray *)bids{
    NSError *error;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *presetPath = [NSHomeDirectory() stringByAppendingPathComponent:@"webappCachein"];
    if ([fileManager fileExistsAtPath:presetPath]) {
        [fileManager removeItemAtPath:presetPath error:nil];
    }
    [fileManager createDirectoryAtPath:presetPath withIntermediateDirectories:YES attributes:nil error:&error];
    if (error) {
        [self addLogMsg:[NSString stringWithFormat:@"文件操作失败: %@", error]];
        return;
    } else {
        [self addLogMsg:[NSString stringWithFormat:@"离线包临时目录创建成功: %@", presetPath]];
    }
    [self addLogMsg:@">>>>"];
    for (NSString *bid in bids) {
        [self addLogMsg:@"    ----"];
        NSString *toBidPath = [presetPath stringByAppendingPathComponent:bid];
        NSString *fromBidPath = [TMFWebOfflineService localPathForBID:bid];
        [fileManager createDirectoryAtPath:toBidPath withIntermediateDirectories:YES attributes:nil error:&error];
        ERROR_CONTINUE
        [self addLogMsg:[NSString stringWithFormat:@"    离线包 BID 目录 %@ 创建成功", bid]];

        /// copy config.json
        [fileManager copyItemAtPath:[fromBidPath stringByAppendingPathComponent:@"config.json"]
                             toPath:[toBidPath stringByAppendingPathComponent:@"config.json"]
                              error:&error];
        ERROR_CONTINUE
        [self addLogMsg:[NSString stringWithFormat:@"    拷贝 %@/config.json 文件成功", bid]];

        /// copy zip
        NSString *fromZipPath = [[fromBidPath stringByDeletingLastPathComponent] stringByDeletingLastPathComponent];
        fromZipPath = [fromZipPath stringByAppendingPathComponent:@"webappCacheZip"];
        NSString *zipName = [NSString stringWithFormat:@"%@.zip", bid];
        [fileManager copyItemAtPath:[fromZipPath stringByAppendingPathComponent:zipName]
                             toPath:[toBidPath stringByAppendingPathComponent:zipName]
                              error:&error];
        ERROR_CONTINUE
        [self addLogMsg:[NSString stringWithFormat:@"    拷贝离线包 %@.zip 文件成功", bid]];
    }
    [self addLogMsg:@""];
    /// 压缩文件
    NSString *zipPath = [NSHomeDirectory() stringByAppendingPathComponent:@"webappCachein.zip"];
    if ([SSZipArchive createZipFileAtPath:zipPath withContentsOfDirectory:presetPath keepParentDirectory:YES]) {
        [self addLogMsg:[NSString stringWithFormat:@"压缩 webappCachein.zip 文件成功"]];
    } else {
        [self addLogMsg:[NSString stringWithFormat:@"压缩 webappCachein.zip 文件失败"]];
    }

    /// 创建桌面文件
    NSRange loc = [presetPath rangeOfString:@"Library"];
    if (loc.location == NSNotFound) {
        [self addLogMsg:@"未找到桌面目录"];
        return;
    }
    NSString *desktopPath = [[presetPath substringToIndex:loc.location] stringByAppendingPathComponent:@"Desktop"];
    NSString *lnPath = [desktopPath stringByAppendingPathComponent:@"WOPresetFile"];
    if ([fileManager fileExistsAtPath:lnPath]) {
        [fileManager removeItemAtPath:lnPath error:&error];
        if (error) {
            [self addLogMsg:[NSString stringWithFormat:@"桌面文件 WOPresetFile 删除失败: %@", error]];
            return;
        }
    }
    [fileManager createDirectoryAtPath:lnPath withIntermediateDirectories:YES attributes:nil error:&error];
    [fileManager copyItemAtPath:zipPath toPath:[lnPath stringByAppendingPathComponent:@"webappCachein.zip"] error:&error];
    if (error) {
        [self addLogMsg:[NSString stringWithFormat:@"拷贝 webappCachein.zip 失败: %@", error]];
    } else {
        [self addLogMsg:[NSString stringWithFormat:@"webappCachein.zip 已拷贝至桌面,目标文件夹 WOPresetFile"]];
    }
}

#pragma mark - Log Info
- (void)addLogMsg:(NSString *)message {
    dispatch_block_t doSomething = ^(){
        [self.logMessage appendString:@"\n"];
        [self.logMessage appendString:message];
        self.logVC.logTextView.text = self.logMessage;
        NSLog(@"[TMFWebOfflinePresetPackage] -> : %@", message);
        
        if (![self.view.subviews containsObject:self.logVC.view]) {
            [self.view addSubview:self.logVC.view];
            self.logVC.view.frame = CGRectMake(0, self.view.bounds.size.height / 2, self.view.bounds.size.width, self.view.bounds.size.height / 2);
        }
    };
    
    if ([NSThread isMainThread]) {
        doSomething();
    } else {
        dispatch_sync(dispatch_get_main_queue(), ^{
            doSomething();
        });
    }
}

@end
