//
//  TMFWebTestUIViewController.m
//  TMFDemo
//
//  Created by klaudz on 9/5/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "TMFWebTestUIViewController.h"
#import "TMFDemoConfigurations.h"
#import "NSString+Localize.h"

NS_ASSUME_NONNULL_BEGIN

@interface TMFWebTestUIViewController ()

@property (nonatomic, strong, nullable) NSURL *URL;
@property (nonatomic, strong) WKWebView *webView;

@end

@implementation TMFWebTestUIViewController

@synthesize tmf_JSBridge = _tmf_JSBridge;
@synthesize tmf_webOfflineHandler = _tmf_webOfflineHandler;

- (instancetype)init {
    self = [super init];
    if (self) {
        self.tmf_JSBridge = [[TMFJSBridge alloc] initWithWebViewController:self];
        self.tmf_webOfflineHandler = [[TMFWebOfflineHandler alloc] initWithWebViewController:self];

        self.title = @"TMFWebTest";
    }
    return self;
}

- (instancetype)initWithURL:(NSURL *)URL {
    self = [self init];
    if (self) {
        self.URL = URL;
    }
    return self;
}

- (void)dealloc {
    self.webView.UIDelegate = nil;

    [self.tmf_JSBridge clearWebViewControllerAndCancel];
    [self.tmf_webOfflineHandler clearWebViewController];
}

- (void)viewDidLoad {
    [super viewDidLoad];

    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:[@"more" localizedString] style:UIBarButtonItemStylePlain target:nil action:NULL];
    self.navigationItem.rightBarButtonItem = rightBarButtonItem;

    self.navigationController.toolbarHidden = NO;
    [self setToolbarItems:@[
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRewind target:self action:@selector(backButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFastForward target:self action:@selector(forwardButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(refreshButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCompose target:self action:@selector(goButtonClicked:)],
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
    ]];

    WKWebView *webView = [[WKWebView alloc] initWithFrame:self.view.frame];
    webView.UIDelegate = self;
    webView.navigationDelegate = self;
    self.webView = webView;
    [self.view addSubview:self.webView];

    if (self.URL) {
        NSURLRequest *request = [NSURLRequest requestWithURL:self.URL];
        [self.webView loadRequest:request];
    } else {
        [self showGoInputAlert];
    }
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];

    [self.tmf_JSBridge notifyWithEvent:@"containerAppear" parameters:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];

    [self.tmf_JSBridge notifyWithEvent:@"containerDisappear" parameters:nil];
}

/*
 #pragma mark - Navigation

 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

#pragma mark - WKNavigationDelegate

- (void)webView:(WKWebView *)webView
    decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction
                    decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    // Handle the WebOfflineHandler
    [self.tmf_webOfflineHandler handleRequest:navigationAction.request];

    decisionHandler(WKNavigationActionPolicyAllow);
}

- (void)webView:(WKWebView *)webView didFinishNavigation:(null_unspecified WKNavigation *)navigation {
    // Inject TMFJSBridge into webView
    [self.tmf_JSBridge injectJSBridgeIntoWebView:nil];
}

#pragma mark - Button

- (void)backButtonClicked:(id)button {
    if ([self.webView canGoBack]) {
        [self.webView goBack];
    }
}

- (void)forwardButtonClicked:(id)button {
    if ([self.webView canGoForward]) {
        [self.webView goForward];
    }
}

- (void)refreshButtonClicked:(id)button {
    [self.webView reload];
}

- (void)goButtonClicked:(id)button {
    [self showGoInputAlert];
}

- (void)showGoInputAlert {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Go to URL" message:nil
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
        textField.text = self.webView.URL.absoluteString
                             ?: [NSString stringWithFormat:WEB_OFFLINE_TEST_URL_FORMAT,
                                          [NSString stringWithFormat:@"%@+%@", WEB_OFFLINE_TEST_BID, WEB_OFFLINE_TEST_COMM_BID]];
        textField.placeholder = @"URL";
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Go" style:UIAlertActionStyleDefault handler:^(UIAlertAction *_Nonnull action) {
        NSURL *URL = [NSURL URLWithString:alertController.textFields.firstObject.text];
        if (URL) {
            NSURLRequest *request = [NSURLRequest requestWithURL:URL];
            [self.webView loadRequest:request];
        }
    }]];
    [self presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - TMFJSBridgeWebViewControllerProtocol / TMFWebOfflineWebViewControllerProtocol

- (UIView *)tmf_webView {
    return self.webView;
}

@end

NS_ASSUME_NONNULL_END
