//
//  ViewController.m
//  TMFDemo
//
//  Created by klaudz on 5/4/2019.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "ViewController.h"
#import "NSString+Localize.h"
// TMFShark
#import "TMFSharkViewController.h"

// QMUIKit
#import <QMUIKit/QMUIKit.h>
#import "QDCommonUI.h"
#import "QDUIHelper.h"
#import "QDThemeManager.h"
#import "QDTabBarViewController.h"
#import "QDNavigationController.h"
#import "QDUIKitViewController.h"
#import "QDComponentsViewController.h"
#import "QDLabViewController.h"

// TMFJSBridge
#import "TMFJSBridgeViewController.h"
// TMFWebAnalytics
#import "TMFWebAnalyticsViewController.h"
// TMFWebOffline
#import "TMFWebOfflineViewController.h"
#import "TMFWebOllinePresetViewController.h"
// TMFWebTest
#import "TMFWebTestUIViewController.h"
// TMFHybrid
#import "TMFHybridViewController.h"
// TMFPush
#import "TMFPush.h"
// QAPM
#import "APMMainListViewController.h"
// QRCode
#import "TMFCodeScannerListViewController.h"
// TMFShare
#import "TMFShareViewController.h"
// Map
#import "TMFLocationViewController.h"
// OCR
#import "TMFOCRMainViewController.h"
// SafeKeyboard
#import "TMFSafeKeyboardSelectViewController.h"
// WCDB
#import "TMFWCDBViewController.h"
// LiveDetector
#import "TMFReflectCameraViewController.h"
#import "TMFPoseCameraViewController.h"

// TMFSSLTester
#import "TMFSSLViewController.h"

// TMFChristManager
#import "TMFChristViewController.h"
// TMFUpload
#import "TMFUploadViewController.h"
// TMFDistribution
#import "TMFDistributionViewController.h"
// TMFProfile
#import "TMFProfileViewController.h"
// DevelopSetting
#import "TMFDevelopSettingViewController.h"
// ICDP
#import "TMFICDPViewController.h"
// Analyse
#import "TMFAnalyseViewController.h"
// TMFRouter
#import "TMFRouterViewController.h"
// TMFStorage
#import "TMFStorageViewController.h"


@interface ViewController ()

@property (nonatomic, assign) NSInteger mode;

@end

@implementation ViewController

- (void)didInitialize
{
    [super didInitialize];
    
    self.mode = 0;
}

- (void)viewDidLoad {
    [super viewDidLoad];

    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:nil style:UIBarButtonItemStylePlain target:self
                                                                          action:@selector(rightBarButtonItemClicked:)];
    self.navigationItem.rightBarButtonItem = rightBarButtonItem;
    [self updateRightBarButtonItem];

    // 解决低版本导航条覆盖问题
    if ([[[UIDevice currentDevice] systemVersion] floatValue] < 10.0) {
        if (@available(iOS 7.0, *)) {
            self.edgesForExtendedLayout = UIRectEdgeNone;
        }
    }
#if DEBUG
    if (@available(iOS 13.0, *)) {
        [self fixNavigationBarAlpha];
    }
#endif
}

#pragma mark NavigationbarAlphaFix
- (void)fixNavigationBarAlpha{
    UINavigationBar *bar = self.navigationController.navigationBar;
    UIImageView *imageView = [self printSubViews:bar floor:0];
    if (imageView) {
        [imageView addObserver:self forKeyPath:@"alpha" options:NSKeyValueObservingOptionNew context:nil];
    }
}

static const double imgView_alpha = 0.90980398654937744;
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary<NSKeyValueChangeKey,id> *)change context:(void *)context{
    if ([keyPath isEqualToString:@"alpha"] && [object isKindOfClass:[UIImageView class]]) {
        UIView *view = object;
        if ([[change objectForKey:@"new"] doubleValue] != imgView_alpha) {
            dispatch_async(dispatch_get_main_queue(), ^{
                view.alpha = imgView_alpha;
            });
        }
        return;
    }
    [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
}

- (UIImageView *)printSubViews:(UIView *)view floor:(int)floor{
    if ([view isKindOfClass:[UIImageView class]]) {
        return (UIImageView *)view;
    }
    int i = 0;
    for (; i < floor; i++) {
        printf(" ");
    }
    NSLog(@"%@\n", view);
    for (UIView *subView in view.subviews) {
        UIImageView *resView = [self printSubViews:subView floor:floor+1];
        if (resView) {
            return resView;
        }
    }
    return nil;
}

- (void)awakeFromNib {
    [super awakeFromNib];
    [self initDataSource];
}

// 完整

#define ROW_TMFSHARK                @"TMFShark"
#define ROW_TMFSHARK_DETAIL         [@"mobile gateway" localizedString]

#define ROW_TMFJSBRIDGE             @"TMFJSBridge"
#define ROW_TMFJSBRIDGE_DETAIL      [@"H5 container" localizedString]

#define ROW_TMFWEBOFFLINE           @"TMFWebOffline"
#define ROW_TMFWEBOFFLINE_DETAIL    [@"web offline" localizedString]

#define ROW_TMFHYBRID               @"TMFHybrid"
#define ROW_TMFHYBRID_DETAIL        [@"hybrid develop" localizedString]

#define ROW_VASSONIC                @"VasSonic"
#define ROW_VASSONIC_DETAIL         @"VasSonic"

#define ROW_TMFPUSH                 @"TMFPush"
#define ROW_TMFPUSH_DETAIL          [@"remote push" localizedString]

#define ROW_QMUI                    @"QMUI"
#define ROW_QMUI_DETAIL             [@"UI module" localizedString]

#define ROW_WCDB                    @"WCDB"
#define ROW_WCDB_DETAIL             [@"storage module" localizedString]

#define ROW_TMFSSL                  @"TMFSSL"
#define ROW_TMFSSL_DETAIL           [@"encryption module" localizedString]

#define ROW_QAPM                    @"QAPM"
#define ROW_QAPM_DETAIL             [@"performance analysis" localizedString]

#define ROW_TMFSHARE                @"TMFShare"
#define ROW_TMFSHARE_DETAIL         [@"share module" localizedString]

#define ROW_TMFLOCATION             @"TMFLocation"
#define ROW_TMFLOCATION_DETAIL      [@"locaiton module" localizedString]

#define ROW_TMFQRCODE               @"TMFQRCode"
#define ROW_TMFQRCODE_DETAIL        [@"scan module" localizedString]

#define ROW_TMFOCR                  @"TMFOCR"
#define ROW_TMFOCR_DETAIL           [@"OCR recognize" localizedString]

#define ROW_TMFSAFEKEYBOARD         @"TMFSafeKeyboard"
#define ROW_TMFSAFEKEYBOARD_DETAIL  [@"safety keyboard" localizedString]

#define ROW_TMFLIVEREFLECT          @"LiveReflect"
#define ROW_TMFLIVEREFLECT_DETAIL   [@"live reflect recognize" localizedString]

#define ROW_TMFLIVEPOSE             @"LivePose"
#define ROW_TMFLIVEPOSE_DETAIL      [@"live pose recognize" localizedString]

#define ROW_TMFWEBANALYTICS         @"TMFWebAnalytics"
#define ROW_TMFWEBANALYTICS_DETAIL  [@"data acquisition and analysis" localizedString]

#define ROW_TMFCHRIST               @"TMFChristManager"
#define ROW_TMFCHRIST_DETAIL        [@"JS hotfix" localizedString]

#define ROW_TMFUPLOAD               @"TMFUploader"
#define ROW_TMFUPLOAD_DETAIL        [@"shard upload" localizedString]

#define ROW_TMFDISTRIBUTION         @"TMFDistribution"
#define ROW_TMFDISTRIBUTION_DETAIL  [@"distribute management" localizedString]

#define ROW_TMFPROFILE              @"TMFProfile+TMFInstruction"
#define ROW_TMFPROFILE_DETAIL       [@"data sync" localizedString]

#define ROW_TMFStorage              @"TMFStorage"
#define ROW_TMFStorage_DETAIL       [@"storage center" localizedString]

#define ROW_TMFICDP                 @"TMFICDP"
#define ROW_TMFICDP_DETAIL          [@"smart delivery" localizedString]

#define ROW_TMFANALYSE              @"TMFAnalyse"
#define ROW_TMFANALYSE_DETAIL       [@"data acquisition and analysis" localizedString]

#define ROW_TMFROUTER               @"TMFRouter"
#define ROW_TMFROUTER_DETAIL        [@"framework components" localizedString]

// 精简

#define ROW_TMFWEBTEST              @"TMFJSBridge + TMFWebOffline"
#define ROW_TMFWEBTEST_DETAIL       [@"H5 container entrance" localizedString]

#define ROW_TMFWEBOFFLINE2          @"TMFWebOffline Settings"
#define ROW_TMFWEBOFFLINE2_DETAIL   [@"web offline setting" localizedString]

#define ROW_TMFJSBRIDGE_ISSUE       @"TMFJSBridge"
#define ROW_TMFJSBRIDGE_ISSUE_DETAIL [@"H5 container question & answer" localizedString]

#define ROW_DEVELOPSETTING_ISSUE        @"Develop Setting"
#define ROW_DEVELOPSETTING_ISSUE_DETAIL [@"develop Setting" localizedString]


- (void)initDataSource {
    [super initDataSource];
    
    QMUIOrderedDictionary<NSString *, NSString *> *dataSourceWithDetailText = [[QMUIOrderedDictionary alloc] init];
    
    if (self.mode == 0) {
        [dataSourceWithDetailText setObject:ROW_TMFSHARK forKey:ROW_TMFSHARK_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFJSBRIDGE forKey:ROW_TMFJSBRIDGE_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFWEBOFFLINE forKey:ROW_TMFWEBOFFLINE_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFHYBRID forKey:ROW_TMFHYBRID_DETAIL];
        [dataSourceWithDetailText setObject:ROW_VASSONIC forKey:ROW_VASSONIC_DETAIL];
        [dataSourceWithDetailText setObject:ROW_QMUI forKey:ROW_QMUI_DETAIL];
        [dataSourceWithDetailText setObject:ROW_WCDB forKey:ROW_WCDB_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFSSL forKey:ROW_TMFSSL_DETAIL];
        [dataSourceWithDetailText setObject:ROW_QAPM forKey:ROW_QAPM_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFSHARE forKey:ROW_TMFSHARE_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFLOCATION forKey:ROW_TMFLOCATION_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFPUSH forKey:ROW_TMFPUSH_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFPROFILE forKey:ROW_TMFPROFILE_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFQRCODE forKey:ROW_TMFQRCODE_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFOCR forKey:ROW_TMFOCR_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFSAFEKEYBOARD forKey:ROW_TMFSAFEKEYBOARD_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFLIVEREFLECT forKey:ROW_TMFLIVEREFLECT_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFLIVEPOSE forKey:ROW_TMFLIVEPOSE_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFWEBANALYTICS forKey:ROW_TMFWEBANALYTICS_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFCHRIST forKey:ROW_TMFCHRIST_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFUPLOAD forKey:ROW_TMFUPLOAD_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFDISTRIBUTION forKey:ROW_TMFDISTRIBUTION_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFICDP forKey:ROW_TMFICDP_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFROUTER forKey:ROW_TMFROUTER_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFStorage forKey:ROW_TMFStorage_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFANALYSE forKey:ROW_TMFANALYSE_DETAIL];
    } else {
        [dataSourceWithDetailText setObject:ROW_TMFWEBTEST forKey:ROW_TMFWEBTEST_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFWEBOFFLINE2 forKey:ROW_TMFWEBOFFLINE2_DETAIL];
        [dataSourceWithDetailText setObject:ROW_TMFJSBRIDGE_ISSUE forKey:ROW_TMFJSBRIDGE_ISSUE_DETAIL];
//        [dataSourceWithDetailText setObject:ROW_DEVELOPSETTING_ISSUE forKey:ROW_DEVELOPSETTING_ISSUE_DETAIL];
    }
    
    self.dataSourceWithDetailText = dataSourceWithDetailText;
}

- (void)didSelectCellWithTitle:(NSString *)title {
    
    if ([title isEqualToString:ROW_TMFSHARK_DETAIL]) {
        [self createTMFSharkViewController];
    } else if ([title isEqualToString:ROW_TMFJSBRIDGE_DETAIL]) {
        [self createJSBridgeViewController];
    } else if ([title isEqualToString:ROW_TMFWEBOFFLINE_DETAIL]) {
        [self createTWOfflineController];
    } else if ([title isEqualToString:ROW_TMFHYBRID_DETAIL]){
        [self createTMFHybridViewController];
    } else if ([title isEqualToString:ROW_TMFPUSH_DETAIL]) {
        [self createTMFPushController];
    } else if ([title isEqualToString:ROW_QMUI_DETAIL]) {
        [self createQMUITabBarController];
    } else if ([title isEqualToString:ROW_WCDB_DETAIL]) {
        [self createTMFWCDBController];
    } else if ([title isEqualToString:ROW_TMFSSL_DETAIL]) {
        [self createTMFSSLController];
    } else if ([title isEqualToString:ROW_QAPM_DETAIL]) {
        [self createQAPMController];
    } else if ([title isEqualToString:ROW_TMFSHARE_DETAIL]) {
        [self createTMFShareViewController];
    } else if ([title isEqualToString:ROW_TMFLOCATION_DETAIL]) {
        [self createMapController];
    } else if ([title isEqualToString:ROW_TMFQRCODE_DETAIL]) {
        [self createQRCodeController];
    } else if ([title isEqualToString:ROW_TMFOCR_DETAIL]) {
        [self createTMFOCRController];
    } else if ([title isEqualToString:ROW_TMFSAFEKEYBOARD_DETAIL]) {
        [self createTMFSakeKeyboardController];
    }else if ([title isEqualToString:ROW_TMFLIVEREFLECT_DETAIL]) {
        [self createYTReflectController];
    } else if ([title isEqualToString:ROW_TMFLIVEPOSE_DETAIL]) {
        [self createYTPoseController];
    } else if ([title isEqualToString:ROW_VASSONIC_DETAIL]) {
        [self createVasSonicController];
    } else if ([title isEqualToString:ROW_TMFWEBANALYTICS_DETAIL]) {
        [self createTMFWebAnalyticsController];
    } else if ([title isEqualToString:ROW_TMFCHRIST_DETAIL]) {
        [self createPatchViewController];
    } else if ([title isEqualToString:ROW_TMFUPLOAD_DETAIL]) {
        [self createUploadViewController];
    } else if ([title isEqualToString:ROW_TMFDISTRIBUTION_DETAIL]) {
        [self createDistributionController];
    } else if ([title isEqualToString:ROW_TMFPROFILE_DETAIL]) {
        [self createProfileController];
    } else if ([title isEqualToString:ROW_TMFStorage_DETAIL]) {
        [self createStorageController];
    } else if ([title isEqualToString:ROW_TMFANALYSE_DETAIL]) {
        [self createAnalyseController];
    }
    
    else if ([title isEqualToString:ROW_TMFWEBTEST_DETAIL]) {
        [self createWebTestViewController];
    } else if ([title isEqualToString:ROW_TMFWEBOFFLINE2_DETAIL]) {
        [self createTWOfflineController];
    } else if ([title isEqualToString:ROW_TMFJSBRIDGE_ISSUE_DETAIL]) {
        [self createJSBridgeViewController];
    } else if ([title isEqualToString:ROW_DEVELOPSETTING_ISSUE_DETAIL]) {
        [self createDevelopSettingViewController];
    }else if ([title isEqualToString:ROW_TMFICDP_DETAIL]) {
        [self createICDPController];
    }else if ([title isEqualToString:ROW_TMFROUTER_DETAIL]) {
        [self createRouterController];
    }
}

- (void)setupNavigationItems {
    [super setupNavigationItems];
    self.title = @"TMF-Demo";
}

#pragma mark - TMFShark

- (void)createTMFSharkViewController {
    TMFSharkViewController *viewController = [[TMFSharkViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - QMUIKit

- (void)createQMUITabBarController {
    QDTabBarViewController *tabBarViewController = [[QDTabBarViewController alloc] init];

    // QMUIKit
    QDUIKitViewController *uikitViewController = [[QDUIKitViewController alloc] init];
    uikitViewController.hidesBottomBarWhenPushed = NO;
    //    QDNavigationController *uikitNavController = [[QDNavigationController alloc] initWithRootViewController:uikitViewController];
    uikitViewController.tabBarItem =
        [QDUIHelper tabBarItemWithTitle:@"QMUIKit" image:[UIImageMake(@"icon_tabbar_uikit") imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]
                          selectedImage:UIImageMake(@"icon_tabbar_uikit_selected")
                                    tag:0];
    AddAccessibilityHint(uikitViewController.tabBarItem, @"展示一系列对系统原生控件的拓展的能力");

    // UIComponents
    QDComponentsViewController *componentViewController = [[QDComponentsViewController alloc] init];
    componentViewController.hidesBottomBarWhenPushed = NO;
    //    QDNavigationController *componentNavController = [[QDNavigationController alloc] initWithRootViewController:componentViewController];
    componentViewController.tabBarItem =
        [QDUIHelper tabBarItemWithTitle:@"Components"
                                  image:[UIImageMake(@"icon_tabbar_component") imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]
                          selectedImage:UIImageMake(@"icon_tabbar_component_selected")
                                    tag:1];
    AddAccessibilityHint(componentViewController.tabBarItem, @"展示 QMUI 自己的组件库");

    // Lab
    QDLabViewController *labViewController = [[QDLabViewController alloc] init];
    labViewController.hidesBottomBarWhenPushed = NO;
    //    QDNavigationController *labNavController = [[QDNavigationController alloc] initWithRootViewController:labViewController];
    labViewController.tabBarItem =
        [QDUIHelper tabBarItemWithTitle:@"Lab" image:[UIImageMake(@"icon_tabbar_lab") imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]
                          selectedImage:UIImageMake(@"icon_tabbar_lab_selected")
                                    tag:2];
    AddAccessibilityHint(labViewController.tabBarItem, @"集合一些非正式但可能很有用的小功能");

    tabBarViewController.title = @"QMUI_demo";
    tabBarViewController.viewControllers = @[uikitViewController, componentViewController, labViewController];
    [self.navigationController pushViewController:tabBarViewController animated:YES];
}

#pragma mark - TMFJSBridge

- (void)createJSBridgeViewController
{
    TMFJSBridgeViewController *viewController = [[TMFJSBridgeViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFWebAnalytics

- (void)createTMFWebAnalyticsController {
    TMFWebAnalyticsViewController *viewController =  [[TMFWebAnalyticsViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFWebOffline

- (void)createTWOfflineController {
    TMFWebOfflineViewController *viewController =  [[TMFWebOfflineViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

- (void)createTWOfflinePresetController {
    TMFWebOllinePresetViewController *viewController =  [[TMFWebOllinePresetViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFHybrid
- (void)createTMFHybridViewController{
    TMFHybridViewController * hybridViewController = [TMFHybridViewController new];
    [self.navigationController pushViewController:hybridViewController animated:YES];
}

#pragma mark - TMFSSL

- (void)createTMFSSLController {
    TMFSSLViewController *viewController =  [[TMFSSLViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFPush

- (void)createTMFPushController {
    NSString *deviceToken = [TMFPushTokenManager defaultTokenManager].deviceTokenString;
    NSMutableString *message = [NSMutableString string];
    if (deviceToken) {
        [[UIPasteboard generalPasteboard] setString:deviceToken];
        [message appendFormat:@"deviceToken: %@\n", deviceToken];
        [message appendString:[@"copied to clipboard" localizedString]];
    } else {
        [message appendFormat:@"deviceToken: %@", deviceToken];
    }
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"TMFPush" message:message
                                                                      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}
#pragma mark - QAPM

- (void)createQAPMController {
    APMMainListViewController *viewController =  [[APMMainListViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - QRCode

- (void)createQRCodeController {
    TMFCodeScannerListViewController *viewController = [[TMFCodeScannerListViewController alloc] init];
    viewController.view.frame = self.view.frame;
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFShare

- (void)createTMFShareViewController {
    TMFShareViewController *viewController = [[TMFShareViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}


#pragma mark - Map

- (void)createMapController {
    TMFLocationViewController *viewController = [[TMFLocationViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - OCR

- (void)createTMFOCRController {
    TMFOCRMainViewController *viewController = [[TMFOCRMainViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - SafeKeyboard

- (void)createTMFSakeKeyboardController {
    TMFSafeKeyboardSelectViewController *viewController = [[TMFSafeKeyboardSelectViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - WCDB

- (void)createTMFWCDBController {
    TMFWCDBViewController *viewController = [[TMFWCDBViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFWebTest

- (void)createWebTestViewController
{
    TMFWebTestUIViewController *viewController = [[TMFWebTestUIViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark -

- (void)rightBarButtonItemClicked:(UIBarButtonItem *)item
{
    self.mode = (self.mode == 0 ? 1 : 0);
    [self initDataSource];
    [self.tableView reloadData];
    
    [self updateRightBarButtonItem];
}

- (void)updateRightBarButtonItem
{
    self.navigationItem.rightBarButtonItem.title = self.mode == 0 ? [@"reduced" localizedString] : [@"full" localizedString];
}

#pragma mark - YTReflect

- (void)createYTReflectController {
    TMFReflectCameraViewController *viewController = [[TMFReflectCameraViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - YTPose

- (void)createYTPoseController {
    TMFPoseCameraViewController *viewController = [[TMFPoseCameraViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - VasSonic

- (void)createVasSonicController {
//    SonicRootViewController *viewController = [[SonicRootViewController alloc] init];
//    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - PatchViewController

- (void)createPatchViewController {
    TMFChristViewController *viewController = [[TMFChristViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFUploadViewController

- (void)createUploadViewController {
    TMFUploadViewController *viewController = [[TMFUploadViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFDistributionController

- (void)createDistributionController {
    TMFDistributionViewController *VC = [[TMFDistributionViewController alloc] init];
    [self.navigationController pushViewController:VC animated:YES];
}

#pragma mark - TMFProfileController

- (void)createProfileController
{
    TMFProfileViewController *viewController = [[TMFProfileViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFStorageController

- (void)createStorageController
{
    TMFStorageViewController *viewController = [[TMFStorageViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - TMFAnalyseController
- (void)createAnalyseController{
    TMFAnalyseViewController *viewController = [[TMFAnalyseViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - DevelopSettingViewController
- (void)createDevelopSettingViewController {
    TMFDevelopSettingViewController *viewController = [[TMFDevelopSettingViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

#pragma mark - ICDPController
- (void)createICDPController {
    UIStoryboard *iCDPSB = [UIStoryboard storyboardWithName:@"ICDP" bundle:nil];
    [self.navigationController pushViewController:[iCDPSB instantiateInitialViewController] animated:YES];
}

#pragma mark - TMFURLRouterViewController
- (void)createRouterController {
    TMFRouterViewController *viewController = [[TMFRouterViewController alloc] init];
    [self.navigationController pushViewController:viewController animated:YES];
}

@end
